Package: SpatialDecon
Title: Deconvolution of mixed cells from spatial and/or bulk gene
        expression data
Version: 1.22.0
Authors@R: c(person("Maddy", "Griswold", email = "mgriswold@nanostring.com", role = c("cre", "aut")),
             person("Patrick", "Danaher", email = "pdanaher@nanostring.com", role = c("aut")))
Description: Using spatial or bulk gene expression data, estimates abundance of mixed cell types
 within each observation. Based on "Advances in mixed cell deconvolution enable quantification of 
 cell types in spatial transcriptomic data", Danaher (2022). Designed for use with
 the NanoString GeoMx platform, but applicable to any gene expression data. 
Depends: R (>= 4.0.0)
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: TRUE
RoxygenNote: 7.2.3
Imports: grDevices, stats, utils, graphics, SeuratObject, Biobase,
        GeomxTools, repmis, methods, Matrix, logNormReg (>= 0.4)
Suggests: testthat, knitr, rmarkdown, qpdf, Seurat
biocViews: ImmunoOncology, FeatureExtraction, GeneExpression,
        Transcriptomics, Spatial
VignetteBuilder: knitr
BugReports: https://github.com/Nanostring-Biostats/SpatialDecon/issues
git_url: https://git.bioconductor.org/packages/SpatialDecon
git_branch: RELEASE_3_23
git_last_commit: 9ab4ca1
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 00:53:53 UTC; biocbuild
Author: Maddy Griswold [cre, aut],
  Patrick Danaher [aut]
Maintainer: Maddy Griswold <mgriswold@nanostring.com>

# Generated by roxygen2: do not edit by hand

export(TIL_barplot)
export(collapseCellTypes)
export(create_profile_matrix)
export(derive_GeoMx_background)
export(download_profile_matrix)
export(florets)
export(mergeTumorIntoX)
export(reverseDecon)
export(runErrorModel)
export(spatialdecon)
exportMethods(runCollapseCellTypes)
exportMethods(runMergeTumorIntoX)
exportMethods(runReverseDecon)
exportMethods(runspatialdecon)
import(graphics)
import(stats)
import(utils)
importClassesFrom(GeomxTools,NanoStringGeoMxSet)
importFrom(Biobase,assayData)
importFrom(SeuratObject,GetAssayData)
importFrom(grDevices,colors)
importFrom(graphics,barplot)
importFrom(graphics,frame)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,plot)
importFrom(graphics,polygon)
importFrom(graphics,text)
importFrom(logNormReg,lognlm)
importFrom(methods,is)
importFrom(repmis,source_data)
importFrom(stats,cor)
importFrom(stats,cutree)
importFrom(stats,dist)
importFrom(stats,hclust)
importFrom(stats,lm)
importFrom(stats,pnorm)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(utils,data)
importFrom(utils,read.csv)

## Overview

The SpatialDecon library implements the SpatialDecon algorithm for mixed cell deconvolution in spatial gene expression datasets. (This algorithm also works in bulk expression profiling data.)

Details can be found in the SpatialDecon manuscript: Danaher & Kim (2022), "Advances in mixed cell deconvolution enable quantification of cell types in spatial transcriptomic data." https://www.nature.com/articles/s41467-022-28020-5

## Installation

### Download the release version from Bioconductor
<https://bioconductor.org/packages/release/bioc/html/SpatialDecon.html>

### Install the release version from Bioconductor
``` r
if (!requireNamespace("BiocManager", quietly=TRUE))
    install.packages("BiocManager")

BiocManager::install(version="release")

BiocManager::install("SpatialDecon")
```

## Documentation

To learn how to start using SpatialDecon, view documentation for the
version of this package installed in your system, start R and enter:

``` r
browseVignettes("SpatialDecon")
```

## Guide to functions

#### Data preparation functions:

* "download_profile_matrix" Downloads any one of ~75 cell profile matrices compiled for deconvolution of diverse tissue types. 
* "create_profile_matrix" Creates custom profile matrix from single cell data: count matrix and cell type annotations.  
* "derive_GeoMx_background_at_normalized_scale" Estimates the background expected from each data point in a GeoMx dataset. Accurate background estimation is key for SpatialDecon's accuracy. 

#### SpatialDecon: the core deconvolution function:

spatialdecon runs the SpatialDecon algorithm for estimating mixed cell type abundance in the regions
 of spatially-resolved gene expression studies. 
 It is also appropriate to apply to bulk gene expression data.

Its minimal required input is:

* A normalized data matrix
* A matrix of expected background counts at each element of the normalized data matrix 
* A matrix of expected cell type expression profiles


spatialdecon has numerous advanced options specified in the help file and demonstrated in the vignette.
These include:

* Merge closely-related cell types
* Estimate cell abundance on the scale of absolute cell counts
* Use regions of pure tumor cells (or any other cell type missing from the cell profile matrix) to infer a missing cell type's profile and estimate it alongside the cells with known profiles.


#### Plotting functions:

* "TIL_barplot" is a convenient way to draw barplots of cell type abundance/ proportion
* "florets" is for plotting cells in space. For each data point specified in xy space, it draws a circular barplot showing the localized abundance of cell types. 

#### Post-deconvolution analyses:

* "reverseDecon" Models genes ~ decon-derived cell scores. It produces fitted values, residuals, and various metrics of how much genes depend on cell mixing. In studies where cell mixing is a dominant source of variance, these residuals aid interpretation. 


## Other

* Depends: R >= 4.0, Mac, Unix or Windows
* Typical installation time: seconds
* Expected run time for vignette: 1 minute
* For reproducible code of all analyses in the SpatialDecon manuscript, see https://github.com/Nanostring-Biostats/SpatialDecon-manuscript-analyses
* license: GPL-3

## Computational benchmarking

The below memory usage and runtimes were gathered from applying spatialdecon to increasing numbers of GeoMx AOIs, using a 544-gene x 18 cell-type cell profile matrix:

| n  | memory (MB) | runtime (ms)  |
|---|---|---|
| 10  | 324  | 500  |
| 50  |  1598 | 2190  |
| 100  | 3130  | 4280  |
| 200  | 6266  | 8610  |
| 500  | 15244  | 21610  |
| 1000  | 30451  | 42580  |
| 5000  | 61522  | 83720  |

# SpatialDecon: mixed cell deconvolution for spatial and/or bulk gene expression data
# Copyright (C) 2020, NanoString Technologies, Inc.
#    This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
#    This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#    You should have received a copy of the GNU General Public License along with this program.  If not, see https://www.gnu.org/licenses/.
# Contact us:
# NanoString Technologies, Inc.
# 530 Fairview Avenue N
# Seattle, WA 98109
# Tel: (888) 358-6266
# pdanaher@nanostring.com


#' Reverse deconvolution
#'
#' Performs "reverse deconvolution", modelling each gene expression's ~ cell scores.
#' Returns a matrix of "fitted" expression values, a matrix of residuals, a matrix of
#' reverse decon coefficients for genes * cells.
#'
#' @param norm Matrix of normalized data, with genes in rows and observations in columns
#' @param beta Matrix of cell abundance estimates, with cells in rows and observations in columns.
#'  Columns are aligned to "norm".
#' @param epsilon All y and yhat values are thresholded up to this point when performing decon.
#'  Essentially says, "ignore variability in counts below this threshold."
#' @return A list:
#' \itemize{
#' \item coefs, a matrix of coefficients for genes * cells, where element i,j is interpreted as
#' "every unit increase in cell score j is expected to increase expression of gene i by _".
#' \item yhat, a matrix of fitted values, in the same dimension as norm
#' \item resids, a matrix of log2-scale residuals from the reverse decon fit, in the same
#'  dimension as norm
#' \item cors, a vector giving each gene's correlation between fitted and observed expression
#' \item resid.sd, a vector of each gene's residual SD, a metric of how much variability genes
#'  have independend of cell mixing.
#' }
#' @examples
#' data(mini_geomx_dataset)
#' data(safeTME)
#' # estimate background:
#' mini_geomx_dataset$bg <- derive_GeoMx_background(
#'   norm = mini_geomx_dataset$normalized,
#'   probepool = rep(1, nrow(mini_geomx_dataset$normalized)),
#'   negnames = "NegProbe"
#' )
#' # run basic decon:
#' res0 <- spatialdecon(
#'   norm = mini_geomx_dataset$normalized,
#'   bg = mini_geomx_dataset$bg,
#'   X = safeTME
#' )
#' # run reverse decon:
#' rdecon <- reverseDecon(
#'   norm = mini_geomx_dataset$norm,
#'   beta = res0$beta
#' )
#' @importFrom stats lm sd quantile cor
#' @export
reverseDecon <- function(norm, beta, epsilon = NULL) {
  
  # remove cell types with no SD:
  beta = beta[apply(beta, 1, stats::sd) > 0, ]
  # remove cell types that get removed by lm() (presumably removed due to linear dependence)
  lm1 = stats::lm(norm[1,] ~ t(beta))
  beta = beta[!is.na(lm1$coef[setdiff(names(lm1$coef), "(Intercept)")]), , drop = FALSE]
  
  # run reverse decon for all genes:
  rd <- function(y) {
    fit <- suppressWarnings(
      logNormReg::lognlm(y ~ t(beta),
                         lik = FALSE,
                         method = "L-BFGS-B",
                         lower = rep(0, ncol(beta) + 1),
                         upper = rep(Inf, ncol(beta) + 1),
                         opt = "optim",
                         control = list(maxit = 1000)
      )
    )
    return(fit$coefficients)
  }
  coefs <- t(apply(norm, 1, rd))
  colnames(coefs)[-1] <- rownames(beta)
  
  # get yhat
  yhat <- norm * NA
  for (ind in seq_len(ncol(yhat))) {
    yhat[, ind] <- coefs %*% c(1, beta[, ind])
  }
  
  # auto-select a reasonable epsilon if not provided
  if (length(epsilon) == 0) {
    epsilon <- stats::quantile(norm[norm > 0], 0.01)
  }
  
  # get resids:
  resids <- log2(pmax(norm, epsilon)) - log2(pmax(yhat, epsilon))
  
  # get summary stats:
  cors <- suppressWarnings(diag(stats::cor(t(norm), t(yhat))))
  resid.sd <- apply(resids, 1, stats::sd)
  
  out <- list(coefs = coefs, yhat = yhat, resids = resids, cors = cors, resid.sd = resid.sd)
  return(out)
}

context("spatialdecon")

library(testthat)


# strategy here:
# spatialdecon calls almost every function in the package
# we'll run it, then dissect its results to check that every piece of the
# puzzle worked we'll also test individual functions as a sanity check

#### load test data ----------------------
rm(list = ls())
load("testdata.RData")
load("expectedtestresults.RData")
data('safeTME')
data("safeTME.matches")
sharedgenes <- intersect(rownames(safeTME), rownames(snr))



#### test subsidiary functions individually -----------------------------------

test_that("deriveWeights is as expected", {
  wts <- deriveWeights(
    norm = snr,
    raw = raw,
    error.model = "dsp"
  )
  expect_true(all(abs(wts.test - wts) < 1e-3))
})

# merging tumor profiles:
test_that("mergeTumorIntoX is as expected", {
  set.seed(0)
  mergedX <- mergeTumorIntoX(
    norm = snr[sharedgenes, ],
    bg = replace(snr, TRUE, 1)[sharedgenes, ],
    pure_tumor_ids = annot$AOI.name == "Tumor",
    X = safeTME[sharedgenes, ],
    K = 5
  )
  expect_true(all(abs(mergedX.test - mergedX) < 1e-3))
})



# spatialdecon:
set.seed(0)
gres <- spatialdecon(
  norm = snr[sharedgenes, ],
  X = safeTME[sharedgenes, ],
  bg = replace(snr[sharedgenes, ], TRUE, 1),
  resid_thresh = 3, lower_thresh = 0.5
)

test_that("spatialdecon is as expected: beta", {
  expect_true(cor(as.vector(ires.test$beta), as.vector(gres$beta)) > 0.999)
})

test_that("spatialdecon is as expected: sigma", {
  expect_true(cor(as.vector(ires.test$sigmas), as.vector(gres$sigmas)) > 0.999)
})

test_that("spatialdecon is as expected: yhat", {
  expect_true(cor(as.vector(ires.test$yhat), as.vector(gres$yhat)) > 0.999)
})

test_that("spatialdecon is as expected: resids", {
  expect_true(all(abs(replace(ires.test$resids, is.na(ires.test$resids), 0) -
    replace(gres$resids, is.na(gres$resids), 0)) < 1e-2))
})

test_that("spatialdecon is as expected: p", {
  expect_true(all(abs(ires.test$p - gres$p) < 1e-2))
})



#### test SpatialDecon with TILs options ---------------------------------------
res <- spatialdecon(
  norm = snr,
  raw = raw,
  bg = replace(snr, TRUE, 1),
  cellmerges = safeTME.matches,
  is_pure_tumor = annot$AOI.name == "Tumor",
  cell_counts = annot$nuclei,
  n_tumor_clusters = 5
)

test_that("cell matching is as expected", {
  expect_equal(rownames(res.test$beta), rownames(res$beta))
})

test_that("spatialdeconTILs returned results as expected", {
  cols <- c("beta", "yhat", "sigma", "p", "t", "se", "prop_of_all", "prop_of_nontumor", 
            "cell.counts", "beta.granular", "sigma.granular", "cell.counts.granular", 
            "resids", "X")
  
  expect_true(all(cols %in% names(res)))
  expect_true(all(names(res) %in% cols))
})


test_that("spatialdeconTILs is as expected: beta", {
  expect_true(all(abs(res.test$beta - res$beta) < 1e-2))
})

test_that("spatialdeconTILs is as expected: sigma", {
  expect_true(all(abs(res.test$sigma - res$sigma) < 1e-2))
})

test_that("spatialdeconTILs is as expected: yhat", {
  expect_true(mean(abs(res.test$yhat - res$yhat)) < 1e-3)
})

test_that("spatialdeconTILs is as expected: resids", {
  expect_true(all(abs(replace(res.test$resids, is.na(res.test$resids), 0) -
    replace(res$resids, is.na(res$resids), 0)) < 1e-2))
})

test_that("spatialdeconTILs is as expected: p", {
  expect_true(all(abs(res.test$p - res$p) < 1e-2))
})

test_that("spatialdeconTILs is as expected: props", {
  expect_true(all(abs(res.test$prob_of_all - res$prob_of_all) < 1e-2))
  expect_true(all(abs(res.test$prob_of_nontumor - res$prob_of_nontumor) < 1e-2))
})

test_that("spatialdeconTILs is as expected: props of all", {
  expect_true(all(abs(res.test$prob_of_all - res$prob_of_all) < 1e-2))
  expect_true(all(abs(res.test$prob_of_nontumor - res$prob_of_nontumor) < 1e-2))
})

test_that("spatialdeconTILs is as expected: t", {
  expect_true(all(abs(res.test$t - res$t) < 1e-2))
})

test_that("spatialdeconTILs is as expected: se", {
  expect_true(all(abs(res.test$se - res$se) < 1e-2))
})

test_that("spatialdeconTILs is as expected: beta.granular", {
  expect_true(all(abs(res.test$beta.granular[,-7] - res$beta.granular[,-7]) < 2e-2))
})

### test reverse decon:
rdres <- suppressWarnings(reverseDecon(
  norm = snr,
  beta = res.test$beta,
  epsilon = 1
))

test_that("reverseDecon is as expected", {
    expect_true(all(abs(rdres.test$resids - rdres$resids) < 1e-1))
    expect_true(all(abs(rdres.test$yhat - rdres$yhat) < 1))
    expect_true(all(abs(rdres.test$coefs - rdres$coefs) < 100))
    expect_true(all(abs(rdres.test$cors - rdres$cors) < 1e-2, na.rm = TRUE))
    expect_true(all(abs(rdres.test$resid.sd - rdres$resid.sd) < 1e-2))
})




### test plotting functions
test_that("florets does not error", {
    expect_error(
        florets(
            x = annot$x, y = annot$y,
            b = res$beta.granular[!grepl("tumor", rownames(res$beta.granular)), ],
            legendwindow = TRUE
        ),
        NA
    )
})

test_that("TIL_barplot does not error", {
    expect_error(
        TIL_barplot(mat = res$beta, draw_legend = TRUE),
        NA
    )
})


### test matrix download:
test_that("matrix download works", {
  downloaded.X <- download_profile_matrix(species = "Human", 
                                          age_group = "Adult",
                                          matrixname = "Liver_HCA")
  expect_true(is.matrix(downloaded.X))
  expect_true(is.list(cellGroups))
  expect_true(all(colnames(downloaded.X) %in% unlist(cellGroups)))
  expect_true(all(downloaded.X == profile_matrix))
  
})
### test matrix creation
test_that("matrix creation works", {
  cellNames <- paste0("Cell", 1:1500)
  geneNames <- paste0("Gene", 1:1500)
  mtx <- matrix(data=sample(size = length(cellNames)*length(geneNames),
                            replace = TRUE,
                            x = 0:100,
                            prob = c(0.6784, rep(0.0075, 15), rep(0.005, 25),
                                   rep(0.002, 25),  rep(0.001, 35))),
                ncol = length(cellNames), nrow = length(geneNames),
                dimnames = list(geneNames,
                                cellNames))
  
  celltypes <- c("A", "B", "C", "D")
  cellAnnots <- as.data.frame(cbind(CellID=cellNames,
                                    cellType=sample(size = length(cellNames),
                                                    replace = TRUE,
                                                    x = celltypes,
                                                    prob = c(0.1, 0.4, 0.3, 0.2))))
  
  profile_matrix <- create_profile_matrix(mtx = mtx,
                                          cellAnnots = cellAnnots,
                                          cellTypeCol = "cellType",
                                          cellNameCol = "CellID",
                                          minGenes = 10,
                                          scalingFactor = 1,
                                          outDir = NULL
                                          )
  
  

  expect_true(is.matrix(profile_matrix))
  
  for(CT in celltypes){
    w2kp <- which(cellAnnots$cellType == CT)
    expect_true(all(rowMeans(mtx[,w2kp]) == profile_matrix[,CT]))
  }
  # expect_true(file.exists("Custom_profileMatrix.csv"))
  # 
  # unlink("Custom_profileMatrix.csv", force = TRUE)
})

### test collapseCellTypes:

# uncollapsed result:
res2 <- spatialdecon(
  norm = snr,
  raw = raw,
  bg = replace(snr, TRUE, 1),
  cellmerges = NULL,
  is_pure_tumor = annot$AOI.name == "Tumor",
  cell_counts = annot$nuclei,
  n_tumor_clusters = 5
)
# collapse them:
res2.collapsed <- collapseCellTypes(
  fit = res2,
  matching = safeTME.matches
)
# compare collapsed results from within spatialdecon vs. post-hoc:
test_that("collapseCellTypes works", {
  expect_true(all(abs(res2.collapsed$beta - res$beta) < 1e-2))
  expect_true(all(abs(res2.collapsed$sigmas - res$sigmas) < 1e-2))
  expect_true(all(abs(res2.collapsed$p - res$p) < 1e-2))
  expect_true(all(abs(res2.collapsed$t - res$t) < 1e-2))
  expect_true(all(abs(res2.collapsed$se - res$se) < 1e-2))
})


## test wrapper for seurat objects:
#make seurat object:
options(Seurat.object.assay.version = "v3")
seur <- SeuratObject::CreateSeuratObject(counts = raw, assay="Spatial")
test_that("runspatialdecon works on seurat objects - v3", {
  res <- runspatialdecon(seur)
  res2 <- spatialdecon(norm = raw, raw = raw, bg = 0.1)
  
  expect_true(is.matrix(res$beta)) # test beta is a matrix
  expect_true(is.matrix(res$yhat)) 
  expect_true(is.matrix(res$resids))
  expect_true(length(dim(res$sigmas)) == 3) # test sigmas is a 3d assar
  expect_true(is.matrix(res$p))
  expect_true(is.matrix(res$t))
  expect_true(is.matrix(res$se))
  expect_true(is.matrix(res$prop_of_all))
  expect_true(is.matrix(res$prop_of_nontumor))
  expect_true(is.matrix(res$X))
  
  expect_identical(res, res2)
})

options(Seurat.object.assay.version = "v5")
seur <- suppressWarnings(SeuratObject::CreateSeuratObject(counts = raw, assay="Spatial"))
test_that("runspatialdecon works on seurat objects - v5", {
  res <- suppressWarnings(runspatialdecon(seur))
  
  res2 <- spatialdecon(norm = raw, raw = raw, bg = 0.1)
  
  expect_true(is.matrix(res$beta)) # test beta is a matrix
  expect_true(is.matrix(res$yhat)) 
  expect_true(is.matrix(res$resids))
  expect_true(length(dim(res$sigmas)) == 3) # test sigmas is a 3d assar
  expect_true(is.matrix(res$p))
  expect_true(is.matrix(res$t))
  expect_true(is.matrix(res$se))
  expect_true(is.matrix(res$prop_of_all))
  expect_true(is.matrix(res$prop_of_nontumor))
  expect_true(is.matrix(res$X))
  
  expect_identical(res, res2)
})

## test wrapper for GeoMxSet objects:
#make GeoMxSet object:

datadir <- system.file("extdata", "DSP_NGS_Example_Data", package = "GeomxTools")
demoData <- readRDS(file.path(datadir, "/demoData.rds"))

demoData <- GeomxTools::shiftCountsOne(demoData)
demoData <- GeomxTools::aggregateCounts(demoData)

demoData <- GeomxTools::normalize(demoData, "quant")

norm <- as.matrix(demoData@assayData$exprs_norm)

bg <- derive_GeoMx_background(norm = norm,
                              # access the probe pool information from the feature metadata
                              probepool = demoData@featureData@data$Module,
                              # access the names of the negative control probes
                              negnames = demoData@featureData@data$TargetName[demoData@featureData@data$Negative])

test_that("derive_GeoMx_background is in correct format", {
  expect_true(all(dim(bg) == dim(norm)))
  expect_true(all(colnames(bg) == colnames(norm)))
  expect_true(all(rownames(bg) == rownames(norm)))
})


res <- runspatialdecon(object = demoData, 
                       norm_elt = "exprs_norm",
                       raw_elt = "exprs")

test_that("runspatialdecon works on GeoMxSet objects", {
  res2 <- spatialdecon(norm = norm, bg = bg, 
                       raw = as.matrix(demoData@assayData$exprs))

  expect_true(is.matrix(res@phenoData@data$beta)) # test beta is a matrix
  expect_true(is.matrix(res@assayData$yhat)) 
  expect_true(is.matrix(res@assayData$resids))
  expect_true(length(dim(res@phenoData@data$sigmas)) == 3) # test sigmas is a 3d assar
  expect_true(is.matrix(res@phenoData@data$p))
  expect_true(is.matrix(res@phenoData@data$t))
  expect_true(is.matrix(res@phenoData@data$se))
  expect_true(is.matrix(res@phenoData@data$prop_of_all))
  expect_true(is.matrix(res@phenoData@data$prop_of_nontumor))
  expect_true(is.matrix(res@experimentData@other$SpatialDeconMatrix))
  
  expect_identical(res@phenoData@data$beta, t(res2$beta))
  expect_identical(res@assayData$yhat[match(rownames(res2$yhat), rownames(res@assayData$yhat)),], 
                   res2$yhat) 
  expect_identical(res@assayData$resids[match(rownames(res2$resids), rownames(res@assayData$resids)),], 
                   res2$resids)
  expect_identical(res@phenoData@data$p, t(res2$p))
  expect_identical(res@phenoData@data$t, t(res2$t))
  expect_identical(res@phenoData@data$se, t(res2$se))
  expect_identical(res@phenoData@data$prop_of_all, t(res2$prop_of_all))
  expect_identical(res@phenoData@data$prop_of_nontumor, t(res2$prop_of_nontumor))
  expect_identical(res@experimentData@other$SpatialDeconMatrix, res2$X)
})

sharedgenes <- intersect(rownames(safeTME), rownames(norm))

test_that("runmergeTumorIntoX works on GeoMxSet objects", {
    set.seed(0)
    mergedX <- mergeTumorIntoX(
      norm = norm,
      bg = bg,
      pure_tumor_ids = demoData@phenoData@data$`scan name` == "cw005 (PTL-10891) Slide1",
      X = safeTME[sharedgenes, ],
      K = 5
    )
    
    mergedX.test <- runMergeTumorIntoX(demoData, 
                                       norm_elt = "exprs_norm", 
                                       pure_tumor_ids = demoData@phenoData@data$`scan name` == "cw005 (PTL-10891) Slide1",
                                       X = safeTME[sharedgenes, ],
                                       K=5)
    
    expect_true(all(abs(mergedX.test - mergedX) < 1e-3))
})

test_that("runreverseDecon works on GeoMxSet objects", {
  rdres <- suppressWarnings(runReverseDecon(demoData, 
                                            norm_elt = "exprs_norm",
                                            beta = res@phenoData@data$beta,
                                            epsilon = 1))
  
  rdres.x <- suppressWarnings(reverseDecon(norm = norm,
                                           beta = t(res@phenoData@data$beta),
                                           epsilon = 1))
  
  
  expect_true(all(abs(rdres@featureData@data$coefs - rdres.x$coefs) < 1e-3))
  expect_true(all(abs(rdres@featureData@data$cors - rdres.x$cors) < 1e-3))
  expect_true(all(abs(rdres@featureData@data$resid.sd - rdres.x$resid.sd) < 1e-3))
  expect_true(all(abs(rdres@assayData$resids - rdres.x$resids) < 1e-3))
  expect_true(all(abs(rdres@assayData$yhat - rdres.x$yhat) < 1e-3))
})

test_that("runCollapseCellTypes works on GeoMxSet objects", {
  collapsed <- runCollapseCellTypes(object = res, 
                                    matching = safeTME.matches)
  
  expect_true(all(colnames(collapsed$beta) %in% names(safeTME.matches)))
  expect_lt(ncol(collapsed$beta), ncol(res$beta))
  expect_equal(nrow(collapsed$beta), nrow(res$beta))
})

# SpatialDecon: mixed cell deconvolution for spatial and/or bulk gene expression
# data
# Copyright (C) 2020, NanoString Technologies, Inc.
#    This program is free software: you can redistribute it and/or modify it
#    under the terms of the GNU General Public License as published by the Free
#    Software Foundation, either version 3 of the License, or (at your option)
#    any later version.
#    This program is distributed in the hope that it will be useful, but WITHOUT
#    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
#    more details.
#    You should have received a copy of the GNU General Public License along
#    with this program.  If not, see https://www.gnu.org/licenses/.
# Contact us:
# NanoString Technologies, Inc.
# 530 Fairview Avenue N
# Seattle, WA 98109
# Tel: (888) 358-6266
# pdanaher@nanostring.com

#' Draw coxcomb plots as points in a graphics window
#'
#' Draws a scatterplot where each point is a circular barplot, intended to show
#' decon results
#'
#' @param x Vector of x coordinates
#' @param y Vector of y coordinates
#' @param b matrix or cell abundances, with columns aligned with the elements
#' of x and y
#' @param col vector of colors, aligned to the rows of b.
#' @param legendwindow Logical. If TRUE, the function draws a color legend in a
#'  new window
#' @param rescale.by.sqrt Logical, for whether to rescale b by its square root
#' to make value proportional to
#'  shape area, not shape length.
#' @param border Color of pie segment border, defauls to NA/none
#' @param add Logical. If TRUE, the function draws florets atop an existing
#' graphics device (TRUE) or call a new device (FALSE).
#' @param cex Floret size. Florets are scaled relative to the range of x and y;
#' this further scales up or down.
#' @param bty bty argument passed to plot()
#' @param xaxt xaxt argument passed to plot()
#' @param yaxt yaxt argument passed to plot()
#' @param xlab xlab, defaults to ""
#' @param ylab ylab, defaults to ""
#' @param ... additional arguments passed to plot()
#' @return Draws a coxcomb plot, returns no data.
#' @examples
#' data(mini_geomx_dataset)
#' data(safeTME)
#' # estimate background:
#' mini_geomx_dataset$bg <- derive_GeoMx_background(
#'   norm = mini_geomx_dataset$normalized,
#'   probepool = rep(1, nrow(mini_geomx_dataset$normalized)),
#'   negnames = "NegProbe"
#' )
#' # run basic decon:
#' res0 <- spatialdecon(
#'   norm = mini_geomx_dataset$normalized,
#'   bg = mini_geomx_dataset$bg,
#'   X = safeTME
#' )
#' # draw florets:
#' florets(
#'   x = mini_geomx_dataset$annot$x,
#'   y = mini_geomx_dataset$annot$y,
#'   b = res0$beta, cex = 2
#' )
#' @importFrom graphics polygon plot lines text
#' @importFrom grDevices colors
#' @importFrom utils data
#' @export
florets <- function(x, y, b, col = NULL, legendwindow = FALSE,
                    rescale.by.sqrt = TRUE, border = NA, add = FALSE, cex = 1,
                    bty = "n", xaxt = "n", yaxt = "n", xlab = "", ylab = "",
                    ...) {
    utils::data("cellcols", envir = environment())
    
    if (rescale.by.sqrt) {
        b <- sqrt(b)
    }
    # make b a matrix:
    if (is.vector(b)) {
        b2 <- matrix(b, nrow = length(b))
        rownames(b2) <- names(b)
        b <- b2
        rm(b2)
    }
    # choose colors if not given:
    if ((length(col) == 0) &
        all(is.element(rownames(b), names(SpatialDecon::cellcols)))) {
        col <- SpatialDecon::cellcols[rownames(b)]
    }
    if ((length(col) == 0) &
        !all(is.element(rownames(b), names(SpatialDecon::cellcols)))) {
        manycols <- c(
            "#8DD3C7", "#FFFFB3", "#BEBADA", "#FB8072", "#80B1D3", "#FDB462",
            "#B3DE69", "#FCCDE5", "#A6CEE3", "#1F78B4", "#B2DF8A", "#33A02C",
            "#FB9A99", "#E31A1C", "#FDBF6F", "#FF7F00", "#1B9E77", "#D95F02",
            "#7570B3", "#E7298A", "#66A61E", "#E6AB02", "#A6761D", "#666666",
            sample(grDevices::colors(), 99)
        )
        col <- manycols[seq_len(nrow(b))]
        names(col) <- rownames(b)
    }
    # convert colors to matrix of the same dimension as b:
    if (length(col) == 1) {
        col <- matrix(col, nrow = nrow(b), ncol = ncol(b))
    }
    if (is.vector(col)) {
        col <- matrix(col, nrow = nrow(b), ncol = ncol(b))
    }
    
    # get radians:
    angles <- seq(0, 2 * pi, length.out = nrow(b) + 1)
    
    # scale b based on the range of x and y:
    maxrange <- max(diff(range(x, na.rm = TRUE)), diff(range(y, na.rm = TRUE)))
    b <- b * maxrange / mean(b, na.rm = TRUE) * 0.007 * cex
    
    # draw plot:
    if (!add) {
        graphics::plot(x, y,
                       col = 0, bty = bty, xaxt = xaxt, yaxt = yaxt,
                       xlab = xlab, ylab = ylab, ...
        )
    }
    
    # draw florets:
    if (nrow(b) > 1) {
        for (i in seq_len(length(x))) {
            for (j in seq_len(nrow(b))) {
                tempangles <- seq(angles[j], angles[j + 1], length.out = 20)
                xt <- b[j, i] * cos(tempangles)
                yt <- b[j, i] * sin(tempangles)
                graphics::polygon(x[i] + c(0, xt), y[i] + c(0, yt),
                                  col = col[j, i],
                                  border = border, lwd = 0.5
                )
            }
        }
    }
    
    # if just one point, draw a full circle:
    if (nrow(b) == 1) {
        for (i in seq_len(length(x))) {
            for (j in seq_len(nrow(b))) {
                tempangles <- seq(angles[j], angles[j + 1], length.out = 20)
                xt <- b[j, i] * cos(tempangles)
                yt <- b[j, i] * sin(tempangles)
                graphics::polygon(x[i] + xt, y[i] + yt,
                                  col = col[j],
                                  border = border, lwd = 0.5
                )
            }
        }
    }
    
    # draw a legend:
    if (legendwindow) {
        graphics::plot(0, 0,
                       col = 0, xlim = c(-1, 1), ylim = c(-1, 1), xaxt = "n",
                       yaxt = "n", xlab = "", ylab = "", ...
        )
        for (j in seq_len(length(angles))) {
            graphics::lines(c(0, 0.75 * cos(angles[j])), c(0, 0.75 * sin(angles[j])),
                            col = col[j], lwd = 2
            )
            graphics::text(0.85 * cos(angles[j]), 0.85 * sin(angles[j]),
                           rownames(b)[j],
                           cex = 1.4
            )
        }
    }
}

# SpatialDecon: mixed cell deconvolution for spatial and/or bulk gene expression
# data
# Copyright (C) 2020, NanoString Technologies, Inc.
#    This program is free software: you can redistribute it and/or modify it
#    under the terms of the GNU General Public License as published by the Free
#    Software Foundation, either version 3 of the License, or (at your option)
#    any later version.
#    This program is distributed in the hope that it will be useful, but WITHOUT
#    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
#    more details.
#    You should have received a copy of the GNU General Public License along
#    with this program.  If not, see https://www.gnu.org/licenses/.
# Contact us:
# NanoString Technologies, Inc.
# 530 Fairview Avenue N
# Seattle, WA 98109
# Tel: (888) 358-6266
# pdanaher@nanostring.com



#' Deconvolution using logNormReg package to run linear mean model and log error
#'  model
#'
#' Calls lognlm() to optimize the model.
#'
#' @param Y p-length expression vector or p * N expression matrix - the actual
#'  (linear-scale) data
#' @param X p * K Training matrix
#' @param bg scalar or matrix of expected background counts per data point.
#' @param weights The same as the weights argument used by lm
#' @param epsilon optional,  a very small non-zero number to use as a lower
#' threshold to make fits well-behaved
#' @param maxit Maximum number of iterations. Default 1000.
#' @return a list: beta (estimate), sigmas (covariance matrix of estimate,
#' derived by inverting the hessian from lognlm)
#' @keywords internal
#' 
#' @importFrom logNormReg lognlm
#' 
#' @noRd
deconLNR <- function(Y, X, bg = 0, weights = NULL, epsilon = NULL,
                     maxit = 1000) {
    if (length(weights) == 0) {
        weights <- replace(Y, TRUE, 1)
    }
    if (is.vector(Y)) {
        Y <- matrix(Y, nrow = length(Y))
    }
    if (length(bg) == 1) {
        bg <- matrix(bg, nrow(Y), ncol(Y))
    }
    # choose "epsilon": a very small non-zero number to make fits well-behaved
    if (length(epsilon) == 0) {
        epsilon <- min(replace(Y, (Y == 0) & !is.na(Y), NA), na.rm = TRUE)
    }
    
    # matrix-like data for apply:
    mat <- rbind(Y, bg, weights)
    # fn to apply:
    fn <- function(zz) {
        # break into y, b, w:
        y <- zz[seq_len((length(zz)) / 3)]
        b <- zz[seq_len((length(zz)) / 3) + (length(zz) / 3)]
        wts <- zz[seq_len((length(zz)) / 3) + (length(zz) / 3) * 2]
        
        # remove NA data:
        use <- !is.na(y)
        y <- y[use]
        b <- b[use]
        Xtemp <- X[use, , drop = FALSE]
        wts <- wts[use]
        
        init <- rep(mean(y) / (mean(X) * ncol(X)), ncol(X))
        names(init) <- colnames(X)
        
        # run lognlm:
        fit <- logNormReg::lognlm(pmax(y, epsilon) ~ b + Xtemp - 1,
                       lik = FALSE,
                       weights = wts,
                       start = c(1, init),
                       method = "L-BFGS-B",
                       lower = c(1, rep(0, ncol(Xtemp))),
                       upper = c(1, rep(Inf, ncol(Xtemp))),
                       opt = "optim",
                       control = list(maxit = maxit)
        )
        fnout <- list(
            beta = fit$coefficients[-1],
            sigma = solve(fit$hessian)[-1, -1]
        )
        return(fnout)
    }
    # apply across all observations:
    fnlist <- apply(mat, 2, fn)
    # extract beta and sigmas:
    getbeta <- function(zz) {
        return(zz$beta)
    }
    getsigma <- function(zz) {
        return(zz$sigma)
    }
    
    beta <- vapply(X = fnlist, FUN = getbeta, FUN.VALUE = numeric(ncol(X)))
    rownames(beta) <- colnames(X)
    sigmas <- array(vapply(
        X = fnlist,
        FUN = getsigma,
        FUN.VALUE = numeric(ncol(X)^2)
    ),
    dim = c(ncol(X), ncol(X), ncol(Y)),
    dimnames = list(colnames(X), colnames(X), colnames(Y))
    )
    
    out <- list(beta = pmax(beta, 0), sigmas = sigmas)
    return(out)
}

# SpatialDecon: mixed cell deconvolution for spatial and/or bulk gene expression
# data
# Copyright (C) 2020, NanoString Technologies, Inc.
#    This program is free software: you can redistribute it and/or modify it
#    under the terms of the GNU General Public License as published by the Free
#    Software Foundation, either version 3 of the License, or (at your option)
#    any later version.
#    This program is distributed in the hope that it will be useful, but WITHOUT
#    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
#    more details.
#    You should have received a copy of the GNU General Public License along
#    with this program.  If not, see https://www.gnu.org/licenses/.
# Contact us:
# NanoString Technologies, Inc.
# 530 Fairview Avenue N
# Seattle, WA 98109
# Tel: (888) 358-6266
# pdanaher@nanostring.com

#' Collapse related cell types within a deconvolution result
#'
#' Given the input of an SpatialDecon result output and a list of which cell
#' types to combine,
#'  returns a reshaped deconvolution result object with the specified cell
#'  types merged.
#' @param fit The object (a list) returned by the SpatialDecon algorithm
#' @param matching A list object holding the mapping from beta's cell names to
#' official cell names.
#'  See str(safeTME.matches) for an example.
#' @return A reshaped deconvolution result object
#' @examples
#' data(mini_geomx_dataset)
#' data(safeTME)
#' data(safeTME.matches)
#' # estimate background:
#' mini_geomx_dataset$bg <- derive_GeoMx_background(
#'   norm = mini_geomx_dataset$normalized,
#'   probepool = rep(1, nrow(mini_geomx_dataset$normalized)),
#'   negnames = "NegProbe"
#' )
#' # run basic decon:
#' res0 <- spatialdecon(
#'   norm = mini_geomx_dataset$normalized,
#'   bg = mini_geomx_dataset$bg,
#'   X = safeTME
#' )
#' res1 <- collapseCellTypes(
#'     fit = res0,
#'     matching = safeTME.matches
#' )
#' @importFrom stats pnorm
#' @export
collapseCellTypes <- function(fit, matching) {
    
    # results object to hold the collapsed results:
    out <- fit
    
    # format matching list as a matrix to take a linear combination of beta:
    startingcellnames <- unlist(matching)
    A <- matrix(0, length(matching), nrow(fit$beta),
                dimnames = list(names(matching), rownames(fit$beta))
    )
    for (name in names(matching)) {
        cellnames <- matching[[name]]
        A[name, cellnames] <- 1
    }
    
    # apply A transformation to beta:
    for (name in c("beta", "prop_of_all", "prop_of_nontumor")) {
        if (is.element(name, names(fit))) {
            out[[name]] <- A[, startingcellnames] %*% fit[[name]][startingcellnames, ]
        }
    }
    
    # if Sigma provided, get vcov of beta2:
    if (is.element("sigmas", names(out))) {
        sigma <- fit$sigmas
        if (length(dim(sigma)) == 2) {
            out$sigmas <- A[, startingcellnames] %*%
                sigma[startingcellnames, startingcellnames, ] %*%
                t(A[, startingcellnames])
        }
        if (length(dim(sigma)) == 3) {
            out$sigmas <- array(NA,
                                dim = c(nrow(A), nrow(A), dim(sigma)[3]),
                                dimnames = list(rownames(A), rownames(A), dimnames(sigma)[[3]])
            )
            for (i in seq_len(dim(sigma)[3])) {
                out$sigmas[, , i] <- A %*% sigma[, , i] %*% t(A)
            }
        }
    }
    
    # re-calculate p, se, t:
    if (is.element("beta", names(out)) & is.element("sigmas", names(out))) {
        # compute p-values
        tempbeta <- out$beta
        tempse <- tempp <- tempt <- tempbeta * NA
        for (i in seq_len(ncol(tempse))) {
            tempse[, i] <- suppressWarnings(sqrt(diag(out$sigmas[, , i])))
        }
        out$se <- tempse
        out$t <- (tempbeta / tempse)
        if (is.element("X", names(out))) {
            out$p <- 2 * (1 - stats::pnorm(out$t))
        }
    }
    
    return(out)
}

# SpatialDecon: mixed cell deconvolution for spatial and/or bulk gene expression
# data
# Copyright (C) 2020, NanoString Technologies, Inc.
#    This program is free software: you can redistribute it and/or modify it
#    under the terms of the GNU General Public License as published by the Free
#    Software Foundation, either version 3 of the License, or (at your option)
#    any later version.
#    This program is distributed in the hope that it will be useful, but WITHOUT
#    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
#    more details.
#    You should have received a copy of the GNU General Public License along
#    with this program.  If not, see https://www.gnu.org/licenses/.
# Contact us:
# NanoString Technologies, Inc.
# 530 Fairview Avenue N
# Seattle, WA 98109
# Tel: (888) 358-6266
# pdanaher@nanostring.com


#' Identify outlier genes in a decon result
#'
#' Analyses a decon result's residuals to identify poorly-fit genes and flag
#' them for removal.
#'  Rule: flag anything with
#'
#' @param Y p-length expression vector or p * N expression matrix - the actual
#' (linear-scale) data
#' @param yhat Expectation of Y given the decon fit
#' @param resids Log2-scale residuals of Y vs. yhat
#' (log2(pmax(Y, 0.5)) - log2(yhat))
#' @param wts Matrix of data point weights, aligned to dimensions of resids
#' @param resid_thresh A scalar, sets a threshold on how extreme individual
#' data points' values
#'  can be (in log2 units) before getting flagged as outliers and set to NA.
#' @return a vector of names of poorly-fit genes
#' @keywords internal
#' @noRd
flagOutliers <- function(Y, yhat, resids, wts, resid_thresh = 3) {
    
    # get weighted resids:
    if (length(wts) == 0) {
        wres <- resids
    }
    if (length(wts) > 0) {
        wres <- resids * wts
    }
    
    # flag bad genes:
    outlier_genes <- c() # <-- this line makes it so no outlier genes are filtered
    # flag bad data points: (not doing anything for now)
    outlier_data_points <- abs(resids) > resid_thresh
    return(outlier_data_points)
}

# SpatialDecon: mixed cell deconvolution for spatial and/or bulk gene expression
# data
# Copyright (C) 2020, NanoString Technologies, Inc.
#    This program is free software: you can redistribute it and/or modify it
#    under the terms of the GNU General Public License as published by the Free
#    Software Foundation, either version 3 of the License, or (at your option)
#    any later version.
#    This program is distributed in the hope that it will be useful, but WITHOUT
#    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
#    more details.
#    You should have received a copy of the GNU General Public License along
#    with this program.  If not, see https://www.gnu.org/licenses/.
# Contact us:
# NanoString Technologies, Inc.
# 530 Fairview Avenue N
# Seattle, WA 98109
# Tel: (888) 358-6266
# pdanaher@nanostring.com


#' Estimate a tumor-specific profile and merge it with the pre-specified cell
#'  profile matrix (X)
#'
#' Given the input of "tumor-only" AOI's, estimates an collection of
#'  tumor-specific
#' expression profiles and merges them with the immune cell expression
#' training matrix.
#' The process:
#' \enumerate{
#' \item log2/normalized data from tumor-only AOIs is clustered with hclust,
#' and cutree() is used to define clusters.
#' \item 2. Each cluster's geomean profile is merged into the immune cell
#' profile matrix.
#' }
#'
#' @param norm matrix of normalized data
#' @param bg matrix of expected background, on the scale of norm.
#' @param pure_tumor_ids Vector identifying columns of norm that are pure tumor.
#'  Can be indices, logicals or column names.
#' @param X The training matrix
#' @param K the number of clusters to fit
#' @return an updated X matrix with new columns, "tumor.1", "tumor.2", ...
#' @examples
#' data(mini_geomx_dataset)
#' data(safeTME)
#' mini_geomx_dataset$bg <- derive_GeoMx_background(
#'   norm = mini_geomx_dataset$normalized,
#'   probepool = rep(1, nrow(mini_geomx_dataset$normalized)),
#'   negnames = "NegProbe"
#' )
#' safeTME.with.tumor <- mergeTumorIntoX(
#'   norm = mini_geomx_dataset$norm,
#'   bg = mini_geomx_dataset$bg,
#'   pure_tumor_ids = mini_geomx_dataset$annot$AOI.name == "Tumor",
#'   X = safeTME,
#'   K = 3
#' )
#' @importFrom stats dist cutree hclust quantile
#' @export
mergeTumorIntoX <- function(norm, bg, pure_tumor_ids, X, K = 10) {
    
    # round up 0 values in norm:
    min.nonzero <- min(norm[norm > 0], na.rm = TRUE)
    norm <- pmax(norm, min.nonzero)
    
    # subset data to only the pure tumor IDs:
    norm <- norm[, pure_tumor_ids, drop = FALSE]
    bg <- bg[, pure_tumor_ids, drop = FALSE]
    
    # bg-subtract:
    norm <- pmax(norm - bg, min(norm) / 20)
    
    # fix K if too big:
    if (ncol(norm) < K) {
        K <- ncol(norm)
    }
    
    # case 1: want to use every column in norm as a separate profile:
    #  (includes case of just one column in norm)
    if (K == ncol(norm)) {
        tumorX <- norm
    }
    
    # case 2: if many tumor AOIs, get profiles for K clusters of data:
    if (K < ncol(norm)) {
        # cluster and cut:
        h <- stats::hclust(stats::dist(t(log2(norm))))
        cut <- stats::cutree(h, k = K)
        # get clusters' geomean profiles:
        tumorX <- c()
        for (cid in unique(cut)) {
            tumorX <- cbind(
                tumorX,
                exp(rowMeans(log(norm[, cut == cid, drop = FALSE])))
            )
        }
        colnames(tumorX) <- paste0("tumor.", seq_len(ncol(tumorX)))
    }
    
    # align tumorX with X:
    sharedgenes <- intersect(rownames(tumorX), rownames(X))
    tumorX <- tumorX[sharedgenes, , drop = FALSE]
    X <- X[sharedgenes, , drop = FALSE]
    
    # rescale tumor X:
    meanq90 <- max(mean(apply(X, 2, stats::quantile, 0.9)), 1e-3)
    tumorq90s <- apply(tumorX, 2, stats::quantile, 0.9)
    tumorX <- sweep(tumorX, 2, tumorq90s, "/") * meanq90
    
    # merge:
    out <- cbind(X, tumorX)
    return(out)
}

#' Run Reversedecon
#' 
#' Runs reversedecon from an S4 object
#' @param object An S4 object such as a GeoMxSet object
#' @param ... Arguments passed to reversedecon
#' 
#'

setGeneric("runReverseDecon", signature = "object",
           function(object, ...) standardGeneric("runReverseDecon"))

#' Run reversedecon on a NanostringGeomxSet object
#'
#' A wrapper for applying reversedecon to a NanostringGeomxSet object.
#'
#'
#' @param norm_elt normalized data element in assayData.
#' @param beta Matrix of cell abundance estimates, with cells in columns and observations in rows.
#'  Columns are aligned to "norm".
#' @param epsilon All y and yhat values are thresholded up to this point when performing decon.
#'  Essentially says, "ignore variability in counts below this threshold."
#' @return a valid GeoMx S4 object including the following items:
#' \itemize{
#'  \item in fData
#'  \itemize{
#'  \item coefs, a matrix of coefficients for genes * cells, where element i,j is interpreted as
#'               "every unit increase in cell score j is expected to increase expression of gene i by _".
#'  \item cors, a vector giving each gene's correlation between fitted and observed expression
#'  \item resid.sd, a vector of each gene's residual SD, a metric of how much variability genes
#'                  have independend of cell mixing.
#'  }
#'  \item in assayData
#'  \itemize{
#'    \item yhat, a matrix of fitted values, in the same dimension as norm
#'    \item resids, a matrix of log2-scale residuals from the reverse decon fit, in the same
#'                dimension as norm
#'  }
#'  
#' }
#' @importFrom Biobase assayData
#' @examples
#' library(GeomxTools)
#' datadir <- system.file("extdata", "DSP_NGS_Example_Data", package = "GeomxTools")
#' demoData <- readRDS(file.path(datadir, "/demoData.rds"))
#' 
#' demoData <- shiftCountsOne(demoData)
#' target_demoData <- aggregateCounts(demoData)
#' 
#' target_demoData <- normalize(target_demoData, "quant")
#'                 
#' # run basic decon:
#' res0 <- runspatialdecon(object = target_demoData,
#'                         norm_elt = "exprs_norm",
#'                         raw_elt = "exprs")
#'
#' # run reverse decon:
#' target_demoData <- runReverseDecon(object = target_demoData,
#'                                    norm_elt = "exprs_norm",
#'                                    beta = pData(res0)$beta)
#'
#' @export
#' @rdname runReverseDecon
setMethod("runReverseDecon",  "NanoStringGeoMxSet",
          function(object, norm_elt = NULL, beta, epsilon = NULL){
              
              if(is.null(norm_elt)){
                  stop("norm_elt must be set")
              }
              
              if (!is.element(norm_elt, names(object@assayData))) {
                  stop(paste(norm_elt, "is not an element in assaysData slot"))
              }
              
              # prep components:
              norm <- as.matrix(Biobase::assayDataElement(object, elt = norm_elt))
              
              # run runReverseDeconGeomx:
              result <- reverseDecon(norm = norm,
                                     beta = t(beta),
                                     epsilon = epsilon)
              
              
              # append results to the object
              # add coefs
              Biobase::fData(object)[["coefs"]] <- matrix(NA, nrow = nrow(object), ncol = ncol(result$coefs),
                                                          dimnames = list(Biobase::featureNames(object), colnames(result$coefs)))
              Biobase::fData(object)[["coefs"]][rownames(result$coefs), ] <- result$coefs
              
              
              # add yhat matrix
              yhat_mat <- matrix(NA, nrow = nrow(object), ncol = ncol(object),
                                 dimnames = dimnames(object))
              yhat_mat[rownames(result$yhat), colnames(result$yhat)] <- result$yhat
              Biobase::assayDataElement(object, "yhat") <- yhat_mat
              
              
              # add resids matrix
              resids_mat <- matrix(NA, nrow = nrow(object), ncol = ncol(object),
                                   dimnames = dimnames(object))
              resids_mat[rownames(result$resids), colnames(result$resids)] <- result$resids
              Biobase::assayDataElement(object, "resids") <- resids_mat
              
              
              # add cors
              Biobase::fData(object)[["cors"]] <- NA
              Biobase::fData(object)[["cors"]][match(names(result$cors), Biobase::featureNames(object), nomatch = 0)] <- result$cors
              
              
              # add resid.sd
              Biobase::fData(object)[["resid.sd"]] <- NA
              Biobase::fData(object)[["resid.sd"]][match(names(result$resid.sd), Biobase::featureNames(object), nomatch = 0)] <- result$resid.sd
              
              return(object)
              
          })

# SpatialDecon: mixed cell deconvolution for spatial and/or bulk gene expression
# data
# Copyright (C) 2020, NanoString Technologies, Inc.
#        This program is free software: you can redistribute it and/or modify it
#        under the terms of the GNU General Public License as published by the Free
#        Software Foundation, either version 3 of the License, or (at your option)
#        any later version.
#        This program is distributed in the hope that it will be useful, but WITHOUT
#        ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#        FITNESS FOR A PARTICULAR PURPOSE.    See the GNU General Public License for
#        more details.
#        You should have received a copy of the GNU General Public License along
#        with this program.    If not, see https://www.gnu.org/licenses/.
# Contact us:
# NanoString Technologies, Inc.
# 530 Fairview Avenue N
# Seattle, WA 98109
# Tel: (888) 358-6266
# pdanaher@nanostring.com


#' Create Custom Cell Profile Matrix
#'
#' Create custom cell profile matrix using single cell data. The average gene expression for each cell type is returned.
#'
#' @param mtx gene x cell count matrix
#' @param cellAnnots cell annotations with cell type and cell name as columns
#' @param cellTypeCol column containing cell type
#' @param cellNameCol column containing cell ID/name
#' @param matrixName name of final profile matrix
#' @param outDir path to desired output directory, set to NULL if matrix should not be written
#' @param geneList gene list to filter profile matrix to 
#' @param normalize Should data be normalized? (TRUE/FALSE) if TRUE data will be normalize using total gene count
#' @param scalingFactor what should all values be multiplied by for final matrix, set to 1 if no scaling is wanted
#' @param minCellNum minimum number of cells of one type needed to create profile, exclusive 
#' @param minGenes minimum number of genes expressed in a cell, exclusive
#' @param discardCellTypes should cell types be filtered for types like mitotic, doublet, low quality, unknown, etc.
#' @return A custom cell profile matrix genes (rows) by cell types (columns), matrix gets written to disk and outDir
#' @examples
#' cellNames <- paste0("Cell", seq_len(1500))
#' geneNames <- paste0("Gene", seq_len(1500))
#' mtx <- matrix(data=sample(size = length(cellNames)*length(geneNames),
#'                           replace = TRUE,
#'                           x = c(0,seq_len(100)), 
#'                           prob = c(0.6784, rep(0.0075, 15), rep(0.005, 25),
#'                                    rep(0.002, 25), rep(0.001, 35))), 
#'                           ncol = length(cellNames), nrow = length(geneNames), 
#'                           dimnames = list(geneNames, cellNames))
#' cellAnnots <- as.data.frame(cbind(CellID=cellNames, 
#'                                   cellType=sample(size = length(cellNames), 
#'                                                   replace = TRUE,
#'                                                   x = c("A", "B", "C", "D"),
#'                                                   prob = c(0.1, 0.4, 0.3, 0.2))))
#' table(cellAnnots$cellType)
#' profile_matrix <- create_profile_matrix(mtx = mtx,    
#'                                         cellAnnots = cellAnnots, 
#'                                         cellTypeCol = "cellType",
#'                                         cellNameCol = "CellID",
#'                                         minGenes = 10,
#'                                         scalingFactor = 1)
#' head(profile_matrix)
#' @importFrom methods is
#' @export

create_profile_matrix <- function(mtx, cellAnnots, cellTypeCol, cellNameCol,
                                  matrixName = "Custom", outDir = "./", 
                                  geneList = NULL, normalize = FALSE,
                                  scalingFactor = 5, minCellNum = 15,
                                  minGenes = 100, discardCellTypes = FALSE) {
    
    # checking user input values
    if(is.null(mtx)){
        stop("count matrix is necessary")
    }
    if(!is.null(outDir)){
        if (!dir.exists(outDir) ) {
            stop("Output directory is not valid")
        }
    }
    if (is.null(cellAnnots)){
        stop("Cell Annotations are needed")
    }
    
    if(is.null(cellTypeCol)){
        stop("cellTypeCol must not be NULL")
    }else if (!cellTypeCol %in% colnames(cellAnnots)){
        stop("cellTypeCol not in cellAnnots")
    }
    
    if(is.null(cellNameCol)){
        stop("cellNameCol must not be NULL")
    }else if (!cellNameCol %in% colnames(cellAnnots)){
        stop("cellNameCol not in cellAnnots")
    }
    
    if(!is(normalize, "logical")){
        warning("normalize not a boolean, continuing with assumption that data should not be normalized")
        normalize <- TRUE
    }
    if(!is(discardCellTypes, "logical")){
        warning("discardCellTypes not a boolean, continuing with default of discarding cell types")
        discardCellTypes <- TRUE
    }
    
    if(!is(scalingFactor, "numeric")){
        warning("scalingFactor not a numeric, continuing with default value of 5")
        scalingFactor <- 5
    }
    if(!is(minCellNum, "numeric")){
        warning("minCellNum not a numeric, continuing with default value of 15")
        minCellNum <- 15
    }
    if(!is(minGenes, "numeric")){
        warning("minGenes not a numeric, continuing with default value of 100")
        minGenes <- 100
    }
    
    #make a sparse matrix
    mtx <- Matrix::Matrix(as.matrix(mtx), sparse = TRUE) 
    
    cellTypes <- NULL
    
    #read in cell type annotation file
    #get cell types 
    cellTypes <- cellAnnots[[cellTypeCol]]
    #assign cell name to type
    names(cellTypes) <- cellAnnots[[cellNameCol]]
    
    if(is.null(cellTypes)){
        stop("cellAnnots and/or cellTypeCol arguments are incorrectly formatted. 
                 cellAnnots should be a data frame, and cellTypeCol should give the 
                 name of the column holding each cell's cell type.")
    }
    
    rm(cellAnnots)
    
    # mtx <- as.data.frame(mtx)
    
    if(!any(names(cellTypes) %in% colnames(mtx)) & 
       any(names(cellTypes) %in% rownames(mtx))){
        print("Transposing Matrix")
        mtx<- t(mtx)
    }
    
    if(!any(names(cellTypes) %in% colnames(mtx))){
        stop(paste("cellNameCol names does not match count matrix column names", 
                   "matrix cell names:", colnames(mtx)[1], "annots cell names:", names(cellTypes)[1]))
    }else if(!all(names(cellTypes) %in% colnames(mtx))){
        missing <- length(which(!names(cellTypes) %in% colnames(mtx)))
        warning(paste("not all cellNameCol names are in count matrix;", missing, "cells are missing"))
    }
    
    if(discardCellTypes == TRUE){
        #remove cells with no cell type assignment
        w2rm <- which(is.na(cellTypes) | tolower(cellTypes) %in% c("unspecified", "unknown", "not available")) 
        w2rm <- unique(c(w2rm, grep(pattern = "doublet|dividing|low q|filtered|mitotic", x = tolower(cellTypes))))
        if(length(w2rm) > 0){
            cellTypes <- cellTypes[-w2rm]
        }
    }
    
    #normalize data if necessary 
    if(normalize == TRUE){
        print("Normalizing Matrix")
        med <- median(Matrix::colSums(mtx))
        cols <- colnames(mtx)
        rows <- rownames(mtx)
        mtx <- Matrix::Matrix(sweep(mtx, 2, Matrix::colSums(mtx), "/") * med, 
                              sparse = TRUE) 
        
        colnames(mtx) <- cols
        rownames(mtx) <- rows
        
        rm(cols,rows)
    }
    
    atlas <- NULL
    
    #get all unique cell types
    CTs <- unique(cellTypes)
    
    print("Creating Atlas")
    
    #change to apply() if bioconductor requires it
    for(i in CTs){
        #print log of progress
        print(paste(which(CTs == i), "/", length(CTs), ":", i))
        
        #get cell names for this cell type
        cellsType <- names(cellTypes)[which(cellTypes == i)]
        #confirm cells are in matrix
        cellsType <- cellsType[which(cellsType %in% colnames(mtx))]
        
        if(length(cellsType) > minCellNum){
            
            if(length(cellsType) > 1){
                #remove cells with low gene expression
                cellsType <- cellsType[which(Matrix::colSums(mtx[,cellsType] > 0) > minGenes)]
            }else{
                cellsType <- cellsType[which(sum(mtx[,cellsType] > 0) > minGenes)]
            }
            
            if(length(cellsType) > minCellNum){
                #get average expression if there are enough cells for cell type
                if(length(cellsType) > minCellNum & length(cellsType) != 1){
                    atlas <- as.data.frame(cbind(atlas, Matrix::rowMeans(mtx[,cellsType], na.rm = TRUE)))
                    colnames(atlas)[ncol(atlas)] <- i
                }else{
                    atlas <- as.data.frame(cbind(atlas, mtx[,cellsType]))
                    colnames(atlas)[ncol(atlas)] <- i
                }
            }else{
                warning(paste("\n", i, "was dropped from matrix because it didn't have enough viable cells based on current filtering thresholds. 
                                        If this cell type is necessary consider changing minCellNum or minGenes\n"))
            }
        }else{
            warning(paste("\n", i, "was dropped from matrix because it didn't have enough viable cells based on current filtering thresholds. 
                                        If this cell type is necessary consider changing minCellNum or minGenes\n"))
        }
    }
    
    rm(cellTypes)
    
    numCellTypesExpr <- 1
    
    #subset to genes expressed in at least a user defined number of cell type(s)
    if(ncol(atlas) == 1){
        w2kp <- which(Matrix::rowSums(atlas > 0) >= numCellTypesExpr)
        cols <- colnames(atlas)
        rows <- rownames(atlas)[w2kp]
        
        atlas <- as.matrix(atlas[w2kp,])
        
        colnames(atlas) <- cols
        rownames(atlas) <- rows
        
        rm(cols, rows, w2kp)
    }else{
        atlas <- atlas[which(Matrix::rowSums(atlas > 0) >= numCellTypesExpr),]
    }
    
    #scale data
    atlas <- atlas * scalingFactor
    
    if(!is.null(geneList)){
        if(any(geneList %in% rownames(atlas))){
            #subset to genes in panel
            atlas <- atlas[rownames(atlas) %in% geneList,]
        }else{
            warning("geneList genes do not match genes in matrix, no filtering done")
        }
    }
    
    #ensure cell types don't contain ","
    colnames(atlas) <- gsub(pattern = ",", replacement = "-", x = colnames(atlas))
    
    if(!is.null(outDir)){
        #write profile matrix
        write.table(atlas, file = paste0(outDir, "/", matrixName, "_profileMatrix.csv"), 
                    row.names = TRUE, col.names = NA, quote = FALSE, sep = ",")
    }
    
    return(as.matrix(atlas))
}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----installation, eval=FALSE-------------------------------------------------
# if(!requireNamespace("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# BiocManager::install("SpatialDecon")
# BiocManager::install("GeoMxTools")

## ----setup--------------------------------------------------------------------
library(SpatialDecon)
library(GeomxTools)

## ----loaddata-----------------------------------------------------------------
data("nsclc")

nsclc <- updateGeoMxSet(nsclc)

dim(nsclc)
head(pData(nsclc))
nsclc@assayData$exprs[seq_len(5), seq_len(5)]

# better segment names:
sampleNames(nsclc) <-  
  paste0(nsclc$ROI, nsclc$AOI.name)

#This dataset is already on Target level (genes instead of probeIDs in rows), 
# just need to reflect that in object. 
featureType(nsclc) <- "Target"

## ----derivebg-----------------------------------------------------------------
bg = derive_GeoMx_background(norm = nsclc@assayData$exprs_norm,
                             probepool = fData(nsclc)$Module,
                             negnames = c("NegProbe-CTP01", "NegProbe-Kilo"))

## ----showsafetme, fig.height=5, fig.width=10, fig.cap = "The safeTME cell profile matrix"----
data("safeTME")
data("safeTME.matches")

signif(safeTME[seq_len(3), seq_len(3)], 2)

heatmap(sweep(safeTME, 1, apply(safeTME, 1, max), "/"),
        labRow = NA, margins = c(10, 5))


## ----downloadmatrix, fig.height=7, fig.width=10, fig.cap = "The Mouse Spleen profile matrix", eval=T----
mousespleen <- download_profile_matrix(species = "Mouse",
                                       age_group = "Adult", 
                                       matrixname = "Spleen_MCA")
dim(mousespleen)

mousespleen[1:4,1:4]

head(cellGroups)

metadata

heatmap(sweep(mousespleen, 1, apply(mousespleen, 1, max), "/"),
        labRow = NA, margins = c(10, 5), cexCol = 0.7)


## ----single cell data---------------------------------------------------------
data("mini_singleCell_dataset")

mini_singleCell_dataset$mtx@Dim # genes x cells

as.matrix(mini_singleCell_dataset$mtx)[1:4,1:4]

head(mini_singleCell_dataset$annots)

table(mini_singleCell_dataset$annots$LabeledCellType)


## ----creatematrix, fig.height=7, fig.width=10, fig.cap = "Custom profile matrix"----
custom_mtx <- create_profile_matrix(mtx = mini_singleCell_dataset$mtx,            # cell x gene count matrix
                                    cellAnnots = mini_singleCell_dataset$annots,  # cell annotations with cell type and cell name as columns 
                                    cellTypeCol = "LabeledCellType",  # column containing cell type
                                    cellNameCol = "CellID",           # column containing cell ID/name
                                    matrixName = "custom_mini_colon", # name of final profile matrix
                                    outDir = NULL,                    # path to desired output directory, set to NULL if matrix should not be written
                                    normalize = FALSE,                # Should data be normalized? 
                                    minCellNum = 5,                   # minimum number of cells of one type needed to create profile, exclusive
                                    minGenes = 10,                    # minimum number of genes expressed in a cell, exclusive
                                    scalingFactor = 5,                # what should all values be multiplied by for final matrix
                                    discardCellTypes = TRUE)          # should cell types be filtered for types like mitotic, doublet, low quality, unknown, etc.

head(custom_mtx)

heatmap(sweep(custom_mtx, 1, apply(custom_mtx, 1, max), "/"),
        labRow = NA, margins = c(10, 5), cexCol = 0.7)


## ----runiss-------------------------------------------------------------------
res = runspatialdecon(object = nsclc,
                      norm_elt = "exprs_norm",
                      raw_elt = "exprs",
                      X = safeTME,
                      align_genes = TRUE)

str(pData(res))
names(res@assayData)

## ----plotissres, fig.height = 5, fig.width = 8, fig.cap = "Cell abundance estimates"----
heatmap(t(res$beta), cexCol = 0.5, cexRow = 0.7, margins = c(10,7))

## ----showmatches--------------------------------------------------------------
str(safeTME.matches)

## ----runisstils---------------------------------------------------------------
# vector identifying pure tumor segments:
nsclc$istumor = nsclc$AOI.name == "Tumor"

# run spatialdecon with all the bells and whistles:
restils = runspatialdecon(object = nsclc,
                          norm_elt = "exprs_norm",                # normalized data
                          raw_elt = "exprs",                      # expected background counts for every data point in norm
                          X = safeTME,                            # safeTME matrix, used by default
                          cellmerges = safeTME.matches,           # safeTME.matches object, used by default
                          cell_counts = nsclc$nuclei,      # nuclei counts, used to estimate total cells
                          is_pure_tumor = nsclc$istumor,   # identities of the Tumor segments/observations
                          n_tumor_clusters = 5)                   # how many distinct tumor profiles to append to safeTME

str(pData(restils))
names(restils@assayData)
str(restils@experimentData@other)

## ----shownewX, fig.height=5, fig.width=8, fig.cap = "safeTME merged with newly-derived tumor profiles"----
heatmap(sweep(restils@experimentData@other$SpatialDeconMatrix, 1, apply(restils@experimentData@other$SpatialDeconMatrix, 1, max), "/"),
         labRow = NA, margins = c(10, 5))


## ----barplot, fig.width=9, fig.height=6, fig.cap="Barplots of TIL abundance"----
# For reference, show the TILs color data object used by the plotting functions 
# when safeTME has been used:
data("cellcols")
cellcols

# show just the TME segments, since that's where the immune cells are:
o = hclust(dist(t(restils$cell.counts$cell.counts)))$order
layout(mat = (matrix(c(1, 2), 1)), widths = c(7, 3))
TIL_barplot(t(restils$cell.counts$cell.counts[, o]), draw_legend = TRUE, 
            cex.names = 0.5)
# or the proportions of cells:
temp = replace(restils$prop_of_nontumor, is.na(restils$prop_of_nontumor), 0)
o = hclust(dist(temp[restils$AOI.name == "TME",]))$order
TIL_barplot(t(restils$prop_of_nontumor[restils$AOI.name == "TME",])[, o], 
            draw_legend = TRUE, cex.names = 0.75)


## ----florets, fig.width=8, fig.height=6, fig.cap = "TIL abundance plotted on PC space"----
# PCA of the normalized data:
pc = prcomp(t(log2(pmax(nsclc@assayData$exprs_norm, 1))))$x[, c(1, 2)]

# run florets function:
par(mar = c(5,5,1,1))
layout(mat = (matrix(c(1, 2), 1)), widths = c(6, 2))
florets(x = pc[, 1], y = pc[, 2],
        b = t(restils$beta), cex = .5,
        xlab = "PC1", ylab = "PC2")
par(mar = c(0,0,0,0))
frame()
legend("center", fill = cellcols[colnames(restils$beta)], 
       legend = colnames(restils$beta), cex = 0.7)

## ----collapse, fig.width=5, fig.height=5, fig.cap="Cell abundance estimates with related cell types collapsed"----
matching = list()
matching$myeloid = c( "macrophages", "monocytes", "mDCs")
matching$T.NK = c("CD4.T.cells","CD8.T.cells", "Treg", "NK")
matching$B = c("B")
matching$mast = c("mast")
matching$neutrophils = c("neutrophils")
matching$stroma = c("endothelial.cells", "fibroblasts")


collapsed = runCollapseCellTypes(object = restils, 
                                 matching = matching)

heatmap(t(collapsed$beta), cexRow = 0.85, cexCol = 0.75)

## ----appendtumor, fig.width = 10, fig.height= 5, fig.cap = "safeTME merged with newly-derived tumor profiles"----
pure.tumor.ids = res$AOI.name == "Tumor"
safeTME.with.tumor = runMergeTumorIntoX(object = nsclc,
                                        norm_elt = "exprs_norm", 
                                        pure_tumor_ids = pure.tumor.ids,
                                        X = safeTME,
                                        K = 3) 

heatmap(sweep(safeTME.with.tumor, 1, apply(safeTME.with.tumor, 1, max), "/"),
        labRow = NA, margins = c(10, 5))


## ----reverse, fig.height=6, fig.width=6, fig.cap="Residuals from reverseDecon"----
rdecon = runReverseDecon(object = nsclc,
                         norm_elt = "exprs_norm",
                         beta = res$beta)
str(fData(rdecon))
names(rdecon@assayData)

# look at the residuals:
heatmap(pmax(pmin(rdecon@assayData$resids, 2), -2))

## ----reverse2, fig.height=6, fig.width=6, fig.cap="Genes' dependency on cell mixing"----
# look at the two metrics of goodness-of-fit:
plot(fData(rdecon)$cors, fData(rdecon)$resid.sd, col = 0)
showgenes = c("CXCL14", "LYZ", "FOXP3")
text(fData(rdecon)$cors[!rownames(fData(rdecon)) %in% showgenes], 
     fData(rdecon)$resid.sd[!rownames(fData(rdecon)) %in% showgenes], 
     setdiff(rownames(fData(rdecon)), showgenes), cex = 0.5)
text(fData(rdecon)$cors[rownames(fData(rdecon)) %in% showgenes], fData(rdecon)$resid.sd[rownames(fData(rdecon)) %in% showgenes], 
     showgenes, cex = 0.75, col = 2)


## ----sessioninfo--------------------------------------------------------------
sessionInfo()

# SpatialDecon: mixed cell deconvolution for spatial and/or bulk gene expression
# data
# Copyright (C) 2020, NanoString Technologies, Inc.
#    This program is free software: you can redistribute it and/or modify it
#    under the terms of the GNU General Public License as published by the Free
#    Software Foundation, either version 3 of the License, or (at your option)
#    any later version.
#    This program is distributed in the hope that it will be useful, but WITHOUT
#    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
#    more details.
#    You should have received a copy of the GNU General Public License along
#    with this program.  If not, see https://www.gnu.org/licenses/.
# Contact us:
# NanoString Technologies, Inc.
# 530 Fairview Avenue N
# Seattle, WA 98109
# Tel: (888) 358-6266
# pdanaher@nanostring.com



#' Compute gene weights
#'
#' Compute gene weights from pre-defined biological noise estimates and from
#'  raw count/ error-model-defined technical noise estimates
#' @param norm Matrix of data used in decon. Used to define the gene list and
#' the
#' shape of the output "wts" matrix.
#' @param raw Matrix of raw data. If provided, used to define technical noise
#' @param error.model Which error model to use. Defaults to "dsp"
#' @param weight.by.TIL.resid.sd If TRUE, then genes are weighted in part based
#' on their
#'  biological variability as estimated by their residual SD from decon
#'   performed on TCGA.
#' @return A matrix of weights, in the same dimension as norm
#' @keywords internal
#' @noRd
#' @import utils
#' @importFrom utils data
deriveWeights <- function(norm, raw = NULL, error.model = "dsp",
                          weight.by.TIL.resid.sd = FALSE) {
    
    # get tech SDs if raw data provided:
    if (length(raw) == 0) {
        sds.tech <- matrix(0.1, nrow(raw), ncol(raw), dimnames = dimnames(raw))
    }
    if (length(raw) > 0) {
        sds.tech <- runErrorModel(
            counts = raw,
            platform = "dsp"
        )
    }
    
    # if the mean.resid.sd vector (which defines genes' biological SD) is in
    # the environment, get biological noise:
    if (!weight.by.TIL.resid.sd) {
        sds.bio <- matrix(0.1, nrow(raw), ncol(raw), dimnames = dimnames(raw))
    }
    if (weight.by.TIL.resid.sd) {
        utils::data("mean.resid.sd", envir = environment())
        sds.bio <- matrix(NA, nrow(raw), ncol(raw), dimnames = dimnames(raw))
        for (gene in intersect(
            names(SpatialDecon::mean.resid.sd),
            rownames(sds.bio)
        )) {
            sds.bio[gene, ] <- SpatialDecon::mean.resid.sd[gene]
        }
        sds.bio <- replace(sds.bio, is.na(sds.bio), mean(sds.bio, na.rm = TRUE))
    }
    
    # define total SD, and invert to get weights
    sds.tot <- sqrt(sds.tech^2 + sds.bio^2)
    wts <- 1 / sds.tech
    return(wts)
}

# SpatialDecon: mixed cell deconvolution for spatial and/or bulk gene expression
# data
# Copyright (C) 2020, NanoString Technologies, Inc.
#    This program is free software: you can redistribute it and/or modify it
#    under the terms of the GNU General Public License as published by the Free
#    Software Foundation, either version 3 of the License, or (at your option)
#    any later version.
#    This program is distributed in the hope that it will be useful, but WITHOUT
#    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
#    more details.
#    You should have received a copy of the GNU General Public License along
#    with this program.  If not, see https://www.gnu.org/licenses/.
# Contact us:
# NanoString Technologies, Inc.
# 530 Fairview Avenue N
# Seattle, WA 98109
# Tel: (888) 358-6266
# pdanaher@nanostring.com



#' Apply error model to estimate technical SD from raw counts
#'
#' Based on raw counts, uses past data to estimate each raw count's log-scale
#' SD from technical noise.
#' Specifies different error models for different platforms.
#'
#' @param counts vector or matrix of raw counts
#' @param platform String specifying which platform was used to create
#' "rawCounts". Default to "dsp", for digital spatial profiler/ aka GeoMx.
#'  Other options include "ncounter", "rsem", "quantile", and "st" for spatial transcriptomics/visium.
#' @return a matrix of log2-scale SDs
#' @examples 
#' library(GeomxTools)
#' datadir <- system.file("extdata", "DSP_NGS_Example_Data", package = "GeomxTools")
#' demoData <- readRDS(file.path(datadir, "/demoData.rds"))
#' 
#' demoData <- shiftCountsOne(demoData)
#' target_demoData <- aggregateCounts(demoData)
#' 
#' sd_from_noise <- runErrorModel(counts = exprs(target_demoData), platform = "dsp") 
#' wts <- 1 / sd_from_noise
#' @export

runErrorModel <- function(counts, platform = "general") {
    if (platform == "ncounter") {
        sds <- counts * 0 + 0.1
        sds <- replace(sds, counts < 200, 0.2)
        sds <- replace(sds, counts < 100, 0.3)
        sds <- replace(sds, counts < 75, 0.4)
        sds <- replace(sds, counts < 50, 0.5)
        sds <- replace(sds, counts < 40, 0.7)
        sds <- replace(sds, counts < 30, 1)
        sds <- replace(sds, counts < 20, 3)
    }
    
    
    if (platform == "rsem") {
        sds <- counts * 0 + 0.5930982
        sds <- replace(sds, log2(counts) < 9.5, 0.6458475)
        sds <- replace(sds, log2(counts) < 8.5, 0.7847597)
        sds <- replace(sds, log2(counts) < 7.5, 1.0576471)
        sds <- replace(sds, log2(counts) < 6.5, 1.2990917)
        sds <- replace(sds, log2(counts) < 5.5, 1.5061735)
        sds <- replace(sds, log2(counts) < 4.5, 1.6930872)
        sds <- replace(sds, log2(counts) < 3.5, 1.7894239)
    }
    
    if (platform == "dsp") {
        predictsd.dsp <- function(rawcounts) {
            m <- log2(pmax(rawcounts, 1e-3))
            meanvec <- c(-1e-6, 2, 2.5, 3, 3.5, 4, 4.5, 5, 5.5, 6, Inf)
            sdvec <- c(
                1.5, 1.383, 1.191, 0.800, 0.48, 0.301, 0.301,
                0.301, 0.263, 0.235, 0.235
            )
            
            s <- replace(m, TRUE, sdvec[1])
            for (i in seq_len(length(meanvec) - 1)) {
                s <- replace(s, m >= meanvec[i], sdvec[i + 1])
            }
            return(s)
        }
        
        if (is.vector(counts)) {
            sds <- vapply(
                X = counts,
                FUN = predictsd.dsp,
                FUN.VALUE = numeric(length(counts))
            )
        }
        if (is.matrix(counts)) {
            sds <- predictsd.dsp(counts)
        }
    }
    
    if (platform == "st") {
        # assume poisson error:
        sds <- counts * 0 + 03
        sds <- replace(sds, counts < 500, 0.045)
        sds <- replace(sds, counts < 200, 0.07)
        sds <- replace(sds, counts < 100, 0.1)
        sds <- replace(sds, counts < 50, 0.14)
        sds <- replace(sds, counts < 30, 0.18)
        sds <- replace(sds, counts < 20, 0.23)
        sds <- replace(sds, counts < 15, 0.27)
        sds <- replace(sds, counts < 10, 0.35)
        sds <- replace(sds, counts < 5, 0.61)
        sds <- replace(sds, counts < 2, 1.15)
        sds <- replace(sds, counts < 1, 1.33)
    }
    
    if (platform == "quantile") {
        if (is.vector(counts)) {
            quantile <- rank(counts) / length(counts)
        }
        if (is.matrix(counts)) {
            quantile <- matrix(rank(counts), nrow(counts)) / length(counts)
        }
        
        sds <- quantile * 0 + 0.1
        sds <- replace(sds, quantile < 0.2, 0.2)
        sds <- replace(sds, quantile < 0.15, 0.3)
        sds <- replace(sds, quantile < 0.1, 0.4)
        sds <- replace(sds, quantile < 0.05, 0.5)
        sds <- replace(sds, quantile < 0.01, 1)
    }
    return(sds)
}

# SpatialDecon: mixed cell deconvolution for spatial and/or bulk gene expression
# data
# Copyright (C) 2020, NanoString Technologies, Inc.
#    This program is free software: you can redistribute it and/or modify it
#    under the terms of the GNU General Public License as published by the Free
#    Software Foundation, either version 3 of the License, or (at your option)
#    any later version.
#    This program is distributed in the hope that it will be useful, but WITHOUT
#    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
#    more details.
#    You should have received a copy of the GNU General Public License along
#    with this program.  If not, see https://www.gnu.org/licenses/.
# Contact us:
# NanoString Technologies, Inc.
# 530 Fairview Avenue N
# Seattle, WA 98109
# Tel: (888) 358-6266
# pdanaher@nanostring.com



#' SpatialDecon mixed cell deconvolution algorithm
#'
#' Runs the generic SpatialDecon decon workflow, including:
#' \enumerate{
#' \item run deconvolution once
#' \item remove poorly-fit genes from first round of decon
#' \item re-run decon with cleaned-up gene set
#' \item compute p-values
#' }
#'
#' @param Y p-length expression vector or p * N expression matrix - the actual
#' (linear-scale) data
#' @param X p * K Training matrix.
#' @param bg Expected background counts. Provide a scalar to apply to all
#'  data points, or
#'  else a matrix/vector aligning with Y to provide more nuanced expected
#'   background.
#' @param weights The same as the weights argument used by lm
#' @param resid_thresh A scalar, sets a threshold on how extreme individual
#' data points' values
#'  can be (in log2 units) before getting flagged as outliers and set to NA.
#' @param lower_thresh A scalar. Before log2-scale residuals are calculated,
#' both observed and fitted
#'  values get thresholded up to this value. Prevents log2-scale residuals from
#'  becoming extreme in
#'  points near zero.
#' @param align_genes Logical. If TRUE, then Y, X, bg, and wts are row-aligned
#' by shared genes.
#' @param maxit Maximum number of iterations. Default 1000.
#' @return a list:
#' \itemize{
#' \item beta: matrix of cell abundance estimates, cells in rows and
#' observations in columns
#' \item sigmas: covariance matrices of each observation's beta estimates
#' \item p: matrix of p-values for H0: beta == 0
#' \item t: matrix of t-statistics for H0: beta == 0
#' \item se: matrix of standard errors of beta values
#' \item resids: a matrix of residuals from the model fit.
#' (log2(pmax(y, lower_thresh)) - log2(pmax(xb, lower_thresh))).
#' }
#' @import stats
#' @importFrom stats pnorm
#' @keywords internal
#' @noRd
algorithm2 <- function(Y, X, bg = 0, weights = NULL,
                       resid_thresh = 3, lower_thresh = 0.5,
                       align_genes = TRUE, maxit = 1000) {
    
    # align genes:
    if (align_genes) {
        sharedgenes <- intersect(rownames(X), rownames(Y))
        Y <- Y[sharedgenes, ]
        X <- X[sharedgenes, ]
        if (is.matrix(bg)) {
            bg <- bg[sharedgenes, ]
        }
        if (is.matrix(weights)) {
            weights <- weights[sharedgenes, ]
        }
    }
    
    # format the data nicely:
    tidied <- tidy_X_and_Y(X, Y)
    X <- tidied$X
    Y <- tidied$Y
    if ((length(bg) > 0) & (is.vector(bg))) {
        bg <- matrix(bg, nrow = length(bg))
    }
    
    # select an epsilon (lowest non-zero value to use)
    epsilon <- min(Y[(Y > 0) & !is.na(Y)])
    
    
    # initial run to look for outliers:
    out0 <- deconLNR(
        Y = Y, X = X, bg = bg, weights = weights, epsilon = epsilon,
        maxit = maxit
    )
    # also get yhat and resids:
    out0$yhat <- X %*% out0$beta + bg
    out0$resids <- log2(pmax(Y, lower_thresh)) -
        log2(pmax(out0$yhat, lower_thresh))
    
    # ID bad genes:
    outliers <- flagOutliers(
        Y = Y,
        yhat = out0$yhat,
        wts = weights,
        resids = out0$resids,
        resid_thresh = resid_thresh
    )
    
    # remove outlier data points:
    Y.nooutliers <- replace(Y, outliers, NA)
    
    # re-run decon without outliers:
    out <- deconLNR(
        Y = Y.nooutliers,
        X = X,
        bg = bg,
        weights = weights,
        epsilon = epsilon
    )
    out$yhat <- X %*% out$beta + bg
    out$resids <- log2(pmax(Y.nooutliers, 0.5)) - log2(pmax(out$yhat, 0.5))
    
    # compute p-values
    tempbeta <- out$beta
    tempse <- tempp <- tempt <- tempbeta * NA
    for (i in seq_len(ncol(tempse))) {
        tempse[, i] <- suppressWarnings(sqrt(diag(out$sigmas[, , i])))
    }
    tempt <- (tempbeta / tempse)
    tempp <- 2 * (1 - stats::pnorm(tempt))
    out$p <- tempp
    out$t <- tempt
    out$se <- tempse
    
    # structure of output: beta, hessians, yhat, resids
    return(out)
}



#' Function to format Y, X inputs for decon
#'
#' Takes user-supplied X and Y, checks for accuracy, aligns by dimnames, adds
#' dimnames if missing
#'
#' @param X X matrix
#' @param Y Data matrix
#' @return X and Y, both formatted as matrices, with full dimnames and aligned
#' to each other by dimname
#' @keywords internal
#' @noRd
tidy_X_and_Y <- function(X, Y) {
    
    # format as matrices:
    Ynew <- Y
    if (is.vector(Y)) {
        Ynew <- matrix(Y, nrow = length(Y), dimnames = list(names(Y), "y"))
    }
    Xnew <- X
    
    # check alignment:
    if (!identical(rownames(Y), rownames(X))) {
        warning("Rows (genes) of X and Y are mis-aligned.")
    }
    out <- list(X = Xnew, Y = Ynew)
}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----installation, eval=FALSE-------------------------------------------------
# if(!requireNamespace("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# BiocManager::install("SpatialDecon")
# 

## ----setup--------------------------------------------------------------------
library(SpatialDecon)

## ----loaddata-----------------------------------------------------------------
data("mini_geomx_dataset")
norm = mini_geomx_dataset$normalized
raw = mini_geomx_dataset$raw
annot = mini_geomx_dataset$annot
dim(raw)
head(annot)
raw[seq_len(5), seq_len(5)]

# better segment names:
colnames(norm) = colnames(raw) = rownames(annot) = 
  paste0(annot$ROI, annot$AOI.name)


## ----estimateBG---------------------------------------------------------------
# use the NegProbe to estimate per-observation background
per.observation.mean.neg = norm["NegProbe", ]
# and define a background matrix in which each column (observation) is the
# appropriate value of per-observation background:
bg = sweep(norm * 0, 2, per.observation.mean.neg, "+")
dim(bg)


## ----derivebg-----------------------------------------------------------------
bg2 = derive_GeoMx_background(norm = norm,
                             probepool = rep(1, nrow(norm)),
                             negnames = "NegProbe")

## ----showsafetme, fig.height=5, fig.width=10, fig.cap = "The safeTME cell profile matrix"----
data("safeTME")
data("safeTME.matches")

signif(safeTME[seq_len(3), seq_len(3)], 2)

heatmap(sweep(safeTME, 1, apply(safeTME, 1, max), "/"),
        labRow = NA, margins = c(10, 5))


## ----downloadmatrix, fig.height=7, fig.width=10, fig.cap = "The Mouse Spleen profile matrix", eval=T----
mousespleen <- download_profile_matrix(species = "Mouse",
                                       age_group = "Adult", 
                                       matrixname = "Spleen_MCA")
dim(mousespleen)

mousespleen[1:4,1:4]

head(cellGroups)

metadata

heatmap(sweep(mousespleen, 1, apply(mousespleen, 1, max), "/"),
        labRow = NA, margins = c(10, 5), cexCol = 0.7)


## ----single cell data---------------------------------------------------------
data("mini_singleCell_dataset")

mini_singleCell_dataset$mtx@Dim # genes x cells

as.matrix(mini_singleCell_dataset$mtx)[1:4,1:4]

head(mini_singleCell_dataset$annots)

table(mini_singleCell_dataset$annots$LabeledCellType)


## ----creatematrix, fig.height=7, fig.width=10, fig.cap = "Custom profile matrix"----
custom_mtx <- create_profile_matrix(mtx = mini_singleCell_dataset$mtx,            # cell x gene count matrix
                                    cellAnnots = mini_singleCell_dataset$annots,  # cell annotations with cell type and cell name as columns 
                                    cellTypeCol = "LabeledCellType",  # column containing cell type
                                    cellNameCol = "CellID",           # column containing cell ID/name
                                    matrixName = "custom_mini_colon", # name of final profile matrix
                                    outDir = NULL,                    # path to desired output directory, set to NULL if matrix should not be written
                                    normalize = FALSE,                # Should data be normalized? 
                                    minCellNum = 5,                   # minimum number of cells of one type needed to create profile, exclusive
                                    minGenes = 10,                    # minimum number of genes expressed in a cell, exclusive
                                    scalingFactor = 5,                # what should all values be multiplied by for final matrix
                                    discardCellTypes = TRUE)          # should cell types be filtered for types like mitotic, doublet, low quality, unknown, etc.

head(custom_mtx)

heatmap(sweep(custom_mtx, 1, apply(custom_mtx, 1, max), "/"),
        labRow = NA, margins = c(10, 5), cexCol = 0.7)


## ----createSeuratmatrix-------------------------------------------------------
library(SeuratObject)
library(Seurat)

data("mini_singleCell_dataset")

rownames(mini_singleCell_dataset$annots) <- mini_singleCell_dataset$annots$CellID

seuratObject <- CreateSeuratObject(counts = mini_singleCell_dataset$mtx, 
                                   meta.data = mini_singleCell_dataset$annots)
Idents(seuratObject) <- seuratObject$LabeledCellType

rm(mini_singleCell_dataset)

annots <- data.frame(cbind(cellType=as.character(Idents(seuratObject)), 
                           cellID=names(Idents(seuratObject))))

custom_mtx_seurat <- create_profile_matrix(mtx = Seurat::GetAssayData(object = seuratObject, 
                                                                      assay = "RNA", 
                                                                      layer = "counts"), 
                                           cellAnnots = annots, 
                                           cellTypeCol = "cellType", 
                                           cellNameCol = "cellID", 
                                           matrixName = "custom_mini_colon",
                                           outDir = NULL, 
                                           normalize = FALSE, 
                                           minCellNum = 5, 
                                           minGenes = 10)

head(custom_mtx_seurat)

paste("custom_mtx and custom_mtx_seurat are identical", all(custom_mtx == custom_mtx_seurat))

## ----runiss-------------------------------------------------------------------
res = spatialdecon(norm = norm,
                   bg = bg,
                   X = safeTME,
                   align_genes = TRUE)
str(res)

## ----plotissres, fig.height = 5, fig.width = 8, fig.cap = "Cell abundance estimates"----
heatmap(res$beta, cexCol = 0.5, cexRow = 0.7, margins = c(10,7))

## ----showmatches--------------------------------------------------------------
str(safeTME.matches)

## ----runisstils---------------------------------------------------------------
# vector identifying pure tumor segments:
annot$istumor = (annot$AOI.name == "Tumor")

# run spatialdecon with all the bells and whistles:
restils = spatialdecon(norm = norm,                     # normalized data
                       raw = raw,                       # raw data, used to down-weight low-count observations 
                       bg = bg,                         # expected background counts for every data point in norm
                       X = safeTME,                     # safeTME matrix, used by default
                       cellmerges = safeTME.matches,   # safeTME.matches object, used by default
                       cell_counts = annot$nuclei,      # nuclei counts, used to estimate total cells
                       is_pure_tumor = annot$istumor,   # identities of the Tumor segments/observations
                       n_tumor_clusters = 5)            # how many distinct tumor profiles to append to safeTME

str(restils)

## ----shownewX, fig.height=5, fig.width=8, fig.cap = "safeTME merged with newly-derived tumor profiles"----
heatmap(sweep(restils$X, 1, apply(restils$X, 1, max), "/"),
         labRow = NA, margins = c(10, 5))


## ----compareresults, fig.height=6, fig.width=8, fig.cap = "Cell abundance estimates with and without modelling tumor profiles"----
par(mfrow = c(2, 3))
par(mar = c(5,7,2,1))
for (i in seq_len(6)) {
  cell = rownames(res$beta)[i]
  plot(res$beta[cell, ], restils$beta.granular[cell, ],
       xlab = paste0(cell, " score under basic setting"), 
       ylab = paste0(cell, " score when tumor\ncells are modelled"), 
       pch = 16,
       col = c(rgb(0,0,1,0.5), rgb(1,0,0,0.5))[1 + annot$istumor],
       xlim = range(c(res$beta[cell, ], restils$beta.granular[cell, ])),
       ylim = range(c(res$beta[cell, ], restils$beta.granular[cell, ])))
  abline(0,1)
  if (i == 1) {
    legend("topleft", pch = 16, col = c(rgb(0,0,1,0.5), rgb(1,0,0,0.5)),
           legend = c("microenv.", "tumor"))
  }
}


## ----barplot, fig.width=9, fig.height=6, fig.cap="Barplots of TIL abundance"----
# For reference, show the TILs color data object used by the plotting functions 
# when safeTME has been used:
data("cellcols")
cellcols

# show just the TME segments, since that's where the immune cells are:
layout(mat = (matrix(c(1, 2), 1)), widths = c(7, 3))
TIL_barplot(restils$cell.counts$cell.counts, draw_legend = TRUE, 
            cex.names = 0.5)
# or the proportions of cells:
TIL_barplot(restils$prop_of_nontumor[, annot$AOI.name == "TME"], 
            draw_legend = TRUE, cex.names = 0.75)


## ----florets, fig.width=8, fig.height=6, fig.cap = "TIL abundance plotted on PC space"----
# PCA of the normalized data:
pc = prcomp(t(log2(pmax(norm, 1))))$x[, c(1, 2)]

# run florets function:
par(mar = c(5,5,1,1))
layout(mat = (matrix(c(1, 2), 1)), widths = c(6, 2))
florets(x = pc[, 1], y = pc[, 2],
        b = restils$beta, cex = 2,
        xlab = "PC1", ylab = "PC2")
par(mar = c(0,0,0,0))
frame()
legend("center", fill = cellcols[rownames(restils$beta)], 
       legend = rownames(restils$beta), cex = 0.7)

## ----collapse, fig.width=5, fig.height=5, fig.cap="Cell abundance estimates with related cell types collapsed"----
matching = list()
matching$myeloid = c( "macrophages", "monocytes", "mDCs")
matching$T.NK = c("CD4.T.cells","CD8.T.cells", "Treg", "NK")
matching$B = c("B")
matching$mast = c("mast")
matching$neutrophils = c("neutrophils")
matching$stroma = c("endothelial.cells", "fibroblasts")


collapsed = collapseCellTypes(fit = restils, 
                              matching = matching)

heatmap(collapsed$beta, cexRow = 0.85, cexCol = 0.75)

## ----appendtumor, fig.width = 10, fig.height= 5, fig.cap = "safeTME merged with newly-derived tumor profiles"----
pure.tumor.ids = annot$AOI.name == "Tumor"
safeTME.with.tumor = mergeTumorIntoX(norm = norm, 
                                     bg = bg, 
                                     pure_tumor_ids = pure.tumor.ids, 
                                     X = safeTME, 
                                     K = 3) 

heatmap(sweep(safeTME.with.tumor, 1, apply(safeTME.with.tumor, 1, max), "/"),
        labRow = NA, margins = c(10, 5))


## ----reverse, fig.height=6, fig.width=6, fig.cap="Residuals from reverseDecon"----
rdecon = reverseDecon(norm = norm,
                      beta = res$beta)
str(rdecon)

# look at the residuals:
heatmap(pmax(pmin(rdecon$resids, 2), -2))

## ----reverse2, fig.height=6, fig.width=6, fig.cap="Genes' dependency on cell mixing"----
# look at the two metrics of goodness-of-fit:
plot(rdecon$cors, rdecon$resid.sd, col = 0)
showgenes = c("CXCL14", "LYZ", "NKG7")
text(rdecon$cors[setdiff(names(rdecon$cors), showgenes)], 
     rdecon$resid.sd[setdiff(names(rdecon$cors), showgenes)], 
     setdiff(names(rdecon$cors), showgenes), cex = 0.5)
text(rdecon$cors[showgenes], rdecon$resid.sd[showgenes], 
     showgenes, cex = 0.75, col = 2)


## ----sessioninfo--------------------------------------------------------------
sessionInfo()

# SpatialDecon: mixed cell deconvolution for spatial and/or bulk gene expression
# data
# Copyright (C) 2020, NanoString Technologies, Inc.
#    This program is free software: you can redistribute it and/or modify it
#    under the terms of the GNU General Public License as published by the Free
#    Software Foundation, either version 3 of the License, or (at your option)
#    any later version.
#    This program is distributed in the hope that it will be useful, but WITHOUT
#    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
#    more details.
#    You should have received a copy of the GNU General Public License along
#    with this program.  If not, see https://www.gnu.org/licenses/.
# Contact us:
# NanoString Technologies, Inc.
# 530 Fairview Avenue N
# Seattle, WA 98109
# Tel: (888) 358-6266
# pdanaher@nanostring.com


#' Convert results from an arbitrary training matrix to our standardized cell
#' types.
#'
#' Takes betas from a decon run, using any cell type names whatsoever, and maps
#'  them back to the
#' "official" cell types we use. Allows for multiple rows of beta to map to the
#'  same official cell type,
#' in which case those rows will be added up.
#'
#' @param beta K * n matrix of estimated beta values (cell type abundances)
#' @param matching A list object holding the mapping from beta's cell names to
#'  official cell names.
#'  See str(safeTME.matches) for an example.
#' @param stat The function used to combine related cell types. Defaults to sum.
#' @param na.rm Whether to ignore NAs while computing stat
#' @param sigma A list of covariance matrices of beta estimates, in the format
#'  output by spatialdecon.
#' @return A list with two elements:
#' \itemize{
#' \item beta: a matrix of cell abundances, with specified cell types added
#' together
#' \item sigma: an array of covariance matrices for each observation's beta
#' vector
#' }
#' @keywords internal
#' @noRd
convertCellTypes <- function(beta, matching, stat = sum,
                             na.rm = FALSE, sigma = NULL) {
    # format matching list as a matrix to take a linear combination of beta:
    A <- matrix(0, length(matching), nrow(beta),
                dimnames = list(names(matching), rownames(beta))
    )
    for (name in names(matching)) {
        cellnames <- matching[[name]]
        A[name, cellnames] <- 1
    }
    
    # apply A transformation to beta:
    beta2 <- A %*% beta
    
    # if Sigma provided, get vcov of beta2:
    if (length(sigma) > 0) {
        if (length(dim(sigma)) == 2) {
            sigma2 <- A %*% sigma %*% t(A)
        }
        if (length(dim(sigma)) == 3) {
            sigma2 <- array(NA,
                            dim = c(nrow(A), nrow(A), dim(sigma)[3]),
                            dimnames = list(rownames(A), rownames(A), dimnames(sigma)[[3]])
            )
            for (i in seq_len(dim(sigma)[3])) {
                sigma2[, , i] <- A %*% sigma[, , i] %*% t(A)
            }
        }
    }
    
    # if no Sigma, just return transformed beta:
    if (length(sigma) == 0) {
        return(beta2)
    }
    # if there is a sigma, return beta and the sigma:
    if (length(sigma) > 0) {
        out <- list(beta = beta2, sigma = sigma2)
        return(out)
    }
}

#' Run MergeTumorIntoX
#' 
#' Runs mergeTumorIntoX from an S4 object
#' @param object An S4 object such as a GeoMxSet object
#' @param ... Arguments passed to mergeTumorIntoX
#' @return updated X matrix with new columns, "tumor.1", "tumor.2", ...
#'

setGeneric("runMergeTumorIntoX", signature = "object",
           function(object, ...) standardGeneric("runMergeTumorIntoX"))

#' Run mergeTumorIntoX on a NanostringGeomxSet object
#'
#' A wrapper for applying mergeTumorIntoX to a NanostringGeomxSet object.
#'
#' @param pure_tumor_ids Vector identifying columns of norm that are pure tumor.
#'                       Can be indices, logicals or column names.
#' @param X The training matrix
#' @param K the number of clusters to fit
#' @param norm_elt norm data element in assayData
#' @importFrom Biobase assayData
#' @examples
#' library(GeomxTools)
#' datadir <- system.file("extdata", "DSP_NGS_Example_Data", package = "GeomxTools")
#' demoData <- readRDS(file.path(datadir, "/demoData.rds"))
#' 
#' demoData <- shiftCountsOne(demoData)
#' target_demoData <- aggregateCounts(demoData)
#' 
#' target_demoData <- normalize(target_demoData, "quant")
#' 
#' data(safeTME)
#' tumor.ids <- as.logical(sample(x = c("TRUE","FALSE"), size = 88, replace = TRUE))
#' safeTME.with.tumor <- runMergeTumorIntoX(object = target_demoData,
#'                                          X = safeTME,
#'                                          K = 3,
#'                                          pure_tumor_ids = tumor.ids,
#'                                          norm_elt = "exprs_norm")
#'
#' @importFrom methods is
#' @export
#' @rdname runMergeTumorIntoX

setMethod("runMergeTumorIntoX", "NanoStringGeoMxSet",
          function(object, X, K = 10, pure_tumor_ids = NULL, norm_elt = NULL){
              
              if(is.null(norm_elt)){
                  stop("norm_elt must be set")
              }
              
              if(is.null(pure_tumor_ids)){
                  stop("pure_tumor_ids must be set")
              }
              
              if (!is.element(norm_elt, names(object@assayData))) {
                  stop(paste(norm_elt, "is not an element in assaysData slot"))
              }
              
              if (!any(pure_tumor_ids %in% Biobase::sampleNames(object)) & 
                  !is(pure_tumor_ids,"logical") & !is(pure_tumor_ids, "integer")) {
                  stop(paste("No pure_tumor_ids match sample names in GeoMxSet object"))
              }
              
              #norm
              norm <- Biobase::assayDataElement(object, elt = norm_elt)
              
              # estimate background
              bg <- derive_GeoMx_background(norm = norm,
                                            # access the probe pool information from the feature metadata
                                            probepool = Biobase::fData(object)$Module,
                                            # access the names of the negative control probes
                                            negnames = Biobase::fData(object)$TargetName[Biobase::fData(object)$Negative])
              
              # run runMergeTumorIntoX:
              result <- mergeTumorIntoX(norm = norm,
                                        bg = bg,
                                        pure_tumor_ids = pure_tumor_ids,
                                        X = X,
                                        K = K)
              
              # # append result to the object
              # # add tumor_res matrix
              # Biobase::fData(object)[["tumor_res"]] <- matrix(NA, nrow = nrow(object), ncol = ncol(result),
              #                                         dimnames = list(Biobase::featureNames(object), colnames(result)))
              # Biobase::fData(object)[["tumor_res"]][rownames(result), ] <- result
              
              return(result)
              
          })

# SpatialDecon: mixed cell deconvolution for spatial and/or bulk gene expression
# data
# Copyright (C) 2020, NanoString Technologies, Inc.
#    This program is free software: you can redistribute it and/or modify it
#    under the terms of the GNU General Public License as published by the Free
#    Software Foundation, either version 3 of the License, or (at your option)
#    any later version.
#    This program is distributed in the hope that it will be useful, but WITHOUT
#    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
#    more details.
#    You should have received a copy of the GNU General Public License along
#    with this program.  If not, see https://www.gnu.org/licenses/.
# Contact us:
# NanoString Technologies, Inc.
# 530 Fairview Avenue N
# Seattle, WA 98109
# Tel: (888) 358-6266
# pdanaher@nanostring.com


#' SpatialDecon: A package for estimating mixed cell type abundance in the regions of 
#' spatially-resolved gene expression studies
#'
#' The SpatialDecon package estimates mixed cell type abundance in the regions
#' of spatially-resolved gene
#' expression studies, using the method of Danaher & Kim (2020), "Advances in
#'  mixed cell deconvolution enable
#' quantification of cell types in spatially-resolved gene expression data."
#' It is also appropriate to apply to bulk gene expression data.
#'
#' @section functions:
#' Functions to help set up deconvolution:
#' \itemize{
#'  \item derive_GeoMx_background Estimates the background levels from GeoMx
#'  experiments
#'  \item collapseCellTypes reformats deconvolution results to merge
#'  closely-related cell types
#'  \item download_profile_matrix Downloads a cell profile matrix.
#'  \item safeTME: a data object, a matrix of immune cell profiles for use in
#'   tumor-immune deconvolution.
#' }
#' Deconvolution functions:
#' \itemize{
#'  \item spatialdecon runs the core deconvolution function
#'  \item reverseDecon runs a transposed/reverse deconvolution problem, fitting
#'  the data as a function of cell abundance estimates.
#'   Used to measure genes' dependency on cell mixing and to calculate gene
#'    residuals from cell mixing.
#' }
#' Plotting functions:
#' \itemize{
#'  \item florets Plot cell abundance on a specified x-y space, with each point
#'   a cockscomb plot showing the cell abundances of that region/sample.
#'  \item TIL_barplot Plot abundances of tumor infiltrating lymphocytes (TILs)
#'   estimated from the safeTME cell profile matrix
#' }
#' @examples
#' data(mini_geomx_dataset)
#' data(safeTME)
#' data(safeTME.matches)
#' # estimate background:
#' mini_geomx_dataset$bg <- derive_GeoMx_background(
#'   norm = mini_geomx_dataset$normalized,
#'   probepool = rep(1, nrow(mini_geomx_dataset$normalized)),
#'   negnames = "NegProbe"
#' )
#' # run basic decon:
#' res0 <- spatialdecon(
#'   norm = mini_geomx_dataset$normalized,
#'   bg = mini_geomx_dataset$bg,
#'   X = safeTME
#' )
#' # run decon with bells and whistles:
#' res <- spatialdecon(
#'   norm = mini_geomx_dataset$normalized,
#'   bg = mini_geomx_dataset$bg,
#'   X = safeTME,
#'   cellmerges = safeTME.matches,
#'   cell_counts = mini_geomx_dataset$annot$nuclei,
#'   is_pure_tumor = mini_geomx_dataset$annot$AOI.name == "Tumor"
#' )
#' @docType package
#' @name SpatialDecon-package
NULL

# SpatialDecon: mixed cell deconvolution for spatial and/or bulk gene expression
# data
# Copyright (C) 2020, NanoString Technologies, Inc.
#    This program is free software: you can redistribute it and/or modify it
#    under the terms of the GNU General Public License as published by the Free
#    Software Foundation, either version 3 of the License, or (at your option)
#    any later version.
#    This program is distributed in the hope that it will be useful, but WITHOUT
#    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
#    more details.
#    You should have received a copy of the GNU General Public License along
#    with this program.  If not, see https://www.gnu.org/licenses/.
# Contact us:
# NanoString Technologies, Inc.
# 530 Fairview Avenue N
# Seattle, WA 98109
# Tel: (888) 358-6266
# pdanaher@nanostring.com


#' Derive background at the scale of the normalized data for GeoMx data
#'
#' Estimates per-datapoint background levels from a GeoMx experiment.
#' In studies with two or more probe pools, different probes will have different
#' background levels. This function provides a convenient way to account for
#' this phenomenon.
#'
#' @param norm Matrix of normalized data, genes in rows and segments in columns.
#'  Must include negprobes, and must have rownames.
#' @param probepool Vector of probe pool names for each gene, aligned to the
#' rows of "norm".
#' @param negnames Names of all negProbes in the dataset. Must be at least one
#'  neg.name within each probe pool.
#' @return A matrix of expected background values, in the same scale and
#'  dimensions as the "norm" argument.
#' @examples
#' data(mini_geomx_dataset)
#' # estimate background:
#' mini_geomx_dataset$bg <- derive_GeoMx_background(
#'   norm = mini_geomx_dataset$normalized,
#'   probepool = rep(1, nrow(mini_geomx_dataset$normalized)),
#'   negnames = "NegProbe"
#' )
#' @export
derive_GeoMx_background <- function(norm, probepool, negnames) {
    
    # check data input:
    if (nrow(norm) != length(probepool)) {
        stop("nrow(norm) != length(probepool)")
    }
    
    # initialize:
    bg <- norm * 0
    
    
    # fill in expected background at scale of normalized data:
    for (pool in unique(probepool)) {
        
        # get the pool's negProbes:
        tempnegs <- intersect(negnames, rownames(norm)[probepool == pool])
        if (length(tempnegs) == 0) {
            stop(paste0(pool, " probe pool didn't have any negprobes specified"))
        }
        tempnegfactor <- colMeans(norm[tempnegs, , drop = FALSE])
        
        # fill in the corresponding elements of bg:
        bg[probepool == pool, ] <-
            sweep(bg[probepool == pool, ], 2, tempnegfactor, "+")
    }
    return(bg)
}

# SpatialDecon: mixed cell deconvolution for spatial and/or bulk gene expression
# data
# Copyright (C) 2020, NanoString Technologies, Inc.
#    This program is free software: you can redistribute it and/or modify it
#    under the terms of the GNU General Public License as published by the Free
#    Software Foundation, either version 3 of the License, or (at your option)
#    any later version.
#    This program is distributed in the hope that it will be useful, but WITHOUT
#    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
#    more details.
#    You should have received a copy of the GNU General Public License along
#    with this program.  If not, see https://www.gnu.org/licenses/.
# Contact us:
# NanoString Technologies, Inc.
# 530 Fairview Avenue N
# Seattle, WA 98109
# Tel: (888) 358-6266
# pdanaher@nanostring.com


#' Convert abundance measurements to cell counts
#'
#' Converts cell abundance scores to cell counts, under the assumption that the
#'  observation with
#'  the greatest sum of cell abundance scores is 100% modelled cells.
#' @param beta Matrix of cell abundance scores, with cells in rows and
#'  observations in columns. The
#'  assumption is that this matrix is from well-normalized data.
#' @param nuclei.counts Optional. A vector of total nuclei counts. If provided,
#' the function will
#'  output not only cells.per.100 but also total cells.
#' @param omit.tumor Logical. If FALSE, any rows of beta with "tumor" in their
#' name will be omitted.
#' @return A list with two elements, each a rescaled version of beta.
#' cells.per.100 gives estimated
#'  percents of total, and cell.counts is cells.per.100 * nuclei.counts.
#' @keywords internal
#' @noRd
convertCellScoresToCounts <- function(beta, nuclei.counts = NULL,
                                      omit.tumor = FALSE) {
    # strip tumor rows if called for:
    if (omit.tumor) {
        beta <- beta[!grepl("tumor", rownames(beta)), , drop = FALSE]
    }
    
    # calc max abundance scores:
    max.total.abundance <- max(colSums(beta))
    
    # calculate rescaled scores:
    out <- list()
    out$cells.per.100 <- beta / max.total.abundance * 100
    if (length(nuclei.counts) == ncol(beta)) {
        out$cell.counts <- sweep(out$cells.per.100, 2, nuclei.counts, "*") / 100
    }
    return(out)
}

#' Run collapseCellTypes
#' 
#' Runs collapseCellTypes from an S4 object
#' @param object An S4 object such as a GeoMxSet object
#' @param ... Arguments passed to collapseCellTypes
#' @return A reshaped deconvolution result object
#' 
setGeneric("runCollapseCellTypes", signature = "object",
           function(object, ...) standardGeneric("runCollapseCellTypes"))

#' Collapse related cell types within a deconvolution result
#'
#' Given the input of an SpatialDecon result output and a list of which cell
#' types to combine,
#'  returns a reshaped deconvolution result object with the specified cell
#'  types merged.
#' 
#' @param matching A list object holding the mapping from beta's cell names to
#' official cell names.
#'  See str(safeTME.matches) for an example.
#' @importFrom stats pnorm
#' @importFrom Biobase assayData
#' @examples
#' library(GeomxTools)
#' datadir <- system.file("extdata", "DSP_NGS_Example_Data", package = "GeomxTools")
#' demoData <- readRDS(file.path(datadir, "/demoData.rds"))
#' 
#' demoData <- shiftCountsOne(demoData)
#' target_demoData <- aggregateCounts(demoData)
#' 
#' target_demoData <- normalize(target_demoData, "quant")
#'                 
#' # run basic decon:
#' res0 <- runspatialdecon(object = target_demoData,
#'                         norm_elt = "exprs_norm",
#'                         raw_elt = "exprs")
#'
#' # run reverse decon:
#' target_demoData <- runReverseDecon(object = target_demoData,
#'                                    norm_elt = "exprs_norm",
#'                                    beta = pData(res0)$beta)
#'
#' @export
#' @rdname runCollapseCellTypes
setMethod("runCollapseCellTypes",  "NanoStringGeoMxSet",
          function(object, matching = NULL){
              
              if(is.null(matching)){
                  stop("matching must be set")
              }
              
              fit <- list(beta=t(Biobase::pData(object)$beta),
                          p=t(Biobase::pData(object)$p),
                          t=t(Biobase::pData(object)$t),
                          se=t(Biobase::pData(object)$se),
                          prop_of_all=t(Biobase::pData(object)$prop_of_all),
                          prop_of_nontumor=t(Biobase::pData(object)$prop_of_nontumor),
                          sigma=Biobase::pData(object)$sigmas)
              
              temp <- collapseCellTypes(fit = fit, 
                                        matching = matching)
              
              Biobase::pData(object)$beta <- t(temp$beta)
              Biobase::pData(object)$p <- t(temp$p)
              Biobase::pData(object)$t <- t(temp$t)
              Biobase::pData(object)$se <- t(temp$se)
              Biobase::pData(object)$prop_of_all <- t(temp$prop_of_all)
              Biobase::pData(object)$prop_of_nontumor <- t(temp$prop_of_nontumor)
              Biobase::pData(object)$sigmas <- temp$sigma
              
              return(object)
          })

#' Default colors for the cell types in the safeTME matrix
#'
#' A named vector of colors, giving colors for the cell types of the safeTME
#'  matrix.
#'
#' @format A named vector
"cellcols"

#' Small example GeoMx data 
#'
#' A miniature GeoMx dataset used by the spatialdecon examples. 
#'
#' @format A list with the following elements:
#'  \itemize{
#'  \item normalized: normalized data matrix
#'  \item raw: raw data matrix
#'  \item annot: AOI annotation data frame
#'  }
"mini_geomx_dataset"

#' Large example GeoMx data 
#'
#' A GeoMx dataset with dense AOIs gridded over a NSCLC tumor. Each AOI is split into tumor and microenvironment segments.
#'
#' @format GeoMxSet Object
"nsclc"



#' Mapping from granularly-defined cell populations to broaded cell populations
#'
#' Mapping from granularly-defined cell populations to broaded cell populations,
#'  for use by the convertCellTypes function.
#'
#' @format A list. Each element of the list contains the granular cell types
#'  that roll up
#'  to a single coarse cell type.
"safeTME.matches"


#' SafeTME matrix
#'
#' A matrix of expression profiles of 906 genes over 18 cell types.
#'
#' @format A matrix with 906 genes (rows) and 18 cell types (columns)
"safeTME"


#' Genes' biological variability in immune deconvolution from TCGA.
#'
#' Genes' biological SDs, as estimated from immune deconvolution from TCGA.
#' Used to weight genes in spatialdecon.
#'
#' @format A named vector giving SDs of 1179 genes.
"mean.resid.sd"

#' Mini human colon single cell dataset
#'
#' Random 250 cells and most informative genes (CV > 10) between cell types from
#' Kinchen, J. et al. Structural Remodeling of the Human Colonic 
#' Mesenchyme in Inflammatory Bowel Disease. Cell 175, 372-386.e17 (2018).
#'
#' @format A list with the following elements:
#'  \itemize{
#'  \item mtx: sparse count matrix
#'  \item annots: cell type annotation data frame
#'  }
"mini_singleCell_dataset"

# SpatialDecon: mixed cell deconvolution for spatial and/or bulk gene expression
# data
# Copyright (C) 2020, NanoString Technologies, Inc.
#    This program is free software: you can redistribute it and/or modify it
#    under the terms of the GNU General Public License as published by the Free
#    Software Foundation, either version 3 of the License, or (at your option)
#    any later version.
#    This program is distributed in the hope that it will be useful, but WITHOUT
#    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
#    more details.
#    You should have received a copy of the GNU General Public License along
#    with this program.  If not, see https://www.gnu.org/licenses/.
# Contact us:
# NanoString Technologies, Inc.
# 530 Fairview Avenue N
# Seattle, WA 98109
# Tel: (888) 358-6266
# pdanaher@nanostring.com

#' Barplot of abundance estimates
#'
#' Draw barplot of the "betas" from a decon fit
#'
#' @param mat Matrix of cell proportions or abundances, in the same dimensions
#' output by spatialdecon
#'  (cells in rows, observations in columns). User is free to re-order
#'  columns/observations in
#'  whatever order is best for display.
#' @param draw_legend Logical. If TRUE, the function draws a legend in a new
#' plot frame.
#' @param main Title for barplot
#' @param col Vector of colors for cell types. Defaults to pre-set colors for
#' the safeTME cell types.
#' @param ... Arguments passed to barplot()
#' @return Draws a barplot.
#' @examples
#' data(mini_geomx_dataset)
#' data(safeTME)
#' # estimate background:
#' mini_geomx_dataset$bg <- derive_GeoMx_background(
#'   norm = mini_geomx_dataset$normalized,
#'   probepool = rep(1, nrow(mini_geomx_dataset$normalized)),
#'   negnames = "NegProbe"
#' )
#' # run basic decon:
#' res0 <- spatialdecon(
#'   norm = mini_geomx_dataset$normalized,
#'   bg = mini_geomx_dataset$bg,
#'   X = safeTME
#' )
#' # run barplot:
#' TIL_barplot(mat = res0$beta)
#' # run barplot and draw a color legend
#' TIL_barplot(mat = res0$beta, draw_legend = TRUE)
#' @import graphics
#' @importFrom graphics frame legend barplot
#' @importFrom grDevices colors
#' @importFrom utils data
#' @export
TIL_barplot <- function(mat, draw_legend = FALSE, main = "", col = NULL, ...) {
    
    
    # infer colors:
    if (length(col) == 0) {
        
        # use safeTME colors if the right cells are present:
        #utils::data("cellcols", envir = environment())
        if (all(is.element(rownames(mat), names(SpatialDecon::cellcols)))) {
            col <- SpatialDecon::cellcols[rownames(mat)]
        }
        else {
            manycols <- c(
                "#8DD3C7", "#FFFFB3", "#BEBADA", "#FB8072", "#80B1D3", "#FDB462",
                "#B3DE69", "#FCCDE5", "#A6CEE3", "#1F78B4", "#B2DF8A", "#33A02C",
                "#FB9A99", "#E31A1C", "#FDBF6F", "#FF7F00", "#1B9E77", "#D95F02",
                "#7570B3", "#E7298A", "#66A61E", "#E6AB02", "#A6761D", "#666666",
                sample(grDevices::colors(), 99)
            )
            col <- manycols[seq_len(nrow(mat))]
            names(col) <- rownames(mat)
        }
    }
    
    usecells <- rownames(mat)
    
    # draw barplot:
    graphics::barplot(mat[usecells, ],
                      cex.lab = 1.5,
                      col = col, border = NA,
                      las = 2, main = main, ...
    )
    
    # draw a legend:
    if (draw_legend) {
        graphics::frame()
        graphics::legend("center",
                         fill = rev(col),
                         legend = rev(names(col))
        )
    }
}

# SpatialDecon: mixed cell deconvolution for spatial and/or bulk gene expression
# data
# Copyright (C) 2020, NanoString Technologies, Inc.
#    This program is free software: you can redistribute it and/or modify it
#    under the terms of the GNU General Public License as published by the Free
#    Software Foundation, either version 3 of the License, or (at your option)
#    any later version.
#    This program is distributed in the hope that it will be useful, but WITHOUT
#    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
#    more details.
#    You should have received a copy of the GNU General Public License along
#    with this program.  If not, see https://www.gnu.org/licenses/.
# Contact us:
# NanoString Technologies, Inc.
# 530 Fairview Avenue N
# Seattle, WA 98109
# Tel: (888) 358-6266
# pdanaher@nanostring.com


#' Mixed cell deconvolution of spatiall-resolved gene expression data
#'
#' Runs the spatialdecon algorithm with added optional functionalities.
#' Workflow is:
#' \enumerate{
#' \item compute weights from raw data
#' \item Estimate a tumor profile and merge it into the cell profiles matrix
#' \item run deconvolution once
#' \item remove poorly-fit genes from first round of decon
#' \item re-run decon with cleaned-up gene set
#' \item combine closely-related cell types
#' \item compute p-values
#' \item rescale abundance estimates, to proportions of total, proportions of
#'  immune, cell counts
#' }
#'
#' @param norm p-length expression vector or p * N expression matrix - the
#' actual (linear-scale) data
#' @param bg Same dimension as norm: the background expected at each data point.
#' @param X Cell profile matrix. If NULL, the safeTME matrix is used.
#' @param raw Optional for using an error model to weight the data points.
#'  p-length expression vector or p * N expression matrix - the raw
#'  (linear-scale) data
#' @param wts Optional, a matrix of weights.
#' @param resid_thresh A scalar, sets a threshold on how extreme individual data
#'  points' values
#'  can be (in log2 units) before getting flagged as outliers and set to NA.
#' @param lower_thresh A scalar. Before log2-scale residuals are calculated,
#'  both observed and fitted
#'  values get thresholded up to this value. Prevents log2-scale residuals from
#'  becoming extreme in
#'  points near zero.
#' @param align_genes Logical. If TRUE, then Y, X, bg, and wts are row-aligned
#'  by shared genes.
#' @param is_pure_tumor A logical vector denoting whether each AOI consists of
#'  pure tumor. If specified,
#'  then the algorithm will derive a tumor expression profile and merge it with
#'  the immune profiles matrix.
#' @param cell_counts Number of cells estimated to be within each sample. If
#' provided alongside norm_factors,
#'  then the algorithm will additionally output cell abundance esimtates on the
#'  scale of cell counts.
#' @param cellmerges A list object holding the mapping from beta's cell names to
#'  combined cell names. If left
#'  NULL, then defaults to a mapping of granular immune cell definitions to
#'   broader categories.
#' @param n_tumor_clusters Number of tumor-specific columns to merge into the
#' cell profile matrix.
#'  Has an impact only when is_pure_tumor argument is used to indicate pure
#'   tumor AOIs.
#'  Takes this many clusters from the pure-tumor AOI data and gets the average
#'  expression profile in each cluster.  Default 10.
#' @param maxit Maximum number of iterations. Default 1000.
#' @return a list:
#' \itemize{
#' \item beta: matrix of cell abundance estimates, cells in rows and
#' observations in columns
#' \item sigmas: covariance matrices of each observation's beta estimates
#' \item p: matrix of p-values for H0: beta == 0
#' \item t: matrix of t-statistics for H0: beta == 0
#' \item se: matrix of standard errors of beta values
#' \item prop_of_all: rescaling of beta to sum to 1 in each observation
#' \item prop_of_nontumor: rescaling of beta to sum to 1 in each observation,
#' excluding tumor abundance estimates
#' \item cell.counts: beta rescaled to estimate cell numbers, based on
#' prop_of_all and nuclei count
#' \item beta.granular: cell abundances prior to combining closely-related
#' cell types
#' \item sigma.granular: sigmas prior to combining closely-related cell types
#' \item cell.counts.granular: cell.counts prior to combining closely-related
#' cell types
#' \item resids: a matrix of residuals from the model fit.
#'  (log2(pmax(y, lower_thresh)) - log2(pmax(xb, lower_thresh))).
#' \item X: the cell profile matrix used in the decon fit.
#' }
#' @examples
#' data(mini_geomx_dataset)
#' data(safeTME)
#' data(safeTME.matches)
#' # estimate background:
#' mini_geomx_dataset$bg <- derive_GeoMx_background(
#'   norm = mini_geomx_dataset$normalized,
#'   probepool = rep(1, nrow(mini_geomx_dataset$normalized)),
#'   negnames = "NegProbe"
#' )
#' # run basic decon:
#' res0 <- spatialdecon(
#'   norm = mini_geomx_dataset$normalized,
#'   bg = mini_geomx_dataset$bg,
#'   X = safeTME
#' )
#' # run decon with bells and whistles:
#' res <- spatialdecon(
#'   norm = mini_geomx_dataset$normalized,
#'   bg = mini_geomx_dataset$bg,
#'   X = safeTME,
#'   cellmerges = safeTME.matches,
#'   cell_counts = mini_geomx_dataset$annot$nuclei,
#'   is_pure_tumor = mini_geomx_dataset$annot$AOI.name == "Tumor"
#' )
#' @importFrom stats pnorm
#' @importFrom utils data
#' @export
spatialdecon <- function(norm, bg, X = NULL,
                         raw = NULL, wts = NULL,
                         resid_thresh = 3, lower_thresh = 0.5,
                         align_genes = TRUE,
                         is_pure_tumor = NULL, n_tumor_clusters = 10,
                         cell_counts = NULL,
                         cellmerges = NULL,
                         maxit = 1000) {
    
    #### preliminaries ---------------------------------
    
    # check formatting:
    if (!is.matrix(norm)) {
        stop("norm should be a matrix")
    }
    if ((length(X) > 0) & (!is.matrix(X))) {
        stop("X should be a matrix")
    }
    if ((length(raw) > 0) & (!is.matrix(raw))) {
        stop("raw must be a matrix")
    }
    if ((length(wts) > 0) & (!is.matrix(wts))) {
        stop("wts must be a matrix")
    }
    if ((length(cell_counts) > 0) & (!is.numeric(cell_counts))) {
        stop("cell_counts must be numeric")
    }
    
    
    if (length(bg) == 1) {
        bg <- matrix(bg, nrow(norm), ncol(norm),
                     dimnames = list(rownames(norm), colnames(norm))
        )
    }
    
    # If a matrix other than safeTME is input, rescale training matrix to avoid bad convergence properties:
    if (length(X) > 0) {
        # rescale matrix so its 99th percentile is near that of safeTME (which has a 99th percentile = 2.3)
        X <- X * 2 / quantile(X, 0.99)
    }
    
    # prep training matrix:
    if (length(X) == 0) {
        utils::data("safeTME", envir = environment())
        X <- SpatialDecon::safeTME
    }
    sharedgenes <- intersect(rownames(norm), rownames(X))
    if (length(sharedgenes) == 0) {
        stop("no shared gene names between norm and X")
    }
    if (length(sharedgenes) < 100) {
        stop(paste0(
            "Only ", length(sharedgenes),
            " genes are shared between norm and X - this may not be enough
                to support accurate deconvolution."
        ))
    }
    
    # calculate weights based on expected SD of counts
    # wts = replace(norm, TRUE, 1)
    if (length(raw) > 0) {
        weight.by.TIL.resid.sd <-
            length(intersect(colnames(X), colnames(SpatialDecon::safeTME))) > 10
        wts <- deriveWeights(norm,
                             raw = raw, error.model = "dsp",
                             weight.by.TIL.resid.sd = weight.by.TIL.resid.sd
        )
    }
    
    #### if pure tumor AOIs are specificed, get tumor expression profile --------
    if (sum(is_pure_tumor) > 0) {
        
        # derive tumor profiles and merge into X:
        # (derive a separate profile for each tissue)
        X <- mergeTumorIntoX(
            norm = norm,
            bg = bg,
            pure_tumor_ids = is_pure_tumor,
            X = X[sharedgenes, ],
            K = n_tumor_clusters
        )
        
        sharedgenes <- intersect(rownames(norm), rownames(X))
    }
    
    
    #### Run decon  -----------------------------------
    res <- algorithm2(
        Y = norm[sharedgenes, ],
        bg = bg[sharedgenes, ],
        X = X[sharedgenes, ],
        weights = wts[sharedgenes, ],
        maxit = maxit,
        resid_thresh = resid_thresh,
        lower_thresh = lower_thresh
    )
    
    
    #### combine closely-related cell types ------------------------------------
    
    if (length(cellmerges) > 0) {
        tempconv <- convertCellTypes(
            beta = res$beta,
            matching = cellmerges,
            stat = sum,
            na.rm = FALSE,
            sigma = res$sigmas
        )
        # overwrite original beta with merged beta:
        res$beta.granular <- res$beta
        res$sigma.granular <- res$sigmas
        res$sigmas <- NULL
        res$beta <- tempconv$beta
        res$sigma <- tempconv$sigma
    }
    
    
    #### compute p-values -------------------------------------------
    tempbeta <- res$beta
    tempse <- tempp <- tempt <- tempbeta * NA
    for (i in seq_len(ncol(tempse))) {
        tempse[, i] <- suppressWarnings(sqrt(diag(res$sigma[, , i])))
    }
    tempt <- (tempbeta / tempse)
    tempp <- 2 * (1 - stats::pnorm(tempt))
    res$p <- tempp
    res$t <- tempt
    res$se <- tempse
    
    
    #### rescale abundance estimates --------------------------------
    # (to proportions of total, proportions of immune, cell counts)
    
    # proportions:
    res$prop_of_all <- sweep(res$beta, 2, colSums(res$beta), "/")
    nontumorcellnames <- rownames(res$beta)[!grepl("tumor", rownames(res$beta))]
    res$prop_of_nontumor <- sweep(
        res$beta[nontumorcellnames, ], 2,
        colSums(res$beta[nontumorcellnames, ]), "/"
    )
    
    # on scale of cell counts:
    if (length(cell_counts) > 0) {
        res$cell.counts <- convertCellScoresToCounts(
            beta = res$beta,
            nuclei.counts = cell_counts,
            omit.tumor = TRUE
        )
        if (length(res$beta.granular) > 0) {
            res$cell.counts.granular <- convertCellScoresToCounts(
                beta = res$beta.granular,
                nuclei.counts = cell_counts,
                omit.tumor = TRUE
            )
        }
    }
    
    # add other pertinent info to res:
    res$X <- X[rownames(res$resids), ]
    return(res)
}

#' Run spatialdecon
#' 
#' Runs spatialdecon from an S4 object
#' @param object An S4 object such as a Seurat object that includse a "Spatial" element in the "assays" slot or a GeoMxSet object
#' @param ... Arguments passed to spatialdecon
#' @return decon results in list or in GeoMxSet object
#' 
setGeneric("runspatialdecon", signature = "object",
           function(object, ...) standardGeneric("runspatialdecon"))

#' Run spatialdecon on a NanostringGeomxSet object
#'
#' A wrapper for applying spatialdecon to a NanostringGeomxSet object.
#'
#' @param norm_elt normalized data element in assayData in NanostringGeomxSet object
#' @param raw_elt raw data element in assayData in NanostringGeomxSet object
#' @param X Cell profile matrix. If NULL, the safeTME matrix is used.
#' @param wts Optional, a matrix of weights.
#' @param resid_thresh A scalar, sets a threshold on how extreme individual data
#'  points' values
#'  can be (in log2 units) before getting flagged as outliers and set to NA.
#' @param lower_thresh A scalar. Before log2-scale residuals are calculated,
#'  both observed and fitted
#'  values get thresholded up to this value. Prevents log2-scale residuals from
#'  becoming extreme in
#'  points near zero.
#' @param align_genes Logical. If TRUE, then Y, X, bg, and wts are row-aligned
#'  by shared genes.
#' @param is_pure_tumor A logical vector denoting whether each AOI consists of
#'  pure tumor. If specified,
#'  then the algorithm will derive a tumor expression profile and merge it with
#'  the immune profiles matrix.
#' @param cell_counts Number of cells estimated to be within each sample. If
#' provided alongside norm_factors,
#'  then the algorithm will additionally output cell abundance esimtates on the
#'  scale of cell counts.
#' @param cellmerges A list object holding the mapping from beta's cell names to
#'  combined cell names. If left
#'  NULL, then defaults to a mapping of granular immune cell definitions to
#'   broader categories.
#' @param n_tumor_clusters Number of tumor-specific columns to merge into the
#' cell profile matrix.
#'  Has an impact only when is_pure_tumor argument is used to indicate pure
#'   tumor AOIs.
#'  Takes this many clusters from the pure-tumor AOI data and gets the average
#'  expression profile in each cluster.  Default 10.
#' @param maxit Maximum number of iterations. Default 1000.

#' @importFrom Biobase assayData
#' @importClassesFrom GeomxTools NanoStringGeoMxSet
#'
#' @return For GeoMxSet object, if not given cellmerges and cell_counts, a valid GeoMx S4 object
#' including the following items
#'  \itemize{
#'    \item In pData
#'    \itemize{
#'       \item beta: matrix of cell abundance estimates, cells in rows and
#'                observations in columns
#'       \item p: matrix of p-values for H0: beta == 0
#'       \item t: matrix of t-statistics for H0: beta == 0
#'       \item se: matrix of standard errors of beta values
#'       \item prop_of_all: rescaling of beta to sum to 1 in each observation
#'       \item prop_of_nontumor: rescaling of beta to sum to 1 in each observation,
#'                            excluding tumor abundance estimates
#'       \item sigmas: covariance matrices of each observation's beta estimates
#'      }
#'    \item In assayData
#'    \itemize{
#'       \item yhat: a matrix of fitted values
#'       \item resids: a matrix of residuals from the model fit.
#'                  (log2(pmax(y, lower_thresh)) - log2(pmax(xb, lower_thresh))).
#'    }
#'    \item In experimentData
#'    \itemize{
#'       \item SpatialDeconMatrix: the cell profile matrix used in the decon fit.
#'    }
#'}
#'
#'
#' if given cellmerges, the valid GeoMx S4 object will additionally include
#' the following items
#'  \itemize{
#'    \item In pData
#'    \itemize{
#'       \item beta.granular: cell abundances prior to combining closely-related
#'                         cell types
#'       \item sigma.granular: sigmas prior to combining closely-related cell types
#'    }
#'}
#'
#' if given cell_counts, the valid GeoMx S4 object will additionally include
#'  the following items
#'  \itemize{
#'  \item In pData
#'    \itemize{
#'       \item cell.counts: beta rescaled to estimate cell numbers, based on
#'                       prop_of_all and nuclei count
#'    }
#'}
#'
#' if given both cellmerges and cell_counts, the valid GeoMx S4 object will
#'  additionally include the following items
#'  \itemize{
#'  \item In pData
#'    \itemize{
#'       \item cell.counts.granular: cell.counts prior to combining closely-related
#'                                cell types
#'    }
#'}
#'
#' @examples
#'
#' ## GeoMxSet Object ##
#' library(GeomxTools)
#' datadir <- system.file("extdata", "DSP_NGS_Example_Data", package = "GeomxTools")
#' demoData <- readRDS(file.path(datadir, "/demoData.rds"))
#' 
#' demoData <- shiftCountsOne(demoData)
#' target_demoData <- aggregateCounts(demoData)
#' 
#' target_demoData <- normalize(target_demoData, "quant")
#' 
#' demoData <- runspatialdecon(object = target_demoData, 
#'                             norm_elt = "exprs_norm",
#'                             raw_elt = "exprs")
#'
#' @export
#' @rdname runspatialdecon

setMethod("runspatialdecon", "NanoStringGeoMxSet",
          function(object, X = NULL, norm_elt = NULL, raw_elt = NULL,
                   wts = NULL,
                   resid_thresh = 3, lower_thresh = 0.5,
                   align_genes = TRUE,
                   is_pure_tumor = NULL, n_tumor_clusters = 10,
                   cell_counts = NULL,
                   cellmerges = NULL,
                   maxit = 1000){
            
            # check that norm and raw data exists:
            if(is.null(norm_elt)){
              stop("norm_elt must be set")
            }
            if(is.null(raw_elt)){
              stop("raw_elt must be set")
            }
            if (!is.element(norm_elt, names(object@assayData))) {
              stop(paste(norm_elt, "is not an element in assaysData slot"))
            }
            if (!is.element(raw_elt, names(object@assayData))) {
              stop(paste(raw_elt, "is not an element in assaysData slot"))
            }
            
            # prep components:
            norm <- as.matrix(Biobase::assayDataElement(object , elt = norm_elt))
            raw <- as.matrix(Biobase::assayDataElement(object , elt = raw_elt))
            
            # estimate background
            bg <- derive_GeoMx_background(norm = norm,
                                          # access the probe pool information from the feature metadata
                                          probepool = Biobase::fData(object)$Module,
                                          # access the names of the negative control probes
                                          negnames = Biobase::fData(object)$TargetName[Biobase::fData(object)$Negative])
            
            # run spatialdecon:
            result <- spatialdecon(norm = norm,
                                   bg = bg,
                                   X = X,
                                   raw = raw,
                                   wts = wts,
                                   resid_thresh = resid_thresh, 
                                   lower_thresh = lower_thresh,
                                   align_genes = align_genes,
                                   is_pure_tumor = is_pure_tumor, 
                                   n_tumor_clusters = n_tumor_clusters,
                                   cell_counts = cell_counts,
                                   cellmerges = cellmerges,
                                   maxit = maxit)
            
            
            # append results to the object
            # beta
            Biobase::pData(object)[["beta"]] <- matrix(NA, nrow = ncol(object), ncol = nrow(result$beta),
                                                       dimnames = list(Biobase::sampleNames(object), rownames(result$beta)))
            Biobase::pData(object)[["beta"]][colnames(result$beta), ] <- t(result$beta)
            
            
            # p
            Biobase::pData(object)[["p"]] <- matrix(NA, nrow = ncol(object), ncol = nrow(result$p),
                                                    dimnames = list(Biobase::sampleNames(object), rownames(result$p)))
            Biobase::pData(object)[["p"]][colnames(result$p), ] <- t(result$p)
            
            
            # t
            Biobase::pData(object)[["t"]] <- matrix(NA, nrow = ncol(object), ncol = nrow(result$t),
                                                    dimnames = list(Biobase::sampleNames(object), rownames(result$t)))
            Biobase::pData(object)[["t"]][colnames(result$t), ] <- t(result$t)
            
            
            # se
            Biobase::pData(object)[["se"]] <- matrix(NA, nrow = ncol(object), ncol = nrow(result$se),
                                                     dimnames = list(Biobase::sampleNames(object), rownames(result$se)))
            Biobase::pData(object)[["se"]][colnames(result$se), ] <- t(result$se)
            
            
            # prop_of_all
            Biobase::pData(object)[["prop_of_all"]] <- matrix(NA, nrow = ncol(object), ncol = nrow(result$prop_of_all),
                                                              dimnames = list(Biobase::sampleNames(object), rownames(result$prop_of_all)))
            Biobase::pData(object)[["prop_of_all"]][colnames(result$prop_of_all), ] <- t(result$prop_of_all)
            
            
            # prop_of_nontumor
            Biobase::pData(object)[["prop_of_nontumor"]] <- matrix(NA, nrow = ncol(object), ncol = nrow(result$prop_of_nontumor),
                                                                   dimnames = list(Biobase::sampleNames(object), rownames(result$prop_of_nontumor)))
            Biobase::pData(object)[["prop_of_nontumor"]][colnames(result$prop_of_nontumor), ] <- t(result$prop_of_nontumor)
            
            
            # add yhat matrix
            yhat_mat <- matrix(NA, nrow = nrow(object), ncol = ncol(object),
                               dimnames = dimnames(object))
            yhat_mat[rownames(result$yhat), colnames(result$yhat)] <- result$yhat
            Biobase::assayDataElement(object, "yhat") <- yhat_mat
            
            
            # add resids matrix
            resids_mat <- matrix(NA, nrow = nrow(object), ncol = ncol(object),
                                 dimnames = dimnames(object))
            resids_mat[rownames(result$resids), colnames(result$resids)] <- result$resids
            Biobase::assayDataElement(object, "resids") <- resids_mat
            
            
            # add X matrix
            Biobase::fData(object)[["X"]] <- matrix(NA, nrow = nrow(object), ncol = ncol(result$X),
                                                    dimnames = list(Biobase::featureNames(object), colnames(result$X)))
            Biobase::fData(object)[["X"]][rownames(result$X), ] <- result$X
            
            
            # transpose 3-d array
            trans <- function(array){
              NewArray <- array(NA, dim = c(dim(array)[3], dim(array)[2], dim(array)[1]))
              
              for(or in(seq_len(dim(array)[1]))){
                for(oc in(seq_len(dim(array)[2]))){
                  NewArray[,oc,or] <- array[or,oc,]
                }
              }
              array <- NewArray
            }
            
            if (is.null(cellmerges)) {
              # add sigmas matrix
              sigmas_mat <- trans(result$sigmas)
              sigmas_dimname <- list(Biobase::sampleNames(object),
                                     paste0("var", seq_len((dim(result$sigmas)[1]))),
                                     paste0("var", seq_len((dim(result$sigmas)[2]))))
              dimnames(sigmas_mat) <- sigmas_dimname
              
              Biobase::pData(object)[["sigmas"]] <- array(NA, dim = c(ncol(object),
                                                                      dim(result$sigmas)[1],
                                                                      dim(result$sigmas)[2]),
                                                          dimnames = sigmas_dimname)
              Biobase::pData(object)[["sigmas"]][unlist(dimnames(sigmas_mat)[1]),, ] <- sigmas_mat
            } else {
              # add sigma matrix
              sigma_mat <- trans(result$sigma)
              sigma_dimname <- list(Biobase::sampleNames(object),
                                    paste0("var", seq_len((dim(result$sigma)[1]))),
                                    paste0("var", seq_len((dim(result$sigma)[2]))))
              dimnames(sigma_mat) <- sigma_dimname
              
              Biobase::pData(object)[["sigma"]] <- array(NA, dim = c(ncol(object),
                                                                     dim(result$sigma)[1],
                                                                     dim(result$sigma)[2]),
                                                         dimnames = sigma_dimname)
              Biobase::pData(object)[["sigma"]][unlist(dimnames(sigma_mat)[1]),, ] <- sigma_mat
              
              
              # add beta.granular matrix
              Biobase::pData(object)[["beta.granular"]] <- matrix(NA, nrow = ncol(object),
                                                                  ncol = nrow(result$beta.granular),
                                                                  dimnames = list(Biobase::sampleNames(object),
                                                                                  rownames(result$beta.granular)))
              Biobase::pData(object)[["beta.granular"]][colnames(result$beta.granular), ] <- t(result$beta.granular)
              
              
              # add sigma.granular matrix
              sigma.granular_mat <- trans(result$sigma.granular)
              sigma.granular_dimname <- list(Biobase::sampleNames(object),
                                             paste0("var", seq_len((dim(result$sigma.granular)[1]))),
                                             paste0("var", seq_len((dim(result$sigma.granular)[2]))))
              dimnames(sigma.granular_mat) <- sigma.granular_dimname
              
              Biobase::pData(object)[["sigma.granular"]] <- array(NA, dim = c(ncol(object),
                                                                              dim(result$sigma.granular)[1],
                                                                              dim(result$sigma.granular)[2]),
                                                                  dimnames = sigma.granular_dimname)
              Biobase::pData(object)[["sigma.granular"]][unlist(dimnames(sigma.granular_mat)[1]),, ] <- sigma.granular_mat
            }
            
            
            if (length(cell_counts) > 0) {
              if (length(result$cell.counts) == 2){
                # create cell.counts list
                Biobase::pData(object)[["cell.counts"]] <- data.frame(matrix(NA,
                                                                             nrow = ncol(object),
                                                                             ncol = 2,
                                                                             dimnames = list(Biobase::sampleNames(object),
                                                                                             c("cells.per.100", "cell.counts"))))                                                                                           
                # add cells.per.100 matrix
                Biobase::pData(object)[["cell.counts"]][["cells.per.100"]] <- matrix(NA, nrow = ncol(object),
                                                                                     ncol = nrow(result$cell.counts$cells.per.100),
                                                                                     dimnames = list(Biobase::sampleNames(object),
                                                                                                     rownames(result$cell.counts$cells.per.100)))
                Biobase::pData(object)[["cell.counts"]][["cells.per.100"]][colnames(result$cell.counts$cells.per.100), ] <- t(result$cell.counts$cells.per.100)
                
                
                # add cell.counts matrix
                Biobase::pData(object)[["cell.counts"]][["cell.counts"]] <- matrix(NA, nrow = ncol(object),
                                                                                   ncol = nrow(result$cell.counts$cell.counts),
                                                                                   dimnames = list(Biobase::sampleNames(object),
                                                                                                   rownames(result$cell.counts$cell.counts)))
                Biobase::pData(object)[["cell.counts"]][["cell.counts"]][colnames(result$cell.counts$cell.counts), ] <- t(result$cell.counts$cell.counts)
              } else {
                # create cell.counts list
                Biobase::pData(object)[["cell.counts"]] <- data.frame(matrix(NA,
                                                                             nrow = ncol(object),
                                                                             ncol = 1,
                                                                             dimnames = list(Biobase::sampleNames(object),
                                                                                             "cells.per.100")))
                
                # add cells.per.100 matrix
                Biobase::pData(object)[["cell.counts"]][["cells.per.100"]] <- matrix(NA, nrow = ncol(object),
                                                                                     ncol = nrow(result$cell.counts$cells.per.100),
                                                                                     dimnames = list(Biobase::sampleNames(object),
                                                                                                     rownames(result$cell.counts$cells.per.100)))
                Biobase::pData(object)[["cell.counts"]][["cells.per.100"]][colnames(result$cell.counts$cells.per.100), ] <- t(result$cell.counts$cells.per.100)
              }
            }
            
            
            if (length(result$cell.counts.granular) > 0) {
              
              if(length(result$cell.counts.granular) == 2){
                # create cell.counts.granular list
                Biobase::pData(object)[["cell.counts.granular"]] <- data.frame(matrix(NA,
                                                                                      nrow = ncol(object),
                                                                                      ncol = 2,
                                                                                      dimnames = list(Biobase::sampleNames(object),
                                                                                                      c("cells.per.100",
                                                                                                        "cell.counts"))))
                # add cells.per.100 matrix
                Biobase::pData(object)[["cell.counts.granular"]][["cells.per.100"]] <- matrix(NA, nrow = ncol(object),
                                                                                              ncol = nrow(result$cell.counts.granular$cells.per.100),
                                                                                              dimnames = list(Biobase::sampleNames(object),
                                                                                                              rownames(result$cell.counts.granular$cells.per.100)))
                Biobase::pData(object)[["cell.counts.granular"]][["cells.per.100"]][colnames(result$cell.counts.granular$cells.per.100), ] <- t(result$cell.counts.granular$cells.per.100)
                
                
                # add cells.count matrix
                Biobase::pData(object)[["cell.counts.granular"]][["cell.counts"]] <- matrix(NA, nrow = ncol(object),
                                                                                            ncol = nrow(result$cell.counts.granular$cell.counts),
                                                                                            dimnames = list(Biobase::sampleNames(object),
                                                                                                            rownames(result$cell.counts.granular$cell.counts)))
                Biobase::pData(object)[["cell.counts.granular"]][["cell.counts"]][colnames(result$cell.counts.granular$cells.per.100), ] <- t(result$cell.counts.granular$cell.counts)
              } else {
                # create cell.counts.granular list
                Biobase::pData(object)[["cell.counts.granular"]] <- data.frame(matrix(NA,
                                                                                      nrow = ncol(object),
                                                                                      ncol = 1,
                                                                                      dimnames = list(Biobase::sampleNames(object),
                                                                                                      "cells.per.100")))
                # add cells.per.100 matrix
                Biobase::pData(object)[["cell.counts.granular"]][["cells.per.100"]] <- matrix(NA, nrow = ncol(object),
                                                                                              ncol = nrow(result$cell.counts.granular$cells.per.100),
                                                                                              dimnames = list(Biobase::sampleNames(object),
                                                                                                              rownames(result$cell.counts.granular$cells.per.100)))
                Biobase::pData(object)[["cell.counts.granular"]][["cells.per.100"]][colnames(result$cell.counts.granular$cells.per.100), ] <- t(result$cell.counts.granular$cells.per.100)
              }
            }
            
            object@experimentData@other$SpatialDeconMatrix <- result$X
            
            return(object)
            
          })




#' Run spatialdecon on a Seurat object
#' 
#' A wrapper for applying spatialdecon to the Spatial data element in a Seurat object. 
#' Unlike spatialdecon, which expects a normalized data matrix, this function operates 
#' on raw counts. Scaling for total cells 
#' @param X Cell profile matrix. If NULL, the safeTME matrix is used.
#' @param bg Expected background counts. Either a scalar applied equally to 
#'  all points in the count matrix, or a matrix with the same dimensions 
#'  as the count matrix in GetAssayData(object, assay = "Spatial").
#'  Recommended to use a small non-zero value, default of 0.1.
#' @param wts Optional, a matrix of weights.
#' @param resid_thresh A scalar, sets a threshold on how extreme individual data
#'  points' values
#'  can be (in log2 units) before getting flagged as outliers and set to NA.
#' @param lower_thresh A scalar. Before log2-scale residuals are calculated,
#'  both observed and fitted
#'  values get thresholded up to this value. Prevents log2-scale residuals from
#'  becoming extreme in
#'  points near zero.
#' @param align_genes Logical. If TRUE, then Y, X, bg, and wts are row-aligned
#'  by shared genes.
#' @param is_pure_tumor A logical vector denoting whether each AOI consists of
#'  pure tumor. If specified,
#'  then the algorithm will derive a tumor expression profile and merge it with
#'  the immune profiles matrix.
#' @param cell_counts Number of cells estimated to be within each sample. If
#' provided alongside norm_factors,
#'  then the algorithm will additionally output cell abundance esimtates on the
#'  scale of cell counts.
#' @param cellmerges A list object holding the mapping from beta's cell names to
#'  combined cell names. If left
#'  NULL, then defaults to a mapping of granular immune cell definitions to
#'   broader categories.
#' @param n_tumor_clusters Number of tumor-specific columns to merge into the
#' cell profile matrix.
#'  Has an impact only when is_pure_tumor argument is used to indicate pure
#'   tumor AOIs.
#'  Takes this many clusters from the pure-tumor AOI data and gets the average
#'  expression profile in each cluster.  Default 10.
#' @param maxit Maximum number of iterations. Default 1000.
#' @return For Seurat Object, if not given cellmerges and cell_counts, a list
#' including the following items:
#'  \itemize{
#'    \item beta: matrix of cell abundance estimates, cells in rows and
#'                observations in columns
#'    \item p: matrix of p-values for H0: beta == 0
#'    \item t: matrix of t-statistics for H0: beta == 0
#'    \item se: matrix of standard errors of beta values
#'    \item prop_of_all: rescaling of beta to sum to 1 in each observation
#'    \item prop_of_nontumor: rescaling of beta to sum to 1 in each observation,
#'                            excluding tumor abundance estimates
#'    \item yhat: a matrix of fitted values
#'    \item resids: a matrix of residuals from the model fit.
#'                  (log2(pmax(y, lower_thresh)) - log2(pmax(xb, lower_thresh))).
#'    \item X: the cell profile matrix used in the decon fit.
#'    \item sigmas: covariance matrices of each observation's beta estimates
#'}
#'
#'
#' if given cellmerges, the list will additionally include
#' the following items
#'  \itemize{
#'    \item beta.granular: cell abundances prior to combining closely-related
#'                         cell types
#'    \item sigma.granular: sigmas prior to combining closely-related cell types
#'}
#'
#' if given cell_counts, the list will additionally include
#'  the following items
#'  \itemize{
#'    \item cell.counts: beta rescaled to estimate cell numbers, based on
#'                       prop_of_all and nuclei count
#'}
#'
#' if given both cellmerges and cell_counts, the list will
#'  additionally include the following items
#'  \itemize{
#'    \item cell.counts.granular: cell.counts prior to combining closely-related
#'                                cell types
#'}
#' @importFrom SeuratObject GetAssayData
#' @importClassesFrom GeomxTools NanoStringGeoMxSet
#' @export
#' @examples
#' ## Seurat Object ##
#' # get dataset
#' con <- gzcon(url("https://github.com/almaan/her2st/raw/master/data/ST-cnts/G1.tsv.gz"))
#' txt <- readLines(con)
#' temp <- read.table(textConnection(txt), sep = "\t", header = TRUE, row.names = 1)
#' # parse data
#' raw = t(as.matrix(temp))
#' norm = sweep(raw, 2, colSums(raw), "/") * mean(colSums(raw))
#' x = as.numeric(substr(rownames(temp), 1, unlist(gregexpr("x", rownames(temp))) - 1))
#' y = -as.numeric(substr(rownames(temp), 
#'                  unlist(gregexpr("x", rownames(temp))) + 1, nchar(rownames(temp))))
#' # put into a seurat object:
#' andersson_g1 = SeuratObject::CreateSeuratObject(counts = raw, assay="Spatial")
#' andersson_g1@meta.data$x = x
#' andersson_g1@meta.data$y = y
#' 
#' res <- runspatialdecon(andersson_g1)
#' str(res)
#' @rdname runspatialdecon
setMethod("runspatialdecon", "Seurat", function(
  object,
  X = NULL,
  bg = 0.1,  
  wts = NULL,
  resid_thresh = 3, lower_thresh = 0.5,
  align_genes = TRUE,
  is_pure_tumor = NULL, 
  n_tumor_clusters = 10,
  cell_counts = NULL,
  cellmerges = NULL,
  maxit = 1000) {
  
  # check that it's a spatial assay:
  if (!is.element("Spatial", names(object@assays))) {
    stop("Expecting \'Spatial\' element in assays slot")
  }
  
  # prep components:
  raw <- as.matrix(SeuratObject::GetAssayData(object, assay = "Spatial"))
  # make bg a matrix if only a scalar was specified:
  if (length(bg) == 1) {
    bg <- 0 * raw + bg
  }
  
  # run spatialdecon:
  res <- spatialdecon(norm = raw, 
                      bg = bg, 
                      X = X,
                      raw = raw, 
                      wts = wts,
                      resid_thresh = resid_thresh, 
                      lower_thresh = lower_thresh,
                      align_genes = align_genes,
                      is_pure_tumor = is_pure_tumor, 
                      n_tumor_clusters = n_tumor_clusters,
                      cell_counts = cell_counts,
                      cellmerges = cellmerges,
                      maxit = maxit) 
  
  return(res)
})

# SpatialDecon: mixed cell deconvolution for spatial and/or bulk gene expression
# data
# Copyright (C) 2020, NanoString Technologies, Inc.
#    This program is free software: you can redistribute it and/or modify it
#    under the terms of the GNU General Public License as published by the Free
#    Software Foundation, either version 3 of the License, or (at your option)
#    any later version.
#    This program is distributed in the hope that it will be useful, but WITHOUT
#    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
#    more details.
#    You should have received a copy of the GNU General Public License along
#    with this program.  If not, see https://www.gnu.org/licenses/.
# Contact us:
# NanoString Technologies, Inc.
# 530 Fairview Avenue N
# Seattle, WA 98109
# Tel: (888) 358-6266
# pdanaher@nanostring.com


#' Download a cell profile matrix
#'
#' Download a cell profile matrix from the online library
#'
#' @param species species of profile matrix 
#' @param age_group age_group of profile matrix, if fetal mouse please add the 
#' developmental stage separated with /, i.e. Fetal/E14.5 
#' @param matrixname name of profile matrix 
#' @return A cell profile matrix, suggested cell groups, and paper metadata
#' @details Valid matrices can be found on the github site 
#' \url{https://github.com/Nanostring-Biostats/CellProfileLibrary/tree/master}
#' @examples
#' download_profile_matrix(species = "Human", age_group = "Adult", matrixname = "Colon_HCA")
#' head(profile_matrix)
#' print(cellGroups)
#' print(metadata)
#' @importFrom utils read.csv 
#' @importFrom repmis source_data
#' @export
download_profile_matrix <- function(species, age_group, matrixname) {
    
    # check formatting:
    if (length(species) > 1) {
        stop("specify just one species")
    }
    if (length(age_group) > 1) {
        stop("specify just one age")
    }
    if (length(matrixname) > 1) {
        stop("specify just one matrixname")
    }
    
    valid_species <- c("Human", "Mouse")
    
    if (!species %in% valid_species) {
        stop(paste0("Species input is invalid; must be \"", paste(valid_species, 
                                                                  collapse = "\" or \""), "\" (case sensitive)"))
    }
    
    if(species == "Human"){
        valid_ages <- c("Adult", "COVID-Infected", "Fetal")
    }else{
        valid_ages <- c("Adult", "Fetal/E14.5", "Fetal/E9.5-13.5", "Neonatal")
    }
    
    if (!age_group %in% valid_ages) {
        stop(paste0("Age input is invalid; must be \"", paste(valid_ages, collapse = "\" or \""), "\" (case sensitive)"))
    }
    
    metadata <- utils::read.csv(paste0("https://raw.github.com/Nanostring-Biostats/CellProfileLibrary/master/", species, "/",
                                       species, "_datasets_metadata.csv"), header = TRUE, sep = ",")
    
    librarynames <- paste0(metadata$Tissue, "_", metadata$Profile.Matrix)
    
    
    if (!is.element(matrixname, librarynames)) {
        stop(paste0(matrixname, " is not an expected cell profile matrix name for the species given. Did you mean \"", 
                    paste(librarynames[agrep(matrixname, librarynames)], collapse = "\" or \""), "\"?"))
    }
    
    profilename <- matrixname
    matrixname <- paste(species, age_group, matrixname, sep = "/")
    
    fulllibrarynames <- paste(metadata$Species, metadata$Age.Group, librarynames, sep = "/")
    
    if (!is.element(matrixname, fulllibrarynames)) {
        stop(paste0(profilename, " does not correspond with the age_group given. Did you mean \"", 
                    paste(unique(t(as.data.frame(strsplit(fulllibrarynames[agrep(profilename, fulllibrarynames)], "/")))[,2]), 
                          collapse = "\" or \""), "\"?"))
    }
    
    
    suppressMessages(repmis::source_data(paste0("https://raw.github.com/Nanostring-Biostats/CellProfileLibrary/master/", 
                                                matrixname, ".RData?raw=True"), 
                                         cache = FALSE, rdata = TRUE, envir = globalenv()))
    
    #assign("profile_matrix", as.matrix(profile_matrix), envir = globalenv())
    
    return(as.matrix(profile_matrix))
}

---
title: "Use of SpatialDecon in a small GeoMx dataset"
output: 
  rmarkdown::html_vignette: 
    toc: true
vignette: >
  %\VignetteIndexEntry{Use of SpatialDecon in a small GeoMx dataet}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

<style>
p.caption {
  font-size: 1.5em;
}
</style>

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```


### Installation

```{r installation, eval=FALSE}
if(!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("SpatialDecon")

```

### Overview


This vignette demonstrates the use of the SpatialDecon package to estimate cell 
abundance in spatial gene expression studies. 

We'll analyze a small GeoMx dataset from a lung tumor, looking for abundance of
different immune cell types. 
This dataset has 30 ROIs. In each ROI, Tumor and Microenvironment segments have 
been profiled separately. 


### Data preparation

First, we load the package:
```{r setup}
library(SpatialDecon)
```

Now let's load our example data and examine it:

```{r loaddata}
data("mini_geomx_dataset")
norm = mini_geomx_dataset$normalized
raw = mini_geomx_dataset$raw
annot = mini_geomx_dataset$annot
dim(raw)
head(annot)
raw[seq_len(5), seq_len(5)]

# better segment names:
colnames(norm) = colnames(raw) = rownames(annot) = 
  paste0(annot$ROI, annot$AOI.name)

```


The spatialdecon function takes 3 arguments of expression data:

1. The normalized data.
2. A matrix of expected background for all data points in the normalized 
data matrix.
3. Optionally, either a matrix of per-data-point weights, or the raw data, 
which is used to derive weights (low counts are less statistically stable, 
and this allows spatialdecon to down-weight them.) 


We estimate each data point's expected background from the negative control 
probes from its corresponding observation:

```{r estimateBG}
# use the NegProbe to estimate per-observation background
per.observation.mean.neg = norm["NegProbe", ]
# and define a background matrix in which each column (observation) is the
# appropriate value of per-observation background:
bg = sweep(norm * 0, 2, per.observation.mean.neg, "+")
dim(bg)

```

A note for background estimation: in studies with two probesets, the genes from 
each probeset will have distinct background values, and the above code should be
run separately for each probeset using its corresponding NegProbe value. 
Alternatively, the "derive_GeoMx_background" can do this automatically:

```{r derivebg}
bg2 = derive_GeoMx_background(norm = norm,
                             probepool = rep(1, nrow(norm)),
                             negnames = "NegProbe")
```


### Cell profile matrices

A "cell profile matrix" is a pre-defined matrix that specifies the expected 
expression profiles of each cell type in the experiment. 
The SpatialDecon library comes with one such matrix pre-loaded, the "SafeTME" 
matrix, designed for estimation of immune and stroma cells in the tumor 
microenvironment. 
(This matrix was designed to avoid genes commonly expressed by cancer cells; 
see the SpatialDecon manuscript for details.)

Let's take a glance at the safeTME matrix:

```{r showsafetme, fig.height=5, fig.width=10, fig.cap = "The safeTME cell profile matrix"}
data("safeTME")
data("safeTME.matches")

signif(safeTME[seq_len(3), seq_len(3)], 2)

heatmap(sweep(safeTME, 1, apply(safeTME, 1, max), "/"),
        labRow = NA, margins = c(10, 5))

```


For studies of other tissue types, we have provided a library of cell profile
matrices, available on Github and downloadable with the "download_profile_matrix" function. 

For a complete list of matrices, see [CellProfileLibrary GitHub Page](https://github.com/Nanostring-Biostats/CellProfileLibrary)

Below we download a matrix of cell profiles derived from scRNA-seq of a mouse 
spleen. 

```{r downloadmatrix, fig.height=7, fig.width=10, fig.cap = "The Mouse Spleen profile matrix", eval=T}
mousespleen <- download_profile_matrix(species = "Mouse",
                                       age_group = "Adult", 
                                       matrixname = "Spleen_MCA")
dim(mousespleen)

mousespleen[1:4,1:4]

head(cellGroups)

metadata

heatmap(sweep(mousespleen, 1, apply(mousespleen, 1, max), "/"),
        labRow = NA, margins = c(10, 5), cexCol = 0.7)

```


For studies where the provided cell profile matrices aren't sufficient or if a 
specific single cell dataset is wanted, we can make a custom profile matrix using 
the function create_profile_matrix(). 

This mini single cell dataset is a fraction of the data from Kinchen, J. et al. 
Structural Remodeling of the Human Colonic Mesenchyme in Inflammatory Bowel 
Disease. Cell 175, 372-386.e17 (2018).

```{r single cell data}
data("mini_singleCell_dataset")

mini_singleCell_dataset$mtx@Dim # genes x cells

as.matrix(mini_singleCell_dataset$mtx)[1:4,1:4]

head(mini_singleCell_dataset$annots)

table(mini_singleCell_dataset$annots$LabeledCellType)

```

**Pericyte cell** and **smooth muscle cell of colon** will be dropped from this 
matrix due to low cell count. The average expression across all cells of one type 
is returned so the more cells of one type, the better reflection of the true gene 
expression. The confidence in these averages can be changed using the minCellNum filter.

```{r creatematrix, fig.height=7, fig.width=10, fig.cap = "Custom profile matrix"}
custom_mtx <- create_profile_matrix(mtx = mini_singleCell_dataset$mtx,            # cell x gene count matrix
                                    cellAnnots = mini_singleCell_dataset$annots,  # cell annotations with cell type and cell name as columns 
                                    cellTypeCol = "LabeledCellType",  # column containing cell type
                                    cellNameCol = "CellID",           # column containing cell ID/name
                                    matrixName = "custom_mini_colon", # name of final profile matrix
                                    outDir = NULL,                    # path to desired output directory, set to NULL if matrix should not be written
                                    normalize = FALSE,                # Should data be normalized? 
                                    minCellNum = 5,                   # minimum number of cells of one type needed to create profile, exclusive
                                    minGenes = 10,                    # minimum number of genes expressed in a cell, exclusive
                                    scalingFactor = 5,                # what should all values be multiplied by for final matrix
                                    discardCellTypes = TRUE)          # should cell types be filtered for types like mitotic, doublet, low quality, unknown, etc.

head(custom_mtx)

heatmap(sweep(custom_mtx, 1, apply(custom_mtx, 1, max), "/"),
        labRow = NA, margins = c(10, 5), cexCol = 0.7)

```

Custom matrices can be created from all single cell data classes as long as a 
counts matrix and cell annotations can be passed to the function. Here is an 
example of creating a matrix using a Seurat object. 

```{r createSeuratmatrix}
library(SeuratObject)
library(Seurat)

data("mini_singleCell_dataset")

rownames(mini_singleCell_dataset$annots) <- mini_singleCell_dataset$annots$CellID

seuratObject <- CreateSeuratObject(counts = mini_singleCell_dataset$mtx, 
                                   meta.data = mini_singleCell_dataset$annots)
Idents(seuratObject) <- seuratObject$LabeledCellType

rm(mini_singleCell_dataset)

annots <- data.frame(cbind(cellType=as.character(Idents(seuratObject)), 
                           cellID=names(Idents(seuratObject))))

custom_mtx_seurat <- create_profile_matrix(mtx = Seurat::GetAssayData(object = seuratObject, 
                                                                      assay = "RNA", 
                                                                      layer = "counts"), 
                                           cellAnnots = annots, 
                                           cellTypeCol = "cellType", 
                                           cellNameCol = "cellID", 
                                           matrixName = "custom_mini_colon",
                                           outDir = NULL, 
                                           normalize = FALSE, 
                                           minCellNum = 5, 
                                           minGenes = 10)

head(custom_mtx_seurat)

paste("custom_mtx and custom_mtx_seurat are identical", all(custom_mtx == custom_mtx_seurat))
```

### Performing basic deconvolution with the spatialdecon function

Now our data is ready for deconvolution. 
First we'll show how to use spatialdecon under the basic settings, omitting 
optional bells and whistles. 


```{r runiss}
res = spatialdecon(norm = norm,
                   bg = bg,
                   X = safeTME,
                   align_genes = TRUE)
str(res)
```

We're most interested in "beta", the matrix of estimated cell abundances. 

```{r plotissres, fig.height = 5, fig.width = 8, fig.cap = "Cell abundance estimates"}
heatmap(res$beta, cexCol = 0.5, cexRow = 0.7, margins = c(10,7))
```


### Using the advanced settings of spatialdecon

spatialdecon has several abilities beyond basic deconvolution:

1. If given the nuclei counts for each region/observation, it returns results on
the scale of total cell counts.
2. If given the identities of pure tumor regions/observations, it infers a
handful of tumor-specific expression profiles and appends them to the cell
profile matrix. Doing this accounts for cancer cell-derived expression from any 
genes in the cell profile matrix, removing contaminating signal from cancer 
cells. 
3. If given raw count data, it derives per-data-point weights, using an error 
model derived for GeoMx data. 
4. If given a "cellmatches" argument, it sums multiple closely-related cell 
types into a single score. E.g. if the safeTME matrix is used with the 
cell-matching data object "safeTME.matches", it e.g. sums the "T.CD8.naive" and
"T.CD8.memory" scores into a single "CD8.T.cells" score. 

Let's take a look at an example cell matching object:
```{r showmatches}
str(safeTME.matches)
```


Now let's run spatialdecon:

```{r runisstils}
# vector identifying pure tumor segments:
annot$istumor = (annot$AOI.name == "Tumor")

# run spatialdecon with all the bells and whistles:
restils = spatialdecon(norm = norm,                     # normalized data
                       raw = raw,                       # raw data, used to down-weight low-count observations 
                       bg = bg,                         # expected background counts for every data point in norm
                       X = safeTME,                     # safeTME matrix, used by default
                       cellmerges = safeTME.matches,   # safeTME.matches object, used by default
                       cell_counts = annot$nuclei,      # nuclei counts, used to estimate total cells
                       is_pure_tumor = annot$istumor,   # identities of the Tumor segments/observations
                       n_tumor_clusters = 5)            # how many distinct tumor profiles to append to safeTME

str(restils)
```

There are quite a few readouts here. Let's review the important ones:

* beta: the cell abundance scores of the rolled-up/major cell types
* beta.granular: the cell abundance scores of the granular cell types, 
corresponding to the columns of the cell profile matrix
* yhat, resids: the fitted values and log2-scale residuals from the 
deconvolution fit. Can be used to measure each observation's goodness-of-fit, a 
possible QC metric. 
* cell.counts, cell.counts.granular: estimated numbers of each cell type, 
derived using the nuclei count input
* prop_of_nontumor: the beta matrix rescaled to give the proportions of 
non-tumor cells in each observation. 
* X: the cell profile matrix used, including newly-derived tumor-specific 
columns.

To illustrate the derivation of tumor profiles, let's look at the cell profile 
matrix output by spatialdecon:

```{r shownewX, fig.height=5, fig.width=8, fig.cap = "safeTME merged with newly-derived tumor profiles"}
heatmap(sweep(restils$X, 1, apply(restils$X, 1, max), "/"),
         labRow = NA, margins = c(10, 5))

```

Note the new tumor-specific columns. 

Finally, let's compare deconvolution results from basic vs. the advanced setting
with tumor profiles appended (just for a few cell types):

```{r compareresults, fig.height=6, fig.width=8, fig.cap = "Cell abundance estimates with and without modelling tumor profiles"}
par(mfrow = c(2, 3))
par(mar = c(5,7,2,1))
for (i in seq_len(6)) {
  cell = rownames(res$beta)[i]
  plot(res$beta[cell, ], restils$beta.granular[cell, ],
       xlab = paste0(cell, " score under basic setting"), 
       ylab = paste0(cell, " score when tumor\ncells are modelled"), 
       pch = 16,
       col = c(rgb(0,0,1,0.5), rgb(1,0,0,0.5))[1 + annot$istumor],
       xlim = range(c(res$beta[cell, ], restils$beta.granular[cell, ])),
       ylim = range(c(res$beta[cell, ], restils$beta.granular[cell, ])))
  abline(0,1)
  if (i == 1) {
    legend("topleft", pch = 16, col = c(rgb(0,0,1,0.5), rgb(1,0,0,0.5)),
           legend = c("microenv.", "tumor"))
  }
}

```

So the impact of modelling tumor is two-fold:

* Estimated immune cell content in tumor segments is driven down all the way, or
almost all the way, to 0.
* Estimated immune cell abundance in microenvironment segments in suppressed, as
part of the gene expression is attributed to cancer cells instead of immune 
cells. 


### Plotting deconvolution results

The SpatialDecon package contains two specialized plotting functions, and a 
default color palette for the safeTME matrix. 

The first function is "TIL_barplot", which is just a convenient way of drawing 
barplots of cell type abundance. 

```{r barplot, fig.width=9, fig.height=6, fig.cap="Barplots of TIL abundance"}
# For reference, show the TILs color data object used by the plotting functions 
# when safeTME has been used:
data("cellcols")
cellcols

# show just the TME segments, since that's where the immune cells are:
layout(mat = (matrix(c(1, 2), 1)), widths = c(7, 3))
TIL_barplot(restils$cell.counts$cell.counts, draw_legend = TRUE, 
            cex.names = 0.5)
# or the proportions of cells:
TIL_barplot(restils$prop_of_nontumor[, annot$AOI.name == "TME"], 
            draw_legend = TRUE, cex.names = 0.75)

```


The second function is "florets", used for plotting cell abundances atop some 
2-D projection. 
Here, we'll plot cell abundances atop the first 2 principal components of the 
data:

```{r florets, fig.width=8, fig.height=6, fig.cap = "TIL abundance plotted on PC space"}
# PCA of the normalized data:
pc = prcomp(t(log2(pmax(norm, 1))))$x[, c(1, 2)]

# run florets function:
par(mar = c(5,5,1,1))
layout(mat = (matrix(c(1, 2), 1)), widths = c(6, 2))
florets(x = pc[, 1], y = pc[, 2],
        b = restils$beta, cex = 2,
        xlab = "PC1", ylab = "PC2")
par(mar = c(0,0,0,0))
frame()
legend("center", fill = cellcols[rownames(restils$beta)], 
       legend = rownames(restils$beta), cex = 0.7)
```

So we can see that PC1 roughly tracks many vs. few immune cells, and PC2 tracks 
the relative abundance of lymphoid/myeloid populations.


### Other functions

The SpatialDecon library includes several helpful functions for further 
analysis/fine-tuning of deconvolution results. 

#### Combining cell types:

When two cell types are too similar, the estimation of their abundances becomes
unstable. However, their sum can still be estimated easily. 
The function "collapseCellTypes" takes a deconvolution results object and 
collapses any colsely-related cell types you tell it to:

```{r collapse, fig.width=5, fig.height=5, fig.cap="Cell abundance estimates with related cell types collapsed"}
matching = list()
matching$myeloid = c( "macrophages", "monocytes", "mDCs")
matching$T.NK = c("CD4.T.cells","CD8.T.cells", "Treg", "NK")
matching$B = c("B")
matching$mast = c("mast")
matching$neutrophils = c("neutrophils")
matching$stroma = c("endothelial.cells", "fibroblasts")


collapsed = collapseCellTypes(fit = restils, 
                              matching = matching)

heatmap(collapsed$beta, cexRow = 0.85, cexCol = 0.75)
```

#### Inferring an expression profile for a cell type omitted from the profile matrix

Sometimes a cell profile matrix will omit a cell type you know to be present in 
a tissue. 
If your data includes any regions that are purely this unmodelled cell type - 
e.g. because you've used the GeoMx platform's segmentation capability to 
specifically select them - then you can infer a profile for that cell type and 
merge it with your cell profile matrix. 
The algorithm clusters all the observations you designate as purely the 
unmodelled cell type, and collapses those clusters into as many profiles of that
cell type as you wish. For cancer cell, it may be appropriate to specify 10 or
more clusters; for highly regular healthy cells, one cluster may suffice.

(Note: this functionality can also be run within the spatialdecon function, as 
is demonstrated further above.)

```{r appendtumor, fig.width = 10, fig.height= 5, fig.cap = "safeTME merged with newly-derived tumor profiles"}
pure.tumor.ids = annot$AOI.name == "Tumor"
safeTME.with.tumor = mergeTumorIntoX(norm = norm, 
                                     bg = bg, 
                                     pure_tumor_ids = pure.tumor.ids, 
                                     X = safeTME, 
                                     K = 3) 

heatmap(sweep(safeTME.with.tumor, 1, apply(safeTME.with.tumor, 1, max), "/"),
        labRow = NA, margins = c(10, 5))

```


#### Reverse deconvolution

Once cell type abundance has been estimated, we can flip the deconvolution 
around, modelling the expression data as a function of cell abundances, and 
thereby deriving:

1. Estimated expression of each gene in each cell type. (Including for genes 
not present in your cell profile matrix)
2. Fitted expression values for each gene based on cell mixing.
3. Residuals of each gene: how does their expression compare to what cell mixing
would predict?
4. Two metrics of how well genes are predicted by/ redundant with cell mixing: 
correlation between observed and fitted expression, and residual SD. 

The function "reversedecon" runs this model.

```{r reverse, fig.height=6, fig.width=6, fig.cap="Residuals from reverseDecon"}
rdecon = reverseDecon(norm = norm,
                      beta = res$beta)
str(rdecon)

# look at the residuals:
heatmap(pmax(pmin(rdecon$resids, 2), -2))
```

```{r reverse2, fig.height=6, fig.width=6, fig.cap="Genes' dependency on cell mixing"}
# look at the two metrics of goodness-of-fit:
plot(rdecon$cors, rdecon$resid.sd, col = 0)
showgenes = c("CXCL14", "LYZ", "NKG7")
text(rdecon$cors[setdiff(names(rdecon$cors), showgenes)], 
     rdecon$resid.sd[setdiff(names(rdecon$cors), showgenes)], 
     setdiff(names(rdecon$cors), showgenes), cex = 0.5)
text(rdecon$cors[showgenes], rdecon$resid.sd[showgenes], 
     showgenes, cex = 0.75, col = 2)

```

From the above plot, we can see that genes like CXCL14 vary independently of 
cell mixing, genes like LYZ are correlated with cell mixing but still have 
variable expression, and genes like NKG7 serve as nothing but obtuse readouts 
of cell mixing. 


### Session Info

```{r sessioninfo}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{mean.resid.sd}
\alias{mean.resid.sd}
\title{Genes' biological variability in immune deconvolution from TCGA.}
\format{
A named vector giving SDs of 1179 genes.
}
\usage{
mean.resid.sd
}
\description{
Genes' biological SDs, as estimated from immune deconvolution from TCGA.
Used to weight genes in spatialdecon.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{mini_geomx_dataset}
\alias{mini_geomx_dataset}
\title{Small example GeoMx data}
\format{
A list with the following elements:
 \itemize{
 \item normalized: normalized data matrix
 \item raw: raw data matrix
 \item annot: AOI annotation data frame
 }
}
\usage{
mini_geomx_dataset
}
\description{
A miniature GeoMx dataset used by the spatialdecon examples.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TIL_barplot.R
\name{TIL_barplot}
\alias{TIL_barplot}
\title{Barplot of abundance estimates}
\usage{
TIL_barplot(mat, draw_legend = FALSE, main = "", col = NULL, ...)
}
\arguments{
\item{mat}{Matrix of cell proportions or abundances, in the same dimensions
output by spatialdecon
 (cells in rows, observations in columns). User is free to re-order
 columns/observations in
 whatever order is best for display.}

\item{draw_legend}{Logical. If TRUE, the function draws a legend in a new
plot frame.}

\item{main}{Title for barplot}

\item{col}{Vector of colors for cell types. Defaults to pre-set colors for
the safeTME cell types.}

\item{...}{Arguments passed to barplot()}
}
\value{
Draws a barplot.
}
\description{
Draw barplot of the "betas" from a decon fit
}
\examples{
data(mini_geomx_dataset)
data(safeTME)
# estimate background:
mini_geomx_dataset$bg <- derive_GeoMx_background(
  norm = mini_geomx_dataset$normalized,
  probepool = rep(1, nrow(mini_geomx_dataset$normalized)),
  negnames = "NegProbe"
)
# run basic decon:
res0 <- spatialdecon(
  norm = mini_geomx_dataset$normalized,
  bg = mini_geomx_dataset$bg,
  X = safeTME
)
# run barplot:
TIL_barplot(mat = res0$beta)
# run barplot and draw a color legend
TIL_barplot(mat = res0$beta, draw_legend = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runmergeTumorIntoXGeo.R
\name{runMergeTumorIntoX}
\alias{runMergeTumorIntoX}
\alias{runMergeTumorIntoX,NanoStringGeoMxSet-method}
\title{Run MergeTumorIntoX}
\usage{
runMergeTumorIntoX(object, ...)

\S4method{runMergeTumorIntoX}{NanoStringGeoMxSet}(object, X, K = 10, pure_tumor_ids = NULL, norm_elt = NULL)
}
\arguments{
\item{object}{An S4 object such as a GeoMxSet object}

\item{...}{Arguments passed to mergeTumorIntoX}

\item{X}{The training matrix}

\item{K}{the number of clusters to fit}

\item{pure_tumor_ids}{Vector identifying columns of norm that are pure tumor.
Can be indices, logicals or column names.}

\item{norm_elt}{norm data element in assayData}
}
\value{
updated X matrix with new columns, "tumor.1", "tumor.2", ...
}
\description{
Runs mergeTumorIntoX from an S4 object

A wrapper for applying mergeTumorIntoX to a NanostringGeomxSet object.
}
\examples{
library(GeomxTools)
datadir <- system.file("extdata", "DSP_NGS_Example_Data", package = "GeomxTools")
demoData <- readRDS(file.path(datadir, "/demoData.rds"))

demoData <- shiftCountsOne(demoData)
target_demoData <- aggregateCounts(demoData)

target_demoData <- normalize(target_demoData, "quant")

data(safeTME)
tumor.ids <- as.logical(sample(x = c("TRUE","FALSE"), size = 88, replace = TRUE))
safeTME.with.tumor <- runMergeTumorIntoX(object = target_demoData,
                                         X = safeTME,
                                         K = 3,
                                         pure_tumor_ids = tumor.ids,
                                         norm_elt = "exprs_norm")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{nsclc}
\alias{nsclc}
\title{Large example GeoMx data}
\format{
GeoMxSet Object
}
\usage{
nsclc
}
\description{
A GeoMx dataset with dense AOIs gridded over a NSCLC tumor. Each AOI is split into tumor and microenvironment segments.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spatialdecon.R
\name{spatialdecon}
\alias{spatialdecon}
\title{Mixed cell deconvolution of spatiall-resolved gene expression data}
\usage{
spatialdecon(
  norm,
  bg,
  X = NULL,
  raw = NULL,
  wts = NULL,
  resid_thresh = 3,
  lower_thresh = 0.5,
  align_genes = TRUE,
  is_pure_tumor = NULL,
  n_tumor_clusters = 10,
  cell_counts = NULL,
  cellmerges = NULL,
  maxit = 1000
)
}
\arguments{
\item{norm}{p-length expression vector or p * N expression matrix - the
actual (linear-scale) data}

\item{bg}{Same dimension as norm: the background expected at each data point.}

\item{X}{Cell profile matrix. If NULL, the safeTME matrix is used.}

\item{raw}{Optional for using an error model to weight the data points.
p-length expression vector or p * N expression matrix - the raw
(linear-scale) data}

\item{wts}{Optional, a matrix of weights.}

\item{resid_thresh}{A scalar, sets a threshold on how extreme individual data
points' values
can be (in log2 units) before getting flagged as outliers and set to NA.}

\item{lower_thresh}{A scalar. Before log2-scale residuals are calculated,
both observed and fitted
values get thresholded up to this value. Prevents log2-scale residuals from
becoming extreme in
points near zero.}

\item{align_genes}{Logical. If TRUE, then Y, X, bg, and wts are row-aligned
by shared genes.}

\item{is_pure_tumor}{A logical vector denoting whether each AOI consists of
pure tumor. If specified,
then the algorithm will derive a tumor expression profile and merge it with
the immune profiles matrix.}

\item{n_tumor_clusters}{Number of tumor-specific columns to merge into the
cell profile matrix.
 Has an impact only when is_pure_tumor argument is used to indicate pure
  tumor AOIs.
 Takes this many clusters from the pure-tumor AOI data and gets the average
 expression profile in each cluster.  Default 10.}

\item{cell_counts}{Number of cells estimated to be within each sample. If
provided alongside norm_factors,
 then the algorithm will additionally output cell abundance esimtates on the
 scale of cell counts.}

\item{cellmerges}{A list object holding the mapping from beta's cell names to
combined cell names. If left
NULL, then defaults to a mapping of granular immune cell definitions to
 broader categories.}

\item{maxit}{Maximum number of iterations. Default 1000.}
}
\value{
a list:
\itemize{
\item beta: matrix of cell abundance estimates, cells in rows and
observations in columns
\item sigmas: covariance matrices of each observation's beta estimates
\item p: matrix of p-values for H0: beta == 0
\item t: matrix of t-statistics for H0: beta == 0
\item se: matrix of standard errors of beta values
\item prop_of_all: rescaling of beta to sum to 1 in each observation
\item prop_of_nontumor: rescaling of beta to sum to 1 in each observation,
excluding tumor abundance estimates
\item cell.counts: beta rescaled to estimate cell numbers, based on
prop_of_all and nuclei count
\item beta.granular: cell abundances prior to combining closely-related
cell types
\item sigma.granular: sigmas prior to combining closely-related cell types
\item cell.counts.granular: cell.counts prior to combining closely-related
cell types
\item resids: a matrix of residuals from the model fit.
 (log2(pmax(y, lower_thresh)) - log2(pmax(xb, lower_thresh))).
\item X: the cell profile matrix used in the decon fit.
}
}
\description{
Runs the spatialdecon algorithm with added optional functionalities.
Workflow is:
\enumerate{
\item compute weights from raw data
\item Estimate a tumor profile and merge it into the cell profiles matrix
\item run deconvolution once
\item remove poorly-fit genes from first round of decon
\item re-run decon with cleaned-up gene set
\item combine closely-related cell types
\item compute p-values
\item rescale abundance estimates, to proportions of total, proportions of
 immune, cell counts
}
}
\examples{
data(mini_geomx_dataset)
data(safeTME)
data(safeTME.matches)
# estimate background:
mini_geomx_dataset$bg <- derive_GeoMx_background(
  norm = mini_geomx_dataset$normalized,
  probepool = rep(1, nrow(mini_geomx_dataset$normalized)),
  negnames = "NegProbe"
)
# run basic decon:
res0 <- spatialdecon(
  norm = mini_geomx_dataset$normalized,
  bg = mini_geomx_dataset$bg,
  X = safeTME
)
# run decon with bells and whistles:
res <- spatialdecon(
  norm = mini_geomx_dataset$normalized,
  bg = mini_geomx_dataset$bg,
  X = safeTME,
  cellmerges = safeTME.matches,
  cell_counts = mini_geomx_dataset$annot$nuclei,
  is_pure_tumor = mini_geomx_dataset$annot$AOI.name == "Tumor"
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runErrorModel.R
\name{runErrorModel}
\alias{runErrorModel}
\title{Apply error model to estimate technical SD from raw counts}
\usage{
runErrorModel(counts, platform = "general")
}
\arguments{
\item{counts}{vector or matrix of raw counts}

\item{platform}{String specifying which platform was used to create
"rawCounts". Default to "dsp", for digital spatial profiler/ aka GeoMx.
 Other options include "ncounter", "rsem", "quantile", and "st" for spatial transcriptomics/visium.}
}
\value{
a matrix of log2-scale SDs
}
\description{
Based on raw counts, uses past data to estimate each raw count's log-scale
SD from technical noise.
Specifies different error models for different platforms.
}
\examples{
library(GeomxTools)
datadir <- system.file("extdata", "DSP_NGS_Example_Data", package = "GeomxTools")
demoData <- readRDS(file.path(datadir, "/demoData.rds"))

demoData <- shiftCountsOne(demoData)
target_demoData <- aggregateCounts(demoData)

sd_from_noise <- runErrorModel(counts = exprs(target_demoData), platform = "dsp") 
wts <- 1 / sd_from_noise
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{mini_singleCell_dataset}
\alias{mini_singleCell_dataset}
\title{Mini human colon single cell dataset}
\format{
A list with the following elements:
 \itemize{
 \item mtx: sparse count matrix
 \item annots: cell type annotation data frame
 }
}
\usage{
mini_singleCell_dataset
}
\description{
Random 250 cells and most informative genes (CV > 10) between cell types from
Kinchen, J. et al. Structural Remodeling of the Human Colonic 
Mesenchyme in Inflammatory Bowel Disease. Cell 175, 372-386.e17 (2018).
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{cellcols}
\alias{cellcols}
\title{Default colors for the cell types in the safeTME matrix}
\format{
A named vector
}
\usage{
cellcols
}
\description{
A named vector of colors, giving colors for the cell types of the safeTME
 matrix.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/package.R
\docType{package}
\name{SpatialDecon-package}
\alias{SpatialDecon-package}
\title{SpatialDecon: A package for estimating mixed cell type abundance in the regions of 
spatially-resolved gene expression studies}
\description{
The SpatialDecon package estimates mixed cell type abundance in the regions
of spatially-resolved gene
expression studies, using the method of Danaher & Kim (2020), "Advances in
 mixed cell deconvolution enable
quantification of cell types in spatially-resolved gene expression data."
It is also appropriate to apply to bulk gene expression data.
}
\section{functions}{

Functions to help set up deconvolution:
\itemize{
 \item derive_GeoMx_background Estimates the background levels from GeoMx
 experiments
 \item collapseCellTypes reformats deconvolution results to merge
 closely-related cell types
 \item download_profile_matrix Downloads a cell profile matrix.
 \item safeTME: a data object, a matrix of immune cell profiles for use in
  tumor-immune deconvolution.
}
Deconvolution functions:
\itemize{
 \item spatialdecon runs the core deconvolution function
 \item reverseDecon runs a transposed/reverse deconvolution problem, fitting
 the data as a function of cell abundance estimates.
  Used to measure genes' dependency on cell mixing and to calculate gene
   residuals from cell mixing.
}
Plotting functions:
\itemize{
 \item florets Plot cell abundance on a specified x-y space, with each point
  a cockscomb plot showing the cell abundances of that region/sample.
 \item TIL_barplot Plot abundances of tumor infiltrating lymphocytes (TILs)
  estimated from the safeTME cell profile matrix
}
}

\examples{
data(mini_geomx_dataset)
data(safeTME)
data(safeTME.matches)
# estimate background:
mini_geomx_dataset$bg <- derive_GeoMx_background(
  norm = mini_geomx_dataset$normalized,
  probepool = rep(1, nrow(mini_geomx_dataset$normalized)),
  negnames = "NegProbe"
)
# run basic decon:
res0 <- spatialdecon(
  norm = mini_geomx_dataset$normalized,
  bg = mini_geomx_dataset$bg,
  X = safeTME
)
# run decon with bells and whistles:
res <- spatialdecon(
  norm = mini_geomx_dataset$normalized,
  bg = mini_geomx_dataset$bg,
  X = safeTME,
  cellmerges = safeTME.matches,
  cell_counts = mini_geomx_dataset$annot$nuclei,
  is_pure_tumor = mini_geomx_dataset$annot$AOI.name == "Tumor"
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/derive_GeoMx_background.R
\name{derive_GeoMx_background}
\alias{derive_GeoMx_background}
\title{Derive background at the scale of the normalized data for GeoMx data}
\usage{
derive_GeoMx_background(norm, probepool, negnames)
}
\arguments{
\item{norm}{Matrix of normalized data, genes in rows and segments in columns.
Must include negprobes, and must have rownames.}

\item{probepool}{Vector of probe pool names for each gene, aligned to the
rows of "norm".}

\item{negnames}{Names of all negProbes in the dataset. Must be at least one
neg.name within each probe pool.}
}
\value{
A matrix of expected background values, in the same scale and
 dimensions as the "norm" argument.
}
\description{
Estimates per-datapoint background levels from a GeoMx experiment.
In studies with two or more probe pools, different probes will have different
background levels. This function provides a convenient way to account for
this phenomenon.
}
\examples{
data(mini_geomx_dataset)
# estimate background:
mini_geomx_dataset$bg <- derive_GeoMx_background(
  norm = mini_geomx_dataset$normalized,
  probepool = rep(1, nrow(mini_geomx_dataset$normalized)),
  negnames = "NegProbe"
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runreverseDeconGeo.R
\name{runReverseDecon}
\alias{runReverseDecon}
\alias{runReverseDecon,NanoStringGeoMxSet-method}
\title{Run Reversedecon}
\usage{
runReverseDecon(object, ...)

\S4method{runReverseDecon}{NanoStringGeoMxSet}(object, norm_elt = NULL, beta, epsilon = NULL)
}
\arguments{
\item{object}{An S4 object such as a GeoMxSet object}

\item{...}{Arguments passed to reversedecon}

\item{norm_elt}{normalized data element in assayData.}

\item{beta}{Matrix of cell abundance estimates, with cells in columns and observations in rows.
Columns are aligned to "norm".}

\item{epsilon}{All y and yhat values are thresholded up to this point when performing decon.
Essentially says, "ignore variability in counts below this threshold."}
}
\value{
a valid GeoMx S4 object including the following items:
\itemize{
 \item in fData
 \itemize{
 \item coefs, a matrix of coefficients for genes * cells, where element i,j is interpreted as
              "every unit increase in cell score j is expected to increase expression of gene i by _".
 \item cors, a vector giving each gene's correlation between fitted and observed expression
 \item resid.sd, a vector of each gene's residual SD, a metric of how much variability genes
                 have independend of cell mixing.
 }
 \item in assayData
 \itemize{
   \item yhat, a matrix of fitted values, in the same dimension as norm
   \item resids, a matrix of log2-scale residuals from the reverse decon fit, in the same
               dimension as norm
 }
 
}
}
\description{
Runs reversedecon from an S4 object

A wrapper for applying reversedecon to a NanostringGeomxSet object.
}
\examples{
library(GeomxTools)
datadir <- system.file("extdata", "DSP_NGS_Example_Data", package = "GeomxTools")
demoData <- readRDS(file.path(datadir, "/demoData.rds"))

demoData <- shiftCountsOne(demoData)
target_demoData <- aggregateCounts(demoData)

target_demoData <- normalize(target_demoData, "quant")
                
# run basic decon:
res0 <- runspatialdecon(object = target_demoData,
                        norm_elt = "exprs_norm",
                        raw_elt = "exprs")

# run reverse decon:
target_demoData <- runReverseDecon(object = target_demoData,
                                   norm_elt = "exprs_norm",
                                   beta = pData(res0)$beta)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/download_profile_matrix.R
\name{download_profile_matrix}
\alias{download_profile_matrix}
\title{Download a cell profile matrix}
\usage{
download_profile_matrix(species, age_group, matrixname)
}
\arguments{
\item{species}{species of profile matrix}

\item{age_group}{age_group of profile matrix, if fetal mouse please add the 
developmental stage separated with /, i.e. Fetal/E14.5}

\item{matrixname}{name of profile matrix}
}
\value{
A cell profile matrix, suggested cell groups, and paper metadata
}
\description{
Download a cell profile matrix from the online library
}
\details{
Valid matrices can be found on the github site 
\url{https://github.com/Nanostring-Biostats/CellProfileLibrary/tree/master}
}
\examples{
download_profile_matrix(species = "Human", age_group = "Adult", matrixname = "Colon_HCA")
head(profile_matrix)
print(cellGroups)
print(metadata)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/collapseCellTypes.R
\name{collapseCellTypes}
\alias{collapseCellTypes}
\title{Collapse related cell types within a deconvolution result}
\usage{
collapseCellTypes(fit, matching)
}
\arguments{
\item{fit}{The object (a list) returned by the SpatialDecon algorithm}

\item{matching}{A list object holding the mapping from beta's cell names to
official cell names.
 See str(safeTME.matches) for an example.}
}
\value{
A reshaped deconvolution result object
}
\description{
Given the input of an SpatialDecon result output and a list of which cell
types to combine,
 returns a reshaped deconvolution result object with the specified cell
 types merged.
}
\examples{
data(mini_geomx_dataset)
data(safeTME)
data(safeTME.matches)
# estimate background:
mini_geomx_dataset$bg <- derive_GeoMx_background(
  norm = mini_geomx_dataset$normalized,
  probepool = rep(1, nrow(mini_geomx_dataset$normalized)),
  negnames = "NegProbe"
)
# run basic decon:
res0 <- spatialdecon(
  norm = mini_geomx_dataset$normalized,
  bg = mini_geomx_dataset$bg,
  X = safeTME
)
res1 <- collapseCellTypes(
    fit = res0,
    matching = safeTME.matches
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{safeTME}
\alias{safeTME}
\title{SafeTME matrix}
\format{
A matrix with 906 genes (rows) and 18 cell types (columns)
}
\usage{
safeTME
}
\description{
A matrix of expression profiles of 906 genes over 18 cell types.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runspatialdecon.R
\name{runspatialdecon}
\alias{runspatialdecon}
\alias{runspatialdecon,NanoStringGeoMxSet-method}
\alias{runspatialdecon,Seurat-method}
\title{Run spatialdecon}
\usage{
runspatialdecon(object, ...)

\S4method{runspatialdecon}{NanoStringGeoMxSet}(
  object,
  X = NULL,
  norm_elt = NULL,
  raw_elt = NULL,
  wts = NULL,
  resid_thresh = 3,
  lower_thresh = 0.5,
  align_genes = TRUE,
  is_pure_tumor = NULL,
  n_tumor_clusters = 10,
  cell_counts = NULL,
  cellmerges = NULL,
  maxit = 1000
)

\S4method{runspatialdecon}{Seurat}(
  object,
  X = NULL,
  bg = 0.1,
  wts = NULL,
  resid_thresh = 3,
  lower_thresh = 0.5,
  align_genes = TRUE,
  is_pure_tumor = NULL,
  n_tumor_clusters = 10,
  cell_counts = NULL,
  cellmerges = NULL,
  maxit = 1000
)
}
\arguments{
\item{object}{An S4 object such as a Seurat object that includse a "Spatial" element in the "assays" slot or a GeoMxSet object}

\item{...}{Arguments passed to spatialdecon}

\item{X}{Cell profile matrix. If NULL, the safeTME matrix is used.}

\item{norm_elt}{normalized data element in assayData in NanostringGeomxSet object}

\item{raw_elt}{raw data element in assayData in NanostringGeomxSet object}

\item{wts}{Optional, a matrix of weights.}

\item{resid_thresh}{A scalar, sets a threshold on how extreme individual data
points' values
can be (in log2 units) before getting flagged as outliers and set to NA.}

\item{lower_thresh}{A scalar. Before log2-scale residuals are calculated,
both observed and fitted
values get thresholded up to this value. Prevents log2-scale residuals from
becoming extreme in
points near zero.}

\item{align_genes}{Logical. If TRUE, then Y, X, bg, and wts are row-aligned
by shared genes.}

\item{is_pure_tumor}{A logical vector denoting whether each AOI consists of
pure tumor. If specified,
then the algorithm will derive a tumor expression profile and merge it with
the immune profiles matrix.}

\item{n_tumor_clusters}{Number of tumor-specific columns to merge into the
cell profile matrix.
 Has an impact only when is_pure_tumor argument is used to indicate pure
  tumor AOIs.
 Takes this many clusters from the pure-tumor AOI data and gets the average
 expression profile in each cluster.  Default 10.}

\item{cell_counts}{Number of cells estimated to be within each sample. If
provided alongside norm_factors,
 then the algorithm will additionally output cell abundance esimtates on the
 scale of cell counts.}

\item{cellmerges}{A list object holding the mapping from beta's cell names to
combined cell names. If left
NULL, then defaults to a mapping of granular immune cell definitions to
 broader categories.}

\item{maxit}{Maximum number of iterations. Default 1000.}

\item{bg}{Expected background counts. Either a scalar applied equally to 
all points in the count matrix, or a matrix with the same dimensions 
as the count matrix in GetAssayData(object, assay = "Spatial").
Recommended to use a small non-zero value, default of 0.1.}
}
\value{
decon results in list or in GeoMxSet object

For GeoMxSet object, if not given cellmerges and cell_counts, a valid GeoMx S4 object
including the following items
 \itemize{
   \item In pData
   \itemize{
      \item beta: matrix of cell abundance estimates, cells in rows and
               observations in columns
      \item p: matrix of p-values for H0: beta == 0
      \item t: matrix of t-statistics for H0: beta == 0
      \item se: matrix of standard errors of beta values
      \item prop_of_all: rescaling of beta to sum to 1 in each observation
      \item prop_of_nontumor: rescaling of beta to sum to 1 in each observation,
                           excluding tumor abundance estimates
      \item sigmas: covariance matrices of each observation's beta estimates
     }
   \item In assayData
   \itemize{
      \item yhat: a matrix of fitted values
      \item resids: a matrix of residuals from the model fit.
                 (log2(pmax(y, lower_thresh)) - log2(pmax(xb, lower_thresh))).
   }
   \item In experimentData
   \itemize{
      \item SpatialDeconMatrix: the cell profile matrix used in the decon fit.
   }
}


if given cellmerges, the valid GeoMx S4 object will additionally include
the following items
 \itemize{
   \item In pData
   \itemize{
      \item beta.granular: cell abundances prior to combining closely-related
                        cell types
      \item sigma.granular: sigmas prior to combining closely-related cell types
   }
}

if given cell_counts, the valid GeoMx S4 object will additionally include
 the following items
 \itemize{
 \item In pData
   \itemize{
      \item cell.counts: beta rescaled to estimate cell numbers, based on
                      prop_of_all and nuclei count
   }
}

if given both cellmerges and cell_counts, the valid GeoMx S4 object will
 additionally include the following items
 \itemize{
 \item In pData
   \itemize{
      \item cell.counts.granular: cell.counts prior to combining closely-related
                               cell types
   }
}

For Seurat Object, if not given cellmerges and cell_counts, a list
including the following items:
 \itemize{
   \item beta: matrix of cell abundance estimates, cells in rows and
               observations in columns
   \item p: matrix of p-values for H0: beta == 0
   \item t: matrix of t-statistics for H0: beta == 0
   \item se: matrix of standard errors of beta values
   \item prop_of_all: rescaling of beta to sum to 1 in each observation
   \item prop_of_nontumor: rescaling of beta to sum to 1 in each observation,
                           excluding tumor abundance estimates
   \item yhat: a matrix of fitted values
   \item resids: a matrix of residuals from the model fit.
                 (log2(pmax(y, lower_thresh)) - log2(pmax(xb, lower_thresh))).
   \item X: the cell profile matrix used in the decon fit.
   \item sigmas: covariance matrices of each observation's beta estimates
}


if given cellmerges, the list will additionally include
the following items
 \itemize{
   \item beta.granular: cell abundances prior to combining closely-related
                        cell types
   \item sigma.granular: sigmas prior to combining closely-related cell types
}

if given cell_counts, the list will additionally include
 the following items
 \itemize{
   \item cell.counts: beta rescaled to estimate cell numbers, based on
                      prop_of_all and nuclei count
}

if given both cellmerges and cell_counts, the list will
 additionally include the following items
 \itemize{
   \item cell.counts.granular: cell.counts prior to combining closely-related
                               cell types
}
}
\description{
Runs spatialdecon from an S4 object

A wrapper for applying spatialdecon to a NanostringGeomxSet object.

A wrapper for applying spatialdecon to the Spatial data element in a Seurat object. 
Unlike spatialdecon, which expects a normalized data matrix, this function operates 
on raw counts. Scaling for total cells
}
\examples{

## GeoMxSet Object ##
library(GeomxTools)
datadir <- system.file("extdata", "DSP_NGS_Example_Data", package = "GeomxTools")
demoData <- readRDS(file.path(datadir, "/demoData.rds"))

demoData <- shiftCountsOne(demoData)
target_demoData <- aggregateCounts(demoData)

target_demoData <- normalize(target_demoData, "quant")

demoData <- runspatialdecon(object = target_demoData, 
                            norm_elt = "exprs_norm",
                            raw_elt = "exprs")

## Seurat Object ##
# get dataset
con <- gzcon(url("https://github.com/almaan/her2st/raw/master/data/ST-cnts/G1.tsv.gz"))
txt <- readLines(con)
temp <- read.table(textConnection(txt), sep = "\t", header = TRUE, row.names = 1)
# parse data
raw = t(as.matrix(temp))
norm = sweep(raw, 2, colSums(raw), "/") * mean(colSums(raw))
x = as.numeric(substr(rownames(temp), 1, unlist(gregexpr("x", rownames(temp))) - 1))
y = -as.numeric(substr(rownames(temp), 
                 unlist(gregexpr("x", rownames(temp))) + 1, nchar(rownames(temp))))
# put into a seurat object:
andersson_g1 = SeuratObject::CreateSeuratObject(counts = raw, assay="Spatial")
andersson_g1@meta.data$x = x
andersson_g1@meta.data$y = y

res <- runspatialdecon(andersson_g1)
str(res)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reverseDecon.R
\name{reverseDecon}
\alias{reverseDecon}
\title{Reverse deconvolution}
\usage{
reverseDecon(norm, beta, epsilon = NULL)
}
\arguments{
\item{norm}{Matrix of normalized data, with genes in rows and observations in columns}

\item{beta}{Matrix of cell abundance estimates, with cells in rows and observations in columns.
Columns are aligned to "norm".}

\item{epsilon}{All y and yhat values are thresholded up to this point when performing decon.
Essentially says, "ignore variability in counts below this threshold."}
}
\value{
A list:
\itemize{
\item coefs, a matrix of coefficients for genes * cells, where element i,j is interpreted as
"every unit increase in cell score j is expected to increase expression of gene i by _".
\item yhat, a matrix of fitted values, in the same dimension as norm
\item resids, a matrix of log2-scale residuals from the reverse decon fit, in the same
 dimension as norm
\item cors, a vector giving each gene's correlation between fitted and observed expression
\item resid.sd, a vector of each gene's residual SD, a metric of how much variability genes
 have independend of cell mixing.
}
}
\description{
Performs "reverse deconvolution", modelling each gene expression's ~ cell scores.
Returns a matrix of "fitted" expression values, a matrix of residuals, a matrix of
reverse decon coefficients for genes * cells.
}
\examples{
data(mini_geomx_dataset)
data(safeTME)
# estimate background:
mini_geomx_dataset$bg <- derive_GeoMx_background(
  norm = mini_geomx_dataset$normalized,
  probepool = rep(1, nrow(mini_geomx_dataset$normalized)),
  negnames = "NegProbe"
)
# run basic decon:
res0 <- spatialdecon(
  norm = mini_geomx_dataset$normalized,
  bg = mini_geomx_dataset$bg,
  X = safeTME
)
# run reverse decon:
rdecon <- reverseDecon(
  norm = mini_geomx_dataset$norm,
  beta = res0$beta
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/florets.R
\name{florets}
\alias{florets}
\title{Draw coxcomb plots as points in a graphics window}
\usage{
florets(
  x,
  y,
  b,
  col = NULL,
  legendwindow = FALSE,
  rescale.by.sqrt = TRUE,
  border = NA,
  add = FALSE,
  cex = 1,
  bty = "n",
  xaxt = "n",
  yaxt = "n",
  xlab = "",
  ylab = "",
  ...
)
}
\arguments{
\item{x}{Vector of x coordinates}

\item{y}{Vector of y coordinates}

\item{b}{matrix or cell abundances, with columns aligned with the elements
of x and y}

\item{col}{vector of colors, aligned to the rows of b.}

\item{legendwindow}{Logical. If TRUE, the function draws a color legend in a
new window}

\item{rescale.by.sqrt}{Logical, for whether to rescale b by its square root
to make value proportional to
 shape area, not shape length.}

\item{border}{Color of pie segment border, defauls to NA/none}

\item{add}{Logical. If TRUE, the function draws florets atop an existing
graphics device (TRUE) or call a new device (FALSE).}

\item{cex}{Floret size. Florets are scaled relative to the range of x and y;
this further scales up or down.}

\item{bty}{bty argument passed to plot()}

\item{xaxt}{xaxt argument passed to plot()}

\item{yaxt}{yaxt argument passed to plot()}

\item{xlab}{xlab, defaults to ""}

\item{ylab}{ylab, defaults to ""}

\item{...}{additional arguments passed to plot()}
}
\value{
Draws a coxcomb plot, returns no data.
}
\description{
Draws a scatterplot where each point is a circular barplot, intended to show
decon results
}
\examples{
data(mini_geomx_dataset)
data(safeTME)
# estimate background:
mini_geomx_dataset$bg <- derive_GeoMx_background(
  norm = mini_geomx_dataset$normalized,
  probepool = rep(1, nrow(mini_geomx_dataset$normalized)),
  negnames = "NegProbe"
)
# run basic decon:
res0 <- spatialdecon(
  norm = mini_geomx_dataset$normalized,
  bg = mini_geomx_dataset$bg,
  X = safeTME
)
# draw florets:
florets(
  x = mini_geomx_dataset$annot$x,
  y = mini_geomx_dataset$annot$y,
  b = res0$beta, cex = 2
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/create_profile_matrix.R
\name{create_profile_matrix}
\alias{create_profile_matrix}
\title{Create Custom Cell Profile Matrix}
\usage{
create_profile_matrix(
  mtx,
  cellAnnots,
  cellTypeCol,
  cellNameCol,
  matrixName = "Custom",
  outDir = "./",
  geneList = NULL,
  normalize = FALSE,
  scalingFactor = 5,
  minCellNum = 15,
  minGenes = 100,
  discardCellTypes = FALSE
)
}
\arguments{
\item{mtx}{gene x cell count matrix}

\item{cellAnnots}{cell annotations with cell type and cell name as columns}

\item{cellTypeCol}{column containing cell type}

\item{cellNameCol}{column containing cell ID/name}

\item{matrixName}{name of final profile matrix}

\item{outDir}{path to desired output directory, set to NULL if matrix should not be written}

\item{geneList}{gene list to filter profile matrix to}

\item{normalize}{Should data be normalized? (TRUE/FALSE) if TRUE data will be normalize using total gene count}

\item{scalingFactor}{what should all values be multiplied by for final matrix, set to 1 if no scaling is wanted}

\item{minCellNum}{minimum number of cells of one type needed to create profile, exclusive}

\item{minGenes}{minimum number of genes expressed in a cell, exclusive}

\item{discardCellTypes}{should cell types be filtered for types like mitotic, doublet, low quality, unknown, etc.}
}
\value{
A custom cell profile matrix genes (rows) by cell types (columns), matrix gets written to disk and outDir
}
\description{
Create custom cell profile matrix using single cell data. The average gene expression for each cell type is returned.
}
\examples{
cellNames <- paste0("Cell", seq_len(1500))
geneNames <- paste0("Gene", seq_len(1500))
mtx <- matrix(data=sample(size = length(cellNames)*length(geneNames),
                          replace = TRUE,
                          x = c(0,seq_len(100)), 
                          prob = c(0.6784, rep(0.0075, 15), rep(0.005, 25),
                                   rep(0.002, 25), rep(0.001, 35))), 
                          ncol = length(cellNames), nrow = length(geneNames), 
                          dimnames = list(geneNames, cellNames))
cellAnnots <- as.data.frame(cbind(CellID=cellNames, 
                                  cellType=sample(size = length(cellNames), 
                                                  replace = TRUE,
                                                  x = c("A", "B", "C", "D"),
                                                  prob = c(0.1, 0.4, 0.3, 0.2))))
table(cellAnnots$cellType)
profile_matrix <- create_profile_matrix(mtx = mtx,    
                                        cellAnnots = cellAnnots, 
                                        cellTypeCol = "cellType",
                                        cellNameCol = "CellID",
                                        minGenes = 10,
                                        scalingFactor = 1)
head(profile_matrix)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{safeTME.matches}
\alias{safeTME.matches}
\title{Mapping from granularly-defined cell populations to broaded cell populations}
\format{
A list. Each element of the list contains the granular cell types
 that roll up
 to a single coarse cell type.
}
\usage{
safeTME.matches
}
\description{
Mapping from granularly-defined cell populations to broaded cell populations,
 for use by the convertCellTypes function.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runCollapseCellTypes.R
\name{runCollapseCellTypes}
\alias{runCollapseCellTypes}
\alias{runCollapseCellTypes,NanoStringGeoMxSet-method}
\title{Run collapseCellTypes}
\usage{
runCollapseCellTypes(object, ...)

\S4method{runCollapseCellTypes}{NanoStringGeoMxSet}(object, matching = NULL)
}
\arguments{
\item{object}{An S4 object such as a GeoMxSet object}

\item{...}{Arguments passed to collapseCellTypes}

\item{matching}{A list object holding the mapping from beta's cell names to
official cell names.
 See str(safeTME.matches) for an example.}
}
\value{
A reshaped deconvolution result object
}
\description{
Runs collapseCellTypes from an S4 object

Given the input of an SpatialDecon result output and a list of which cell
types to combine,
 returns a reshaped deconvolution result object with the specified cell
 types merged.
}
\examples{
library(GeomxTools)
datadir <- system.file("extdata", "DSP_NGS_Example_Data", package = "GeomxTools")
demoData <- readRDS(file.path(datadir, "/demoData.rds"))

demoData <- shiftCountsOne(demoData)
target_demoData <- aggregateCounts(demoData)

target_demoData <- normalize(target_demoData, "quant")
                
# run basic decon:
res0 <- runspatialdecon(object = target_demoData,
                        norm_elt = "exprs_norm",
                        raw_elt = "exprs")

# run reverse decon:
target_demoData <- runReverseDecon(object = target_demoData,
                                   norm_elt = "exprs_norm",
                                   beta = pData(res0)$beta)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mergeTumorIntoX.R
\name{mergeTumorIntoX}
\alias{mergeTumorIntoX}
\title{Estimate a tumor-specific profile and merge it with the pre-specified cell
 profile matrix (X)}
\usage{
mergeTumorIntoX(norm, bg, pure_tumor_ids, X, K = 10)
}
\arguments{
\item{norm}{matrix of normalized data}

\item{bg}{matrix of expected background, on the scale of norm.}

\item{pure_tumor_ids}{Vector identifying columns of norm that are pure tumor.
Can be indices, logicals or column names.}

\item{X}{The training matrix}

\item{K}{the number of clusters to fit}
}
\value{
an updated X matrix with new columns, "tumor.1", "tumor.2", ...
}
\description{
Given the input of "tumor-only" AOI's, estimates an collection of
 tumor-specific
expression profiles and merges them with the immune cell expression
training matrix.
The process:
\enumerate{
\item log2/normalized data from tumor-only AOIs is clustered with hclust,
and cutree() is used to define clusters.
\item 2. Each cluster's geomean profile is merged into the immune cell
profile matrix.
}
}
\examples{
data(mini_geomx_dataset)
data(safeTME)
mini_geomx_dataset$bg <- derive_GeoMx_background(
  norm = mini_geomx_dataset$normalized,
  probepool = rep(1, nrow(mini_geomx_dataset$normalized)),
  negnames = "NegProbe"
)
safeTME.with.tumor <- mergeTumorIntoX(
  norm = mini_geomx_dataset$norm,
  bg = mini_geomx_dataset$bg,
  pure_tumor_ids = mini_geomx_dataset$annot$AOI.name == "Tumor",
  X = safeTME,
  K = 3
)
}
